/*
 * Created on 2004-11-13
 */
package crn;

import java.util.ArrayList;
import java.util.Iterator;

/**
 * The Information Entity.
 * @author Piotrek
 */
public class IE
{
	/**
	 * The list of all cases this IE reference to.
	 */
	private ArrayList caseReferences = new ArrayList();
	
	/**
	 * The name of the value held in this IE instance. 
	 * @author Piotrek
	 */
	public String name;
	
	/**
	 * The value of this IE.
	 * @author Piotrek
	 */
	public String value;
		
	/**
	 * Constructor.
	 * @author Piotrek
	 * @param name The name of the value held in this IE instance.
	 * @param value The value of this IE.
	 */
	public IE(String name, String value)
	{
		if (name == null)
			throw new NullPointerException("Name must not be null in IE.IE().");
		
		if (value == null)
			throw new NullPointerException("Value must not be null in IE.IE().");
		
		this.name = name;
		this.value = value;
	}
	
	/**
	 * Copy constructor.
	 * @author Piotrek
	 * @param other The other IE to be copied.
	 */
	public IE(IE other)
	{
		this.name = other.name;
		this.value = other.value;
	}
	
	/**
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object object)
	{
		if (object == null)
			throw new NullPointerException("Object must not be null in IE.equals().");
		
		if (!IE.class.isInstance(object))
			return false;
		
		IE other = (IE)object;
		
		if (name.compareTo(other.name) == 0 && value.compareTo(other.value) == 0)
			return true;
		
		return false;
	}
	
	/**
	 * Checks whether this and the given IEs are equal.
	 * @author Piotrek
	 * @param otherIE
	 * @return A boolean value indicating whether this and the given IEs are equal.
	 */
	public boolean IsEqual(IE otherIE)
	{
		if (otherIE == null)
			throw new NullPointerException("Other IE must not be null in IE.equals().");
		
		return this.equals(otherIE);
	}
	
	/**
	 * Gets the interator for the cases this IE reference to.
	 * @author Piotrek
	 * @return The requested iterator.
	 */
	public Iterator GetCaseReferencesInterator()
	{
		return caseReferences.iterator();
	}
	
	/**
	 * Adds a case reference to this IE.
	 * @author Piotrek
	 * @param travelCase The case to add reference to.
	 */
	public void AddCaseReference(Case travelCase)
	{
		if (travelCase == null)
			throw new NullPointerException("Travel case must not be null if you wish to set a reference to it in this IE");
		
		caseReferences.add(travelCase);
	}
	
	/**
	 * Gets the count of the case references of this IE.
	 * @author Piotrek
	 * @return The count of the case references of this IE.
	 */
	public int GetCaseReferencesCount()
	{
		return caseReferences.size();
	}
	
	/**
	 * Removes the given case form from the references list of this IE.
	 * @author Piotrek
	 * @param travelCase The case to remove reference to.
	 * @throws IllegalArgumentException Thrown if the given case is not on the refernces list of this IE.
	 */
	public void RemoveCaseReference(Case travelCase) throws IllegalArgumentException
	{
		if (!caseReferences.contains(travelCase))
			throw new IllegalArgumentException("There is no such a Case in this IE's references list.");
		
		caseReferences.remove(travelCase);
	}
	
	/**
	 * Check if there is a given case on the list of references of this IE.
	 * @author Piotrek
	 * @param travelCase
	 * @return Returns a boolean value indicating whether there is a given case on the list of references of this IE.
	 */
	public boolean ContainsCase(Case travelCase)
	{
		return caseReferences.contains(travelCase);
	}
}
