package crn;

import java.util.Hashtable;

/**
 * Provides a way of specific IEs comparison.
 * @author Piotrek
 */
public final class IESpecificComparisonManager
{	
	/**
	 * A hashtable mapping the IE names to the specific comparators.
	 * @author Piotrek
	 */
	private Hashtable specificComparators = new Hashtable();
	
	/**
	 * Checks if there is a specific comparator for the given IE type.
	 * @author Piotrek
	 * @param ieName The name of the IE type to look for.
	 * @return A boolean value indicating whether there is a specific comaprator for the given IE type.
	 */
	public boolean HasSpecificComparator(String ieName)
	{
		if (ieName == null)
			throw new NullPointerException("IE name must not be null in IESpecificComparisonManager.HasSpecificComparator().");
		
		return specificComparators.get(ieName) != null;
	}
	
	/**
	 * Adds a specific comparator for the given IE type.
	 * @author Piotrek
	 * @param ieName The name of the IE for which to add the specific comparator.
	 * @param ieComparator The comparator for the given IE type.
	 */
	public void AddSpecificComparator(String ieName, IEComparator ieComparator)
	{		
		specificComparators.put(ieName, ieComparator);
	}
	
	/**
	 * Removes a specific comparator for the given IE type.
	 * @author Piotrek
	 * @param ieName The name of the IE for which to remove the specific comparator.
	 */
	public void RemoveSpecificComparator(String ieName)
	{
		specificComparators.remove(ieName);
	}
	
	/**
	 * Compares the given IEs.
	 * @author Piotrek
	 * @param ie1 The first of the IEs to be compared.
	 * @param ie2 The second of the IEs to be compared.
	 * @return The similarity of the given IEs.
	 */
	public float CompareIEs(IE ie1, IE ie2)
	{
		if (ie1 == null || ie2 == null)
			throw new NullPointerException("Both IEs must not be null in IESpecificComparisonManager.CompareIEs().");

		if (ie1.name.compareTo(ie2.name) != 0)
			throw new IllegalArgumentException("IE1 and IE2 are of different types in IESpecificComparisonManager.CompareIEs().");
		
		IEComparator comparator = (IEComparator)specificComparators.get(ie1.name);
		
		if (comparator == null)
			throw new RuntimeException("There is no specific comparator for the type of the given IEs in IESpecificComparisonManager.CompareIEs().");
		
		float comparison = comparator.CompareIEs(ie1, ie2);
		
		if (comparison < 0.0f || comparison > 1.0f)
			throw new RuntimeException("The comparator has returned an invalid value in IESpecificComparisonManager.CompareIEs()!"); 
		
		return comparison;
	}
}
