/*
 * Created on 2005-09-01
 */
package crn;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.ParseException;
import javax.swing.JOptionPane;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;
import crn.comparators.ChangesCountsComparator;
import crn.comparators.DepartureHoursOfDayComparator;
import crn.comparators.LocationsComparator;
import crn.comparators.RanksComparator;
import crn.comparators.TotalDurationsComparator;

/**
 * A class used to parse an XML file containing settings used by the comparison functions.
 * @author Piotrek
 */
public final class OtherSettingsFileHandler extends DefaultHandler
{
	/**
	 * Constructor.
	 * @author Piotrek
	 */
	public OtherSettingsFileHandler()
	{
		super();
	}
	
	/**
	 * Stores the `other settings' to the given file.
	 * @author Piotrek
	 * @param fileName The name of file to store the `other seetings' to.
	 * @throws IOException Thrown in case of IO problems.
	 */
	public static void StoreToFile(String fileName) throws IOException
	{
		FileWriter fileWriter = new FileWriter(fileName);
		
		fileWriter.write("<?xml version=\"1.0\"?>\r\n");
		
		fileWriter.write("<OtherSettings>\r\n");
		
		fileWriter.write("   <MaxSimilarChangesCountsDifference value=\"" + ChangesCountsComparator.MAX_SIMILAR_CHANGES_COUNTS_DIFFERENCE + "\"/>\r\n");
		
		fileWriter.write("   <MaxSimilarDepartureHoursOfDayDifference value=\"" + DepartureHoursOfDayComparator.MAX_SIMILAR_HOURS_DIFFERENCE + "\"/>\r\n");
		
		fileWriter.write("   <MaxSimilarLocationsDistance value=\"" + LocationsComparator.MAX_SIMILAR_LOCATIONS_DISTANCE + "\"/>\r\n");
		
		fileWriter.write("   <MaxSimilarRanksDifference value=\"" + RanksComparator.MAX_SIMILAR_RANKS_DIFFERENCE + "\"/>\r\n");
		
		fileWriter.write("   <MaxSimilarTotalDurationsDifference value=\"" + TotalDurationsComparator.MAX_SIMILAR_DURATIONS_DIFFERENCE + "\"/>\r\n");
		
		fileWriter.write("</OtherSettings>\r\n");
		
		fileWriter.flush();
	}
	
	/**
	 * Loads the `other settings' from the given file.
	 * @author Piotrek
	 * @param fileName The name of file to load the data from.
	 */
	public static void LoadFromFile(String fileName)
	{		
		OtherSettingsFileHandler parser = new OtherSettingsFileHandler();
		
		try
		{
			parser.Parse(fileName);
			
			// applying parsed values
			ChangesCountsComparator.MAX_SIMILAR_CHANGES_COUNTS_DIFFERENCE = parser.maxSimilarChangesCountsDifference;
			DepartureHoursOfDayComparator.MAX_SIMILAR_HOURS_DIFFERENCE = parser.maxSimilarDepartureHoursOfDayDifference;
			LocationsComparator.MAX_SIMILAR_LOCATIONS_DISTANCE = parser.maxSimilarLocationsDistance;
			RanksComparator.MAX_SIMILAR_RANKS_DIFFERENCE = parser.maxSimilarRanksDifference;
			TotalDurationsComparator.MAX_SIMILAR_DURATIONS_DIFFERENCE = parser.maxSimilarTotalDurationsDifference;
		}
		
		catch (ParseException xcp)
		{
			JOptionPane.showMessageDialog(null, "File: " + fileName + "\nError: " + xcp.getMessage() + "\nOffset: " + xcp.getErrorOffset(), "Error", JOptionPane.ERROR_MESSAGE);
		}
		
		catch (SAXParseException xcp)
		{
			JOptionPane.showMessageDialog(null, "File: " + fileName + "\nError: " + xcp.getMessage() + "\nLine number: " + xcp.getLineNumber(), "Error", JOptionPane.ERROR_MESSAGE);
		}
				
		catch (Exception xcp)
		{
			JOptionPane.showMessageDialog(null, "File: " + fileName + "\nError: " + xcp.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
		}
	}
	
	/**
	 * Parses the given file.
	 * @author Piotrek
	 * @param fileName The name of the file to be parsed.
	 * @throws Exception Thrown in case of error.
	 * @throws IOException In case of any IO error.
	 * @throws SAXException In case of some parsing error.
	 */
	public void Parse(String fileName) throws Exception, IOException, SAXException
	{
		if (fileName == null)
			throw new NullPointerException("File name must not be null.");
				
		SAXParser saxParser;
		
		try
		{
			saxParser = CreateSAXParser();
		}
		
		catch (Throwable xcp)
		{
			throw new Exception("Failed to create a parser.");
		}
		
		saxParser.parse(new File(fileName), this);
	}
	
	/**
	 * Creates an instance of the SAXParser.
	 * @author Piotrek
	 * @return The requested parser.
	 * @throws ParserConfigurationException If it was imposible to create a parser satisfing the current configuration.
	 * @throws SAXException
	 */
	private static SAXParser CreateSAXParser() throws ParserConfigurationException, SAXException
	{
	 	SAXParserFactory factory = SAXParserFactory.newInstance();
		
	 	SAXParser saxParser = factory.newSAXParser();
	 	
	 	return saxParser;
	}
		
	/**
	 * @author Piotrek
	 * @see org.xml.sax.ContentHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
	 */
	public void startElement(String uriNamespace, String simpleName, String qualifiedName, Attributes attributes) throws SAXException
	{
		String name = simpleName;
		
		if (name.equals("")) 
			name = qualifiedName; // not namespace aware
		
		if (name.equals("OtherSettings"))
			fInOtherSettings = true;
		
		if (name.equals("MaxSimilarChangesCountsDifference"))
		{
			if (!fInOtherSettings)
				throw new SAXParseException("MaxSimilarChangesCountsDifference tag outside the OtherSettings tag.", locator);
			
			if (attributes == null || attributes.getLength() != 1)
				throw new SAXParseException("MaxSimilarChangesCountsDifference tag has invalid number of attributes.", locator);
						
			if (attributes.getValue("value") == null)
				throw new SAXParseException("MaxSimilarChangesCountsDifference tag has no value attribute.", locator);
									
			try
			{
				maxSimilarChangesCountsDifference = Integer.parseInt(attributes.getValue("value"));
			}
			
			catch (NumberFormatException xcp)
			{
				throw new SAXParseException("Max similar changes counts difference value is invalid.", locator);
			}
			
			if (maxSimilarChangesCountsDifference < 0)
				throw new SAXParseException("Max similar changes counts difference value is invalid.", locator);
		}
		
		if (name.equals("MaxSimilarDepartureHoursOfDayDifference"))
		{
			if (!fInOtherSettings)
				throw new SAXParseException("MaxSimilarDepartureHoursOfDayDifference tag outside the OtherSettings tag.", locator);
			
			if (attributes == null || attributes.getLength() != 1)
				throw new SAXParseException("MaxSimilarDepartureHoursOfDayDifference tag has invalid number of attributes.", locator);
						
			if (attributes.getValue("value") == null)
				throw new SAXParseException("MaxSimilarDepartureHoursOfDayDifference tag has no value attribute.", locator);
									
			try
			{
				maxSimilarDepartureHoursOfDayDifference = Integer.parseInt(attributes.getValue("value"));
			}
			
			catch (NumberFormatException xcp)
			{
				throw new SAXParseException("Max similar departure hours of day difference value is invalid.", locator);
			}
			
			if (maxSimilarDepartureHoursOfDayDifference < 0)
				throw new SAXParseException("Max similar departure hours of day difference value is invalid.", locator);
		}
		
		if (name.equals("MaxSimilarLocationsDistance"))
		{
			if (!fInOtherSettings)
				throw new SAXParseException("MaxSimilarLocationsDistance tag outside the OtherSettings tag.", locator);
			
			if (attributes == null || attributes.getLength() != 1)
				throw new SAXParseException("MaxSimilarLocationsDistance tag has invalid number of attributes.", locator);
						
			if (attributes.getValue("value") == null)
				throw new SAXParseException("MaxSimilarLocationsDistance tag has no value attribute.", locator);
									
			try
			{
				maxSimilarLocationsDistance = Double.parseDouble(attributes.getValue("value"));
			}
			
			catch (NumberFormatException xcp)
			{
				String valueString = attributes.getValue("value");
				
				if (valueString.indexOf(".") != -1)
					valueString.replaceAll(".", ",");
				else
					valueString.replaceAll(",", ".");
				
				try
				{
					maxSimilarLocationsDistance = Double.parseDouble(attributes.getValue("value"));
				}
				
				catch (NumberFormatException xcp2)
				{
					throw new SAXParseException("Max similar locations distance value is invalid.", locator);
				}
			}
			
			if (maxSimilarLocationsDistance < 0)
				throw new SAXParseException("Max similar locations distance value is invalid.", locator);
		}
		
		if (name.equals("MaxSimilarRanksDifference"))
		{
			if (!fInOtherSettings)
				throw new SAXParseException("MaxSimilarRanksDifference tag outside the OtherSettings tag.", locator);
			
			if (attributes == null || attributes.getLength() != 1)
				throw new SAXParseException("MaxSimilarRanksDifference tag has invalid number of attributes.", locator);
						
			if (attributes.getValue("value") == null)
				throw new SAXParseException("MaxSimilarRanksDifference tag has no value attribute.", locator);
									
			try
			{
				maxSimilarRanksDifference = Integer.parseInt(attributes.getValue("value"));
			}
			
			catch (NumberFormatException xcp)
			{
				throw new SAXParseException("Max similar ranks difference value is invalid.", locator);
			}
			
			if (maxSimilarRanksDifference < 0)
				throw new SAXParseException("Max similar ranks difference value is invalid.", locator);
		}
		
		if (name.equals("MaxSimilarTotalDurationsDifference"))
		{
			if (!fInOtherSettings)
				throw new SAXParseException("MaxSimilarTotalDurationsDifference tag outside the OtherSettings tag.", locator);
			
			if (attributes == null || attributes.getLength() != 1)
				throw new SAXParseException("MaxSimilarTotalDurationsDifference tag has invalid number of attributes.", locator);
						
			if (attributes.getValue("value") == null)
				throw new SAXParseException("MaxSimilarTotalDurationsDifference tag has no value attribute.", locator);
									
			try
			{
				maxSimilarTotalDurationsDifference = Integer.parseInt(attributes.getValue("value"));
			}
			
			catch (NumberFormatException xcp)
			{
				throw new SAXParseException("Max similar total durations difference value is invalid.", locator);
			}
			
			if (maxSimilarTotalDurationsDifference < 0)
				throw new SAXParseException("Max similar total durations difference value is invalid.", locator);
		}
	}
	
	/**
	 * @author Piotrek
	 * @see org.xml.sax.ContentHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
	 */
	public void endElement(String uriNamespace, String qualifiedName, String simpleName)
	{
		String name = simpleName;
		
		if (name.equals("")) 
			name = qualifiedName; // not namespace aware
		
		if (name.equals("OtherSettings"))
		{
			fInOtherSettings = false;
		}
	}
	
	/**
	 * Indicates whether the parser is inside the `OtherSettings' tag.
	 * @author Piotrek
	 */
	private boolean fInOtherSettings = false;
	
	/**
	 * Max similar changes counts difference.
	 * @author Piotrek
	 */
	public int maxSimilarChangesCountsDifference = ChangesCountsComparator.MAX_SIMILAR_CHANGES_COUNTS_DIFFERENCE;
	
	/**
	 * Max similar departure hours of day difference.
	 * @author Piotrek
	 */
	public int maxSimilarDepartureHoursOfDayDifference = DepartureHoursOfDayComparator.MAX_SIMILAR_HOURS_DIFFERENCE;
	
	/**
	 * Max similar locations distance.
	 * @author Piotrek
	 */
	public double maxSimilarLocationsDistance = LocationsComparator.MAX_SIMILAR_LOCATIONS_DISTANCE;
	
	/**
	 * Max similar ranks difference.
	 * @author Piotrek
	 */
	public int maxSimilarRanksDifference = RanksComparator.MAX_SIMILAR_RANKS_DIFFERENCE;
	
	/**
	 * Max similar total durations difference.
	 * @author Piotrek
	 */
	public int maxSimilarTotalDurationsDifference = TotalDurationsComparator.MAX_SIMILAR_DURATIONS_DIFFERENCE;
						
	/**
	 * @author Piotrek
	 * @see org.xml.sax.ContentHandler#setDocumentLocator(org.xml.sax.Locator)
	 */
	public void setDocumentLocator(Locator locator)
	{
		this.locator = locator;
	}
	
	/**
	 * The SAX locator.
	 * @author Piotrek
	 */
	private Locator locator;
}
