/*
 * Created on 2005-04-04
 */
package gui.controls;

import java.awt.Color;
import java.awt.event.ActionListener;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

/**
 * A basic properties panel. All other properties panel derive from this one.
 * @author Piotrek
 */
public class BasicPropertiesPanel extends JPanel
{
	/**
	 * Constructor.
	 * @param actionListener An action listener for the buttons.
	 * @param title A title of the properties panel.
	 * @author Piotrek
	 */
	public BasicPropertiesPanel(ActionListener actionListener, String title)
	{
		super(null);
		
		if (actionListener == null)
			throw new NullPointerException("Action listener must not be null in BasicPropertiesPanel.BasicPropertiesPanel().");
		
		if (title == null)
			throw new NullPointerException("Title must not be null in BasicPropertiesPanel.BasicPropertiesPanel().");
		
		setBackground(Color.BLUE);
		setVisible(false);
		
		// title
		titleLabel = new JLabel(title, SwingConstants.CENTER);
		
		titleLabel.setBounds(10, 5, 160, 25);
		titleLabel.setBackground(Color.YELLOW);
		titleLabel.setOpaque(true);
		
		add(titleLabel);
		
		// name
		
		JLabel label = new JLabel("Name:");
		
		label.setBounds(10, 50, 160, 25);
		
		add(label);
		
		nameTextField = new JTextField();
		
		nameTextField.setBounds(10, 80, 160, 25);
		
		add(nameTextField);
		
		// continentName
		
		label = new JLabel("Continent:");
		
		label.setBounds(10, 110, 160, 25);
		
		add(label);
		
		continentNameComboBox = new JComboBox();
		
		continentNameComboBox.setBounds(10, 140, 160, 25);
		continentNameComboBox.addItem("C1");
		continentNameComboBox.addItem("C2");
		continentNameComboBox.setSelectedIndex(0);
		
		add(continentNameComboBox);
		
		// island
		
		label = new JLabel("Island:");
		
		label.setBounds(10, 170, 160, 25);
		
		add(label);
		
		islandNameComboBox = new JComboBox();
		
		islandNameComboBox.setBounds(10, 200, 160, 25);
		islandNameComboBox.addItem("Continental part");
		islandNameComboBox.addItem("A");
		islandNameComboBox.addItem("B");
		islandNameComboBox.addItem("C");
		islandNameComboBox.addItem("D");
		islandNameComboBox.addItem("E");
		islandNameComboBox.addItem("F");
		islandNameComboBox.setSelectedIndex(0);
		
		add(islandNameComboBox);
		
		// buttons
		
		applyButton = new JButton("Apply");
		
		applyButton.setBackground(Color.GREEN);
		applyButton.setBounds(10, 235, 160, 25);
		
		applyButton.addActionListener(actionListener);
		
		add(applyButton);
		
		removeButton = new JButton("Remove");
		
		removeButton.setBackground(Color.GREEN);
		removeButton.setBounds(10, 265, 160, 25);
		
		removeButton.addActionListener(actionListener);
		
		add(removeButton);
	}
	
	/**
	 * A title label.
	 * @author Piotrek
	 */
	protected JLabel titleLabel = null;
	
	/**
	 * A text field with the name of the worl object.
	 * @author Piotrek
	 */
	protected JTextField nameTextField = null;
	
	/**
	 * A combobox with a continentName name.
	 * @author Piotrek
	 */
	protected JComboBox continentNameComboBox = null;
	
	/**
	 * A combobox with a name of the island.
	 * @author Piotrek
	 */
	protected JComboBox islandNameComboBox = null;
	
	/**
	 * An `apply' button used when editing.
	 * @author Piotrek
	 */
	public JButton applyButton = null;
	
	/**
	 * A `remove' button used when editing.
	 * @author Piotrek
	 */
	public JButton removeButton = null;
}
