/*
 * Created on 2005-04-03
 */
package gui.controls;

import java.awt.event.ActionListener;
import world.Harbour;
import world.Location;

/**
 * A panel for edition of properties of a harbour.
 * @author Piotrek
 */
public class HarbourPropertiesPanel extends BasicPropertiesPanel
{
	/**
	 * Constructor.
	 * @param actionListener An action listener for the buttons.
	 * @author Piotrek
	 */
	public HarbourPropertiesPanel(ActionListener actionListener)
	{
		// the base class checks for a null value
		super(actionListener, "Harbour Properties");
	}
	
	/**
	 * Stores currently set harbour properties to the given harbour.
	 * @author Piotrek
	 * @param harbour A harbour to receive the currrently set properties.
	 */
	public void StoreData(Harbour harbour)
	{
		if (harbour == null)
			throw new NullPointerException("Harbour must not be null in HarbourPropertiesPanel.StoreData().");
		
		if (nameTextField.getText().compareTo("") == 0)
			harbour.name = "Anonymous Harbour";
		else
			harbour.name = nameTextField.getText();
		
		Location location = harbour.GetLocation();
		
		location.continentName = (String)continentNameComboBox.getSelectedItem();
		location.islandName = (String)islandNameComboBox.getSelectedItem();
	}
	
	/**
	 * Loads the properties of the given harbour.
	 * @author Piotrek
	 * @param harbour A source harbour.
	 */
	public void LoadData(Harbour harbour)
	{
		if (harbour == null)
			throw new NullPointerException("Harbour must not be null in HarbourPropertiesPanel.LoadData().");
		
		nameTextField.setText(harbour.name);
		
		Location location = harbour.GetLocation();
		
		continentNameComboBox.setSelectedItem(location.continentName);
		islandNameComboBox.setSelectedItem(location.islandName);
	}
}
