/*
 * Created on 2005-04-25
 */
package gui.controls;

import gui.dialogs.MainDialog;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import world.Location;

/**
 * Used to show directions to the user to choose two locations (origin and the destination) 
 * of the travel and launch the test.
 * @author Piotrek
 */
public final class LocationsChooser extends JPanel implements ActionListener, CaretListener
{
	/**
	 * Constructor.
	 * @author Piotrek
	 * @param worldView A world view that will get the clicks indicating the origin and the destination.
	 * @param parent A main dialog containing this panel.
	 */
	public LocationsChooser(WorldView worldView, MainDialog parent)
	{
		super(null);
		
		if (worldView == null)
			throw new NullPointerException("The world view must not be null in LocationsChooser.LocationsChooser().");
		
		if (parent == null)
			throw new NullPointerException("The parent must not be null in LocationsChooser.LocationsChooser().");
		
		this.worldView = worldView;
		this.parent = parent;
		
		worldView.SetLocationsChooser(this);
		
		setBackground(Color.BLUE);
		setVisible(false);
		
		// directions label
		
		directions = new JLabel("", SwingConstants.CENTER);
		
		add(directions);
		
		directions.setBounds(10, 10, 160, 25);
		directions.setForeground(Color.YELLOW);
		
		// origin and destination
		
		JPanel panel = new JPanel(null);
		
		panel.setBounds(10, 50, 160, 200);
		panel.setBackground(Color.BLACK);
		
		add(panel);
		
		// origin
		JLabel label = new JLabel("Origin:", SwingConstants.LEFT);
		
		label.setOpaque(false);
		label.setBounds(10, 10, 140, 25);
		label.setForeground(Color.WHITE);
		
		panel.add(label);
		
		label = new JLabel(" x:", SwingConstants.LEFT);
		
		label.setOpaque(false);
		label.setBounds(10, 40, 65, 25);
		label.setForeground(Color.WHITE);
		
		panel.add(label);
		
		label = new JLabel(" y:", SwingConstants.LEFT);
		
		label.setOpaque(false);
		label.setBounds(85, 40, 65, 25);
		label.setForeground(Color.WHITE);
		
		panel.add(label);
				
		originXTextField = new JTextField();
		
		originXTextField.setBounds(10, 70, 65, 25);
		originXTextField.addCaretListener(this);
		
		panel.add(originXTextField);
		
		originYTextField = new JTextField();
		
		originYTextField.setBounds(85, 70, 65, 25);
		originYTextField.addCaretListener(this);
		
		panel.add(originYTextField);
		
		// destination
		
		label = new JLabel("Destination:", SwingConstants.LEFT);
		
		label.setOpaque(false);
		label.setBounds(10, 100, 140, 25);
		label.setForeground(Color.WHITE);
		
		panel.add(label);
		
		label = new JLabel(" x:", SwingConstants.LEFT);
		
		label.setOpaque(false);
		label.setBounds(10, 130, 65, 25);
		label.setForeground(Color.WHITE);
		
		panel.add(label);
		
		label = new JLabel(" y:", SwingConstants.LEFT);
		
		label.setOpaque(false);
		label.setBounds(85, 130, 65, 25);
		label.setForeground(Color.WHITE);
		
		panel.add(label);
		
		destinationXTextField = new JTextField();
		
		destinationXTextField.setBounds(10, 160, 65, 25);
		destinationXTextField.addCaretListener(this);
		
		panel.add(destinationXTextField);
		
		destinationYTextField = new JTextField();
		
		destinationYTextField.setBounds(85, 160, 65, 25);
		destinationYTextField.addCaretListener(this);
		
		panel.add(destinationYTextField);
		
		// launch button
		launchButton = new JButton("Launch");
		
		add(launchButton);
		
		launchButton.setBounds(40, 280, 100, 25);
		launchButton.setBackground(Color.GREEN);
		
		launchButton.addActionListener(this);
		
		launchButton.setEnabled(false);
		
		// cancel button
		cancelButton = new JButton("Cancel");
		
		add(cancelButton);
		
		cancelButton.setBounds(40, 310, 100, 25);
		cancelButton.setBackground(Color.GREEN);
		
		cancelButton.addActionListener(this);
	}
	
	/**
	 * The `Cancel' button.
	 * @author Piotrek
	 */
	private JButton cancelButton = null;
	
	/**
	 * The `Launch' button.
	 * @author Piotrek
	 */
	private JButton launchButton = null;
	
	/**
	 * The x coordinate of the origin.
	 * @author Piotrek
	 */
	private JTextField originXTextField = null;
	
	/**
	 * The y coordinate of the origin.
	 * @author Piotrek
	 */
	private JTextField originYTextField = null;
	
	/**
	 * The x coordinate of the destination.
	 * @author Piotrek
	 */
	private JTextField destinationXTextField = null;
	
	/**
	 * The y coordinate of the destination.
	 * @author Piotrek
	 */
	private JTextField destinationYTextField = null;
	
	/**
	 * A world view. It provides the origin and destination. The changes of the view mode
	 * are applied to it.
	 * @author Piotrek
	 */
	private WorldView worldView = null;
	
	/**
	 * The main dialog containing this panel. It is required to lock the toolbar.
	 * @author Piotrek
	 */
	private MainDialog parent = null;
	
	/**
	 * A label used to display the directions to the user.
	 * @author Piotrek
	 */
	private JLabel directions = null;
		
	/**
	 * Gets the origin of the travel selected by the user.
	 * @author Piotrek
	 * @return The location selected by the user or null if there is none yet.
	 */
	public Location GetOrigin()
	{
		Location location = null;
		
		try
		{
			int x = Integer.parseInt(originXTextField.getText());
			int y = Integer.parseInt(originYTextField.getText());
			
			if (x >=0 && x <= WorldView.width && y >=0 && y <= WorldView.height)
				location = new Location(x, y);
		}
		
		catch (NumberFormatException xcp)
		{
			// ignore
		}
		
		return location;
	}
		
	/**
	 * Gets the destination of the travel selected by the user.
	 * @author Piotrek
	 * @return The location selected by the user or null if there is none yet.
	 */
	public Location GetDestination()
	{
		Location location = null;
		
		try
		{
			int x = Integer.parseInt(destinationXTextField.getText());
			int y = Integer.parseInt(destinationYTextField.getText());
			
			if (x >=0 && x <= WorldView.width && y >=0 && y <= WorldView.height)
				location = new Location(x, y);
		}
		
		catch (NumberFormatException xcp)
		{
			// ignore
		}
		
		return location;
	}
	
	/**
	 * Indicates whether the mouse input is an origin or a destination of the travel.
	 * @author Piotrek
	 */
	private boolean fOrigin = true;
	
	/**
	 * Use it to start a new selection process. It makes the panel visible.
	 * @author Piotrek
	 */
	public void StartChoosing()
	{
		directions.setText("Choose the origin.");
		
		fOrigin = true;
		
		setVisible(true);
		
		worldView.SetMode(WorldView.MODE_ORIGIN_DESTINATION_CHOOSING);
		
		launchButton.setEnabled(GetOrigin() != null && GetDestination() != null);
		cancelButton.setEnabled(true);
		
		originXTextField.setEnabled(true);
		originYTextField.setEnabled(true);
		
		destinationXTextField.setEnabled(true);
		destinationYTextField.setEnabled(true);
	}
	
	/**
	 * Called by the world view after the user has clicked the origin/destination 
	 * of the travel.
	 * @author Piotrek
	 * @param location A locaction clicked by the user.
	 */
	public void ReceiveLocation(Location location)
	{
		if (location == null)
			throw new NullPointerException("Location must not be null in LocationsChooser.ReceiveLocation().");
		
		if (fOrigin)
		{
			originXTextField.setText(Integer.toString(location.x));
			originYTextField.setText(Integer.toString(location.y));
			
			directions.setText("Choose the destinantion.");
		}
		else
		{
			destinationXTextField.setText(Integer.toString(location.x));
			destinationYTextField.setText(Integer.toString(location.y));
			
			directions.setText("Choose the origin.");
		}
		
		worldView.repaint();
		
		fOrigin = !fOrigin;
		
		launchButton.setEnabled(GetOrigin() != null && GetDestination() != null);
	}
		
	/**
	 * @author Piotrek
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e)
	{
		if (e.getSource() == cancelButton)
		{
			setVisible(false);
			
			worldView.SetMode(WorldView.MODE_NORMAL_VIEW);
			
			worldView.repaint();
		}
		
		if (e.getSource() == launchButton)
		{
			worldView.SetMode(WorldView.MODE_NORMAL_VIEW);
			
			directions.setText("Please wait...");
			
			launchButton.setEnabled(false);
			cancelButton.setEnabled(false);
			
			originXTextField.setEnabled(false);
			originYTextField.setEnabled(false);
			
			destinationXTextField.setEnabled(false);
			destinationYTextField.setEnabled(false);
						
			parent.LaunchFirstStepSolver();
			
			worldView.repaint();
		}
	}
		
	/**
	 * @author Piotrek
	 * @see javax.swing.event.CaretListener#caretUpdate(javax.swing.event.CaretEvent)
	 */
	public void caretUpdate(CaretEvent e)
	{
		launchButton.setEnabled(GetOrigin() != null && GetDestination() != null);
		
		worldView.repaint();
	}
}
