/*
 * Created on 2005-04-06
 */
package gui.controls;

import gui.dialogs.ConnectionTimetableCreator;
import gui.dialogs.MainDialog;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import world.MeanOfTransportation;
import world.Station;
import world.World;

/**
 * A universal chooser used to select stations when defining a connection.
 * @author Piotrek
 */
public final class StationsChooser extends JPanel implements ActionListener
{
	/**
	 * Constructor.
	 * @author Piotrek 
	 * @param view A view displaying a world we take the stations from.
	 * @param parent A parent dialog of this control.
	 */
	public StationsChooser(WorldView view, MainDialog parent)
	{
		super(null);
		
		if (view == null)
			throw new NullPointerException("World view must not be null in StationsChooser.StationsChooser().");
		
		if (parent == null)
			throw new NullPointerException("Main dialog must not be null in StationsChooser.StationsChooser().");
				
		setBackground(Color.BLUE);
		setVisible(false);
		
		this.view = view;
		this.parent = parent;
		
		view.SetUniversalStationsChooser(this);
		
		// title
		JLabel titleLabel = new JLabel("Phase 1: Choose Stations", SwingConstants.CENTER);
		
		titleLabel.setBounds(10, 5, 160, 25);
		titleLabel.setBackground(Color.YELLOW);
		titleLabel.setOpaque(true);
		
		add(titleLabel);
		
		// name label
		JLabel label = new JLabel("Name:");
		
		label.setBounds(10, 45, 150, 25);
		label.setForeground(Color.WHITE);
		
		add(label);
		
		// name text field
		nameTextField = new JTextField();
		
		nameTextField.setEditable(false);
		nameTextField.setBounds(10, 70, 160, 25);
		
		add(nameTextField);
		
		// type label
		
		typeLabel = new JLabel("");
		
		typeLabel.setBounds(10, 100, 160, 25);
		typeLabel.setForeground(Color.WHITE);
		
		add(typeLabel);
		
		// remove button
		
		removeButton = new JButton("Remove");
		
		add(removeButton);
		
		removeButton.setBounds(5, 440, 80, 25);
		removeButton.setBackground(Color.ORANGE);
		removeButton.setEnabled(false);
		
		removeButton.addActionListener(this);
		
		// create button
		
		createButton = new JButton("Create");
		
		add(createButton);
		
		createButton.setBounds(95, 440, 80, 25);
		createButton.setBackground(Color.ORANGE);
		createButton.setEnabled(false);
		
		createButton.addActionListener(this);
		
		// list
		
		stationsList = new StationsList();
		
		stationsList.setBounds(5, 130, 170, 300);
		stationsList.setVisible(true);
		
		add(stationsList);
	}
	
	/**
	 * A name text field.
	 * @author Piotrek
	 */
	private JTextField nameTextField = null;
	
	/**
	 * A parent dialog of this control.
	 * @author Piotrek
	 */
	private MainDialog parent = null;
	
	/**
	 * A kind of stations that are to be added to this list view.
	 * @author Piotrek
	 */
	private int wantedStationType = MeanOfTransportation.AIRPLANE;
	
	/**
	 * Sets a type of stations that are to be added to this list.
	 * @author Piotrek
	 * @param stationType A type of stations that are to be added to this list.
	 */
	public void SetStationType(int stationType)
	{
		switch (stationType)
		{
			case MeanOfTransportation.AIRPLANE:
				
				typeLabel.setText("Airports:");
				
				break;
			
			case MeanOfTransportation.TRAIN:
				
				typeLabel.setText("Train Stations:");
				
				break;
			
			case MeanOfTransportation.BUS:
				
				typeLabel.setText("Bus Stops:");
				
				break;
			
			case MeanOfTransportation.SHIP:
				
				typeLabel.setText("Harbours:");
				
				break;
			
			default:
				
				setVisible(false);
			
				throw new IllegalArgumentException("Unexpected type of mean of transportation in StationsChooser.SetStationType().");
		}
		
		nameTextField.setText(World.GetInstance().GetNextConnectionName(stationType));
		
		stationsList.Clear();
		
		wantedStationType = stationType;
	}
	
	/**
	 * Gets a type of stations that are to be added to this list.
	 * @author Piotrek
	 * @return A type of stations that are to be added to this list.
	 */
	public int GetStationType()
	{		
		return wantedStationType;
	}
	
	/**
	 * A label indicating what type of stations are to be added.
	 * @author Piotrek
	 */
	private JLabel typeLabel = null;
	
	/**
	 * A list view used to display an ordered list of currently selected stations.
	 * @author Piotrek
	 */
	private StationsList stationsList = null;
	
	/**
	 * A view of the world we take stations from. Needed to cause redraw.
	 * @author Piotrek
	 */
	private WorldView view = null;
	
	/**
	 * A button for removing stations from the list.
	 * @author Piotrek
	 */
	private JButton removeButton = null;
	
	/**
	 * A button for accepting a list of selected stations and going to the phase 2 of the connection creation.
	 * @author Piotrek
	 */
	private JButton createButton = null;
	
	/**
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 * @author Piotrek
	 */
	public void actionPerformed(ActionEvent e)
	{
		// remove button
		if (e.getSource() == removeButton)
		{
			stationsList.RemoveSelectedStation();
			
			view.repaint();
			
			if (stationsList.GetSize() == 0)
				removeButton.setEnabled(false);
			
			if (stationsList.GetSize() < 2)
				createButton.setEnabled(false);
		}
		
		// create button
		if (e.getSource() == createButton)
		{
			ArrayList stations = GetStations();
			
			ConnectionTimetableCreator timetableCreator = new ConnectionTimetableCreator("Phase 2: Define a Timetable", nameTextField.getText(), stations, wantedStationType, null);
			
			timetableCreator.setVisible(true);
			
			if (timetableCreator.GetDialogResult() == ConnectionTimetableCreator.OK)
			{
				parent.HideStationsChooser();
				
				view.repaint();
			}
		}
	}
	
	/**
	 * Adds a given station to the list.
	 * @author Piotrek
	 * @param station A station to be added to the list.
	 */
	public void AddStation(Station station)
	{
		if (station == null)
			throw new NullPointerException("Station must not be null in StationsChooser.AddStation().");
		
		stationsList.AddStation(station);
		
		stationsList.repaint();
		view.repaint();
		
		if (stationsList.GetSize() > 0)
			removeButton.setEnabled(true);
		
		if (stationsList.GetSize() >= 2)
			createButton.setEnabled(true);
	}
	
	/**
	 * Gets a list of all the chosen stations.
	 * @author Piotrek
	 * @return A list of all the chosen stations.
	 */
	public ArrayList GetStations()
	{
		return stationsList.GetStations();
	}
}
