/*
 * Created on 2005-04-06
 */
package gui.controls;

import java.awt.Color;
import java.awt.Graphics;
import java.util.ArrayList;
import javax.swing.DefaultListModel;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import world.Station;

/**
 * A list displaying stations.
 * @author Piotrek
 */
public class StationsList extends JComponent
{
	/**
	 * Constructor.
	 * @author Piotrek
	 */
	public StationsList()
	{
		super();
		
		setBackground(Color.YELLOW);
		
		list = new JList();
		
		list.setModel(listModel);
		
		list.setCellRenderer(new StationsCellRenderer());
		
		list.setFixedCellWidth(180);
		list.setFixedCellHeight(75);
				
		listPane = new JScrollPane(list);
		
		add(listPane);
		
		list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	}
	
	/**
	 * The list model providing an API to add and remove items from the list.
	 * @author Piotrek
	 */
	private DefaultListModel listModel = new DefaultListModel();
	
	/**
	 * The list presenting the world objects.
	 * @author Piotrek
	 */
	private JList list;
	
	/**
	 * The scroll pane making the list scrollable.
	 * @author Piotrek
	 */
	private JScrollPane listPane;
	
	/**
	 * @author Piotrek
	 * @see JComponent#paint(java.awt.Graphics)
	 */
	public void paint(Graphics g)
	{
		super.paint(g);
		
		g.setColor(getBackground());
		g.fillRect(0, 0 , getSize().width, getSize().height);
						
		paintChildren(g);
	}
	
	/**
	 * @author Piotrek
	 * @see java.awt.Component#setBounds(int, int, int, int)
	 */
	public void setBounds(int x, int y, int width, int height)
	{
		super.setBounds(x, y, width, height);
		
		listPane.setBounds(5, 10, width - 10, height - 20);
	}
	
	/**
	 * Adds a single station to the list.
	 * @author Piotrek
	 * @param station A world object to be added.
	 */
	public void AddStation(Station station)
	{
		if (station == null)
			throw new NullPointerException("Station must not be null in StationsList.AddStation().");
		
		if (listModel.contains(station))
			return;
		
		listModel.addElement(station);
	}
	
	/**
	 * Removes a single station from the list.
	 * @author Piotrek
	 * @param station A world object to be removed.
	 */
	public void RemoveStation(Station station)
	{
		if (station == null)
			throw new NullPointerException("Station must not be null in StationsList.RemoveStation().");
		
		listModel.removeElement(station);
	}
	
	/**
	 * Removes the currently selected station from the list.
	 * @author Piotrek
	 */
	public void RemoveSelectedStation()
	{
		Station selectedStation = (Station)list.getSelectedValue();
		
		if (selectedStation != null)
			listModel.removeElement(selectedStation);	
	}
	
	/**
	 * Empties the list.
	 * @author Piotrek
	 */
	public void Clear()
	{
		listModel.clear();
	}
	
	/**
	 * Gets the number of stations in the list.
	 * @author Piotrek
	 * @return A number of stations in the list. 
	 */
	public int GetSize()
	{
		return listModel.getSize();
	}
	
	/**
	 * Gets a list of all stations on the list.
	 * @author Piotrek
	 * @return A list of all stations on the list.
	 */
	public ArrayList GetStations()
	{
		ArrayList objects = new ArrayList();
		
		int size = listModel.getSize();
		
		for (int i = 0; i < size; ++i)
		{
			Object object = listModel.getElementAt(i);
			
			objects.add(object);
		}
		
		return objects;
	}
}
