/*
 * Created on 2005-06-02
 */
package gui.controls;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.ListCellRenderer;
import scheduler.ConnectionsFinderTask;
import world.TravelTime;

/**
 * A cell renderer used to display a travel solution.
 * @author Piotrek
 */
public class TravelSolutionsCellRenderer extends JComponent implements ListCellRenderer
{
	/**
	 * Constructor.
	 * @author Piotrek
	 */
	public TravelSolutionsCellRenderer()
	{
		super();
				
		setLayout(null);
	}
	
	/**
	 * @author Piotrek
	 * @see javax.swing.ListCellRenderer#getListCellRendererComponent(javax.swing.JList, java.lang.Object, int, boolean, boolean)
	 */
	public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus)
	{
		solution = (ConnectionsFinderTask)value;
		
		itemIndex = index + 1;
	       
		if (isSelected)
		{
            setBackground(list.getSelectionBackground());
            setForeground(list.getSelectionForeground());
		}
        else
        {
        	setBackground(list.getBackground());
        	setForeground(list.getForeground());
        }
		
		setEnabled(list.isEnabled());
		setFont(list.getFont());
	                
        return this;
	}
	
	/**
	 * A solution we are rendering.
	 * @author Piotrek
	 */
	private ConnectionsFinderTask solution = null;
	
	/**
	 * A one-based index on the list of the solutions we are rendering.
	 * @author Piotrek
	 */
	private int itemIndex = -1;
	
	/**
	 * @author Piotrek
	 * @see java.awt.Component#paint(java.awt.Graphics)
	 */
	public void paint(Graphics g)
	{
		g.setColor(getBackground());
		g.fillRect(0, 0 , getSize().width, getSize().height);
			
		g.setColor(Color.RED);
		g.drawString("Travel Solution #" + itemIndex, 5, 15);
		
		g.setColor(Color.BLUE);
		g.drawString("(" + solution.GetConnectionChunksCount() + " chunk(s) / " + solution.GetChangesCount() + " change(s))", 5, 30);
		g.drawString("Total duration:", 5, 44);
		g.drawString(TravelTime.MinutesToString(solution.GetTotalDuration()), 5, 58);
		g.drawString("Avg/Max change duration:", 5, 72);
		g.drawString(TravelTime.GetTime(solution.GetAverageChangeDuration()) + " / " + TravelTime.GetTime(solution.GetMaxChangeDuration()), 5, 86);
		g.drawString("History based judgement:", 5, 100);
		g.drawString(Double.toString(solution.historyBasedJudgement), 5, 114);
			
		if (itemIndex != 1)
			g.drawLine(0, 0, getWidth(), 0);
	}
}
