/*
 * Created on 2005-06-02
 */
package gui.controls;

import gui.SolutionsCharacteristicsFileFilter;
import gui.dialogs.StatsDialog;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileFilter;
import scheduler.ConnectionsFinderTask;
import scheduler.TestOptions;

/**
 * A list used to choose one of the generated travel solutions to be displayed in the view. It might be also sorted.
 * @author Piotrek
 */
public final class TravelSolutionsChooser extends JPanel implements ListSelectionListener, ActionListener
{
	/**
	 * Constructor.
	 * @author Piotrek
	 * @param worldView A view showing the selected travel solution.
	 * @param actionListener An action listener for the `Close' button.
	 */
	public TravelSolutionsChooser(WorldView worldView, ActionListener actionListener)
	{
		super(null);
		
		if (worldView == null)
			throw new NullPointerException("World view must not be null in TravelSolutionsChooser.TravelSolutionsChooser().");
		
		if (actionListener == null)
			throw new NullPointerException("Action listener must not be null in TravelSolutionsChooser.TravelSolutionsChooser().");
		
		this.worldView = worldView;
		
		worldView.SetTravelSolutionsChooser(this);
		
		setBackground(Color.BLUE);
		
		// creating a list view
		 
		solutionsList = new JList();
		
		solutionsList.setModel(listModel);
		solutionsList.setCellRenderer(new TravelSolutionsCellRenderer());
		
		solutionsList.addListSelectionListener(this);
	
		solutionsList.setFixedCellWidth(150);
		solutionsList.setFixedCellHeight(120);
				
		listPane = new JScrollPane(solutionsList);
		
		add(listPane);
		
		solutionsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		
		// second step close button
		closeButton = new JButton("Close");
		
		add(closeButton);
		
		closeButton.setBackground(Color.ORANGE);
		
		closeButton.addActionListener(actionListener);
		
		// accept button
		acceptButton = new JButton("Accept");
		
		add(acceptButton);
		
		acceptButton.setBackground(Color.ORANGE);
		
		acceptButton.addActionListener(actionListener);
		
		acceptButton.setEnabled(false);
		
		// reject button
		rejectButton = new JButton("Reject");
		
		add(rejectButton);
		
		rejectButton.setBackground(Color.ORANGE);
		
		rejectButton.addActionListener(actionListener);
		
		rejectButton.setEnabled(false);
		
		// sort button
		sortButton = new JButton("Sort");
		
		add(sortButton);
		
		sortButton.setBackground(Color.ORANGE);
		
		sortButton.addActionListener(actionListener);
		
		sortButton.setEnabled(false);
		
		// save button
		saveButton = new JButton("Save");
		
		add(saveButton);
		
		saveButton.setBackground(Color.ORANGE);
		
		saveButton.addActionListener(this);
		
		saveButton.setEnabled(false);
		
		// stats button
		statsButton = new JButton("Stats");
		
		add(statsButton);
		
		statsButton.setBackground(Color.ORANGE);
		
		statsButton.addActionListener(this);
		
		statsButton.setEnabled(false);
						
		setVisible(false);
	}
	
	/**
	 * A button used to accept the selected travel proposal.
	 * @author Piotrek
	 */
	private JButton acceptButton = null;
	
	/**
	 * A button used to reject the selected travel proposal.
	 * @author Piotrek
	 */
	private JButton rejectButton = null;
	
	/**
	 * A button used to sort the current list of travel solutions.
	 * @author Piotrek
	 */
	private JButton sortButton = null;
	
	/**
	 * A button used to close the travel solutions choosing.
	 * @author Piotrek
	 */
	private JButton closeButton = null;
	
	/**
	 * A button used to save the travel solutions list to a file.
	 * @author Piotrek
	 */
	private JButton saveButton = null;
	
	/**
	 * A button used to show the queries stats.
	 * @author Piotrek
	 */
	private JButton statsButton = null;
	
	/**
	 * Gets the `accept' button.
	 * @author Piotrek
	 * @return A `accept' button.
	 */
	public JButton GetAcceptButton()
	{
		return acceptButton;
	}
	
	/**
	 * Gets the `reject' button.
	 * @author Piotrek
	 * @return A `reject' button.
	 */
	public JButton GetRejectButton()
	{
		return rejectButton;
	}
	
	/**
	 * Gets the `sort' button.
	 * @author Piotrek
	 * @return A `sort' button.
	 */
	public JButton GetSortButton()
	{
		return sortButton;
	}
	
	/**
	 * Gets the `close' button.
	 * @author Piotrek
	 * @return A `close' button.
	 */
	public JButton GetCloseButton()
	{
		return closeButton;
	}
	
	/**
	 * A view displaying the selected travel solution.
	 * @author Piotrek
	 */
	private WorldView worldView = null;
	
	/**
	 * A list control displaying the travel solutions.
	 * @author Piotrek
	 */
	private JList solutionsList = null;
	
	/**
	 * The list model providing an API to add items to the list.
	 * @author Piotrek
	 */
	private DefaultListModel listModel = new DefaultListModel();
		
	/**
	 * The scroll pane making the list scrollable.
	 * @author Piotrek
	 */
	private JScrollPane listPane;
	
	/**
	 * @author Piotrek
	 * @see JComponent#paint(java.awt.Graphics)
	 */
	public void paint(Graphics g)
	{
		super.paint(g);
		
		g.setColor(getBackground());
		g.fillRect(0, 0 , getSize().width, getSize().height);
						
		paintChildren(g);
	}
	
	/**
	 * @author Piotrek
	 * @see java.awt.Component#setBounds(int, int, int, int)
	 */
	public void setBounds(int x, int y, int width, int height)
	{
		super.setBounds(x, y, width, height);
		
		listPane.setBounds(5, 5, width - 10, height - 100);
		
		// buttons
		acceptButton.setBounds(5, height - 90, (width - 15) / 2, 25);
		
		rejectButton.setBounds(10 + (width - 15) / 2, height - 90, (width - 15) / 2, 25);
		
		sortButton.setBounds(5, height - 60, (width - 15) / 2, 25);
		
		closeButton.setBounds(10 + (width - 15) / 2, height - 60, (width - 15) / 2, 25);
		
		saveButton.setBounds(5, height - 30, (width - 15) / 2, 25);
		
		statsButton.setBounds(10 + (width - 15) / 2, height - 30, (width - 15) / 2, 25);
	}

	/**
	 * @author Piotrek
	 * @see javax.swing.event.ListSelectionListener#valueChanged(javax.swing.event.ListSelectionEvent)
	 */
	public void valueChanged(ListSelectionEvent e)
	{
		worldView.repaint();
		
		boolean fSelectedItemExist = solutionsList.getSelectedValue() != null;
		
		acceptButton.setEnabled(fSelectedItemExist);
		rejectButton.setEnabled(fSelectedItemExist);
		
		sortButton.setEnabled(listModel.size() > 1);
		saveButton.setEnabled(listModel.size() > 1);
	}
	
	/**
	 * Loads the given list of travel solutions into the list control.
	 * @author Piotrek
	 * @param solutions A list of travel solutions to be loaded.
	 */
	public void LoadSolutions(ArrayList solutions)
	{
		if (solutions == null)
			throw new NullPointerException("Solutions must not be null in TravelSolutionsChooser.LoadSolutions().");
		
		listModel.clear();
		
		Iterator iterator = solutions.iterator();
		
		while (iterator.hasNext())
		{
			ConnectionsFinderTask task = (ConnectionsFinderTask)iterator.next();
			
			listModel.addElement(task);
		}
		
		// should the sort & save buttons be enabled???
		sortButton.setEnabled(listModel.size() > 1);
		saveButton.setEnabled(listModel.size() > 1);
		statsButton.setEnabled(true);
	}
	
	/**
	 * Gets the currently selected solution.
	 * @author Piotrek
	 * @return A currently selected solution or a null value if there is none.
	 */
	public ConnectionsFinderTask GetSelectedSolution()
	{
		ConnectionsFinderTask selectedTask = (ConnectionsFinderTask)solutionsList.getSelectedValue();
		
		return selectedTask;
	}
	
	/**
	 * Removes the currently selected travel solution.
	 * @author Piotrek
	 */
	public void RemoveSelectedTravelSolution()
	{
		listModel.removeElement(solutionsList.getSelectedValue());
	}
	
	/**
	 * Gets the current list of travel solutions in the chooser.
	 * @author Piotrek
	 * @return A current list of travel solutions stored in the chooser. If there are no solutions, an empty list is returned.
	 */
	public ArrayList GetSolutionsList()
	{
		int solutionsCount = listModel.size();
		
		ArrayList solutionsList = new ArrayList(solutionsCount);
		
		for (int i = 0; i < solutionsCount; ++i)
			solutionsList.add(listModel.get(i));
		
		return solutionsList;
	}
	
	
	/**
	 * @author Piotrek
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e)
	{
		if (e.getSource() == saveButton)
		{
			JFileChooser chooser = new JFileChooser();
			
			FileFilter filter = new SolutionsCharacteristicsFileFilter();
			
			chooser.setFileFilter(filter);
			chooser.setMultiSelectionEnabled(false);
			chooser.setDialogTitle("Enter a name of the output file");
						
			if (chooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) 
			{
			    String fileName = chooser.getSelectedFile().getPath();
			   
			    if (!fileName.endsWith(SolutionsCharacteristicsFileFilter.FILE_EXTENSION))
			      fileName += SolutionsCharacteristicsFileFilter.FILE_EXTENSION;
					
			    // save to the given file
			    SaveListToFile(fileName);
			}
		}
		
		if (e.getSource() == statsButton)
		{
			StatsDialog statsDialog = new StatsDialog();
			
			statsDialog.setLocation(250, 200);
			
			statsDialog.setVisible(true);
		}
	}
	
	/**
	 * Saves the current list of solutions to the given file.
	 * @author Piotrek
	 * @param fileName A name of the file to save the list to.
	 */
	private void SaveListToFile(String fileName)
	{
		try
		{
			FileWriter fileWriter = new FileWriter(fileName);
						
			int solutionsCount = listModel.size();
			
			fileWriter.write("Average change duration; Max change duration; Changes count; Chunks count; Total distance; Total duration; History based judgement\r\n");
			
			for (int i = 0; i < solutionsCount; ++i)
			{
				ConnectionsFinderTask solution = (ConnectionsFinderTask)listModel.get(i);
				
				fileWriter.write(solution.GetAverageChangeDuration() + "; " + solution.GetMaxChangeDuration() + "; " + solution.GetChangesCount() + "; " + solution.GetConnectionChunksCount() + "; " + solution.GetTotalDistance() + "; " + solution.GetTotalDuration() + "; " + solution.historyBasedJudgement +  "\r\n");
			}
			
			// queries stats...
			
			fileWriter.write("Stations in the range queries:;" + TestOptions.GetStationsInRangeQueriesCount() + "\r\n");
			fileWriter.write("Connections for the given station queries:;" + TestOptions.GetConnectionsForStationsQueriesCount() + "\r\n");
			
			fileWriter.close();
		}
		
		catch (IOException xcp)
		{
			JOptionPane.showMessageDialog(null, "There was an exception while saving the list of travel solutions: " + xcp.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
		}
	}
}
