/*
 * Created on 2005-08-30
 */
package gui.dialogs;

import java.awt.Color;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import scheduler.TestOptions;

/**
 * A dialog used to configure the planning algorithm options.
 * @author Piotrek
 */
public final class AlgorithmConfigurationDialog extends JDialog implements ActionListener
{
	/**
	 * Constructor.
	 * @author Piotrek
	 * @param parent The parent main dialog of this configuration dialog.
	 */
	public AlgorithmConfigurationDialog(MainDialog parent)
	{
		if (parent == null)
			throw new NullPointerException("Parent must not be null in AlgorithmConfigurationDialog.AlgorithmConfigurationDialog().");
		
		this.parent = parent;
		
		setTitle("Planning Algorithm Configuration");
		
		setModal(true);
		setVisible(false);
		setResizable(false);
		
		setSize(540, 450);
		
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		
		// panel	
		panel = getContentPane();
		
		panel.setLayout(null);
		panel.setBackground(Color.YELLOW);
		
		setContentPane(panel);
		
		// ok and cancel buttons
		AddOKCancel();
		
		// options panel
		AddOptionsPanel();
		
		// rules and CRN config buttons
		AddRulesCRNConfigButtons();
	}
	
	/**
	 * The parent dialog of this configuration dialog.
	 * @author Piotrek
	 */
	private MainDialog parent = null;
	
	/**
	 * An OK button.
	 * @author Piotrek
	 */
	private JButton okButton = null;
	
	/**
	 * A cancel button.
	 * @author Piotrek
	 */
	private JButton cancelButton = null;
	
	/**
	 * The OK dialog result.
	 * @author Piotrek
	 */
	public static final int OK = 1;
	
	/**
	 * The cancel dialog result.
	 * @author Piotrek
	 */
	public static final int CANCEL = 2;
	
	/**
	 * The dialog result.
	 * @author Piotrek
	 */
	private int dialogResult = CANCEL;
	
	/**
	 * Gets the dialog result.
	 * @author Piotrek
	 * @return The dialog result.
	 */
	public int GetDialogResult()
	{
		return dialogResult;
	}
	
	/**
	 * A main panel of this dialog.
	 * @author Piotrek
	 */
	private Container panel = null;
	
	/**
	 * Adds the OK and Cancel buttons.
	 * @author Piotrek
	 */
	private void AddOKCancel()
	{
		// ok
		
		okButton = new JButton("OK");
		
		okButton.setBounds(10, 385, 250, 25);
		okButton.setBackground(Color.GREEN);
		
		okButton.addActionListener(this);
		
		add(okButton);
		
		// cancel
		
		cancelButton = new JButton("Cancel");
		
		cancelButton.setBounds(270, 385, 250, 25);
		cancelButton.setBackground(Color.GREEN);
		
		cancelButton.addActionListener(this);
		
		add(cancelButton);
	}
	
	/**
	 * @author Piotrek
	 * @see ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e)
	{
		// OK buttom
		if (e.getSource() == okButton)
		{
			OnOKButton();
		}
		
		// Cancel button
		if (e.getSource() == cancelButton)
		{
			dialogResult = CANCEL;
			
			setVisible(false);
						
			return;
		}
		
		// crn settings
		if (e.getSource() == editCRNSettings)
		{
			CRNSettingsEditionDialog crnSettingsDialog = new CRNSettingsEditionDialog("Edit CRN Settings");
			
			crnSettingsDialog.SetCRN(parent.GetCRN());
			
			crnSettingsDialog.setVisible(true);
		}
		
		// edit means of transportation choosing rules
		if (e.getSource() == editMeansOfTransportationRules)
		{
			try
			{
				// trying windows
				Runtime.getRuntime().exec("notepad.exe \"./Data/JESS means of transportation rules.txt\"");
			}
			
			catch (IOException xcp)
			{
				if (xcp.getMessage().endsWith("error=2"))
				{
					// there is no notepad, unix/linux?
					try
					{
						Runtime.getRuntime().exec("vi.exe \"./Data/JESS means of transportation rules.txt\"");
					}
					
					catch (IOException xcp2)
					{						
						JOptionPane.showMessageDialog(null, "There was an exception: " + xcp2.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
					}
				}
				
				JOptionPane.showMessageDialog(null, "There was an exception: " + xcp.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
			}
		}
		
		// edit changes choosing rules
		if (e.getSource() == editChangeRules)
		{
			try
			{
				// trying windows
				Runtime.getRuntime().exec("notepad.exe \"./Data/JESS changes rules.txt\"");
			}
			
			catch (IOException xcp)
			{
				if (xcp.getMessage().endsWith("error=2"))
				{
					// there is no notepad, unix/linux?
					try
					{						
						Runtime.getRuntime().exec("vi \"./Data/JESS changes rules.txt\"");
					}
					
					catch (IOException xcp2)
					{						
						JOptionPane.showMessageDialog(null, "There was an exception: " + xcp2.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
					}
				}
				
				JOptionPane.showMessageDialog(null, "There was an exception: " + xcp.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
			}
		}
	}
	
	/**
	 * Launched when the user clicks the OK button. It validates the user input.
	 * @author Piotrek
	 */
	private void OnOKButton()
	{
		// algorithm options
		
		TestOptions.SetUseRulesChoosingMeansOfTransportation(meansOfTransportationRulesCheckBox.isSelected());
		TestOptions.SetUseRulesDroppingChanges(changesRulesCheckBox.isSelected());
		TestOptions.SetLimitAlternativeChangesConnections(departuringConnectionsCheckBox.isSelected());
		TestOptions.SetIgnoreContinentIslandInformation(!geographicInfoCheckBox.isSelected());
		
		// rules
		
		// means of transportation
		switch (meansOfTransportationRulesEngineComboBox.getSelectedIndex())
		{
			case 0:
				
				TestOptions.SetMeansOfTransportationChoosingEngine(TestOptions.MEANS_OF_TRANSPORTATION_CHOOSING_RULES_ENGINE_JESS);
			
				break;
				
			case 1:
				
				TestOptions.SetMeansOfTransportationChoosingEngine(TestOptions.MEANS_OF_TRANSPORTATION_CHOOSING_RULES_ENGINE_JAVA);
			
				break;
				
			default:
				
				throw new RuntimeException("Unexpected means of transportation choosing engine.");
		}		
		
		// changes
		switch (changesRulesEngineComboBox.getSelectedIndex())
		{
			case 0:
				
				TestOptions.SetChangesDroppingEngine(TestOptions.CHANGES_DROPPING_RULES_ENGINE_JESS);
			
				break;
				
			case 1:
				
				TestOptions.SetChangesDroppingEngine(TestOptions.CHANGES_DROPPING_RULES_ENGINE_JAVA);
			
				break;
				
			default:
				
				throw new RuntimeException("Unexpected changes dropping engine.");
		}		
		
		// hiding the dialog		
		dialogResult = OK;
		
		setVisible(false);
	}
	
	/**
	 * Adds a panel with the means of transportation to be chosen as allowed.
	 * @author Piotrek
	 */
	private void AddOptionsPanel()
	{
		JPanel optionsPanel = new JPanel(null);
		
		optionsPanel.setBackground(Color.BLUE);
		optionsPanel.setBounds(10, 10, 510, 250);
		
		add(optionsPanel);
		
		// title label
		JLabel titleLabel = new JLabel("Algorithm Options", SwingConstants.CENTER);
		
		titleLabel.setBounds(10, 10, 490, 25);
		titleLabel.setBackground(Color.ORANGE);
		titleLabel.setForeground(Color.BLACK);
		titleLabel.setOpaque(true);
		
		optionsPanel.add(titleLabel);
		
		// check boxes
		
		// means of transportation
		meansOfTransportationRulesCheckBox = new JCheckBox("Use rules to choose the appropriate means of transportation");
		
		meansOfTransportationRulesCheckBox.setForeground(Color.WHITE);
		meansOfTransportationRulesCheckBox.setBounds(10, 60, 470, 25);
		meansOfTransportationRulesCheckBox.setOpaque(false);
		
		meansOfTransportationRulesCheckBox.setSelected(TestOptions.ShouldUseRulesChoosingMeansOfTransportation());
		
		optionsPanel.add(meansOfTransportationRulesCheckBox);
		
		// changes rules
		changesRulesCheckBox = new JCheckBox("Use rules to choose the appropriate changes");
		
		changesRulesCheckBox.setForeground(Color.WHITE);
		changesRulesCheckBox.setBounds(10, 90, 470, 25);
		changesRulesCheckBox.setOpaque(false);
		
		changesRulesCheckBox.setSelected(TestOptions.ShouldUseRulesDroppingChanges());
		
		optionsPanel.add(changesRulesCheckBox);
		
		// limit departuring conenctions and alternative changes
		departuringConnectionsCheckBox = new JCheckBox("Limit departuring connections or alternative changes");
		
		departuringConnectionsCheckBox.setForeground(Color.WHITE);
		departuringConnectionsCheckBox.setBounds(10, 120, 470, 25);
		departuringConnectionsCheckBox.setOpaque(false);
		
		departuringConnectionsCheckBox.setSelected(TestOptions.ShouldLimitAlternativeChangesConnections());
		
		optionsPanel.add(departuringConnectionsCheckBox);
		
		// make use of extended geographic information
		geographicInfoCheckBox = new JCheckBox("Make use of extended geographic information (continents and islands' names)");
		
		geographicInfoCheckBox.setForeground(Color.WHITE);
		geographicInfoCheckBox.setBounds(10, 150, 470, 25);
		geographicInfoCheckBox.setOpaque(false);
		
		geographicInfoCheckBox.setSelected(!TestOptions.ShouldIgnoreContinentIslandInformation());
		
		optionsPanel.add(geographicInfoCheckBox);
		
		// use jess or java
		
		// means of transportation
		
		JLabel label = new JLabel("Means of transportations choosing rules:", SwingConstants.RIGHT);
		
		label.setForeground(Color.WHITE);
		label.setBounds(10, 185, 235, 25);
		label.setOpaque(false);
		
		optionsPanel.add(label);
		
		meansOfTransportationRulesEngineComboBox = new JComboBox();
		
		meansOfTransportationRulesEngineComboBox.setBounds(250, 185, 250, 25);
		
		meansOfTransportationRulesEngineComboBox.addItem("Use Java Expert System Shell engine.");
		meansOfTransportationRulesEngineComboBox.addItem("Use Java hard coded rules engine.");
		
		optionsPanel.add(meansOfTransportationRulesEngineComboBox);
		
		switch (TestOptions.GetMeansOfTransportationChoosingEngine())
		{
			case TestOptions.MEANS_OF_TRANSPORTATION_CHOOSING_RULES_ENGINE_JESS:
				
				meansOfTransportationRulesEngineComboBox.setSelectedIndex(0);
			
				break;
				
			case TestOptions.MEANS_OF_TRANSPORTATION_CHOOSING_RULES_ENGINE_JAVA:
				
				meansOfTransportationRulesEngineComboBox.setSelectedIndex(1);
			
				break;
				
			default:
				
				throw new RuntimeException("Unexpected means of transportation choosing engine.");
		}
		
		// changes
		
		label = new JLabel("Changes dropping rules:", SwingConstants.RIGHT);
		
		label.setForeground(Color.WHITE);
		label.setBounds(10, 215, 235, 25);
		label.setOpaque(false);
		
		optionsPanel.add(label);
		
		changesRulesEngineComboBox = new JComboBox();
		
		changesRulesEngineComboBox.setBounds(250, 215, 250, 25);
		
		changesRulesEngineComboBox.addItem("Use Java Expert System Shell engine.");
		changesRulesEngineComboBox.addItem("Use Java hard coded rules engine.");
		
		optionsPanel.add(changesRulesEngineComboBox);
		
		switch (TestOptions.GetChangesDroppingEngine())
		{
			case TestOptions.CHANGES_DROPPING_RULES_ENGINE_JESS:
				
				changesRulesEngineComboBox.setSelectedIndex(0);
			
				break;
				
			case TestOptions.CHANGES_DROPPING_RULES_ENGINE_JAVA:
				
				changesRulesEngineComboBox.setSelectedIndex(1);
			
				break;
				
			default:
				
				throw new RuntimeException("Unexpected changes dropping engine.");
		}
	}
	
	/**
	 * A combo box used to choose the engine the means of transportation are choosed with.
	 * @author Piotrek
	 */
	private JComboBox meansOfTransportationRulesEngineComboBox = null;
	
	/**
	 * A combo box used to choose the engine the changes are dropped with.
	 * @author Piotrek
	 */
	private JComboBox changesRulesEngineComboBox = null;
	
	/**
	 * A 'use rules to choose appropriate means of transportation' check box.
	 * @author Piotrek
	 */
	private JCheckBox meansOfTransportationRulesCheckBox = null;
	
	/**
	 * A 'use rules to choose appropriate changes' check box.
	 * @author Piotrek
	 */
	private JCheckBox changesRulesCheckBox = null;
	
	/**
	 * A 'limit departuring connections or alternative changes' check box.
	 * @author Piotrek
	 */
	private JCheckBox departuringConnectionsCheckBox = null;
	
	/**
	 * A 'make use of extented geographic information' check box.
	 * @author Piotrek
	 */
	private JCheckBox geographicInfoCheckBox = null;
	
	/**
	 * A 'edit means of transportation choosing rules' button.
	 * @author Piotrek
	 */
	private JButton editMeansOfTransportationRules = null;
	
	/**
	 * A 'edit change choosing rules' button.
	 * @author Piotrek
	 */
	private JButton editChangeRules = null;
	
	/**
	 * A 'edit CRN settings' button.
	 * @author Piotrek
	 */
	private JButton editCRNSettings = null;
	
	/**
	 * Adds buttons used to edit the JESS rules and configure the wieghts for the CRN.
	 * @author Piotrek
	 */
	private void AddRulesCRNConfigButtons()
	{
		// means of transportation rules
		editMeansOfTransportationRules = new JButton("Edit means of transportation choosing rules");
		
		editMeansOfTransportationRules.setBounds(10, 280, 510, 25);
		editMeansOfTransportationRules.setBackground(Color.ORANGE);
		
		editMeansOfTransportationRules.addActionListener(this);
		
		add(editMeansOfTransportationRules);
		
		// changes rules
		editChangeRules = new JButton("Edit changes choosing rules");
		
		editChangeRules.setBounds(10, 315, 510, 25);
		editChangeRules.setBackground(Color.ORANGE);
		
		editChangeRules.addActionListener(this);
		
		add(editChangeRules);
		
		// CRN settings
		editCRNSettings = new JButton("Edit CRN settings");
		
		editCRNSettings.setBounds(10, 350, 510, 25);
		editCRNSettings.setBackground(Color.ORANGE);
		
		editCRNSettings.addActionListener(this);
		
		add(editCRNSettings);
	}
}
