/*
 * Created on 2005-08-31
 */
package gui.dialogs;

import java.awt.Color;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import world.MeanOfTransportation;
import crn.CRN;
import crn.IEImportance;
import crn.comparators.ChangesCountsComparator;
import crn.comparators.DepartureHoursOfDayComparator;
import crn.comparators.LocationsComparator;
import crn.comparators.RanksComparator;
import crn.comparators.TotalDurationsComparator;

/**
 * A dialog used to edit the Case Retrieval Net settings. They will influence the judging of travel proposals.
 * @author Piotrek
 */
public final class CRNSettingsEditionDialog extends JDialog implements ActionListener
{
	/**
	 * Constructor.
	 * @author Piotrek
	 * @param title The title of the window.
	 */
	public CRNSettingsEditionDialog(String title)
	{
		setTitle(title);
		
		setSize(730, 545);
		setLocation(150, 100);
		
		setModal(true);
		setVisible(false);
		setResizable(false);
		
		// panel	
		panel = getContentPane();
		
		panel.setLayout(null);
		panel.setBackground(Color.BLUE);
		
		setContentPane(panel);
		
		// buttons and other stuff
		
		AddOKCancel();
		
		AddIEsWeightsPanel();
		
		AddOtherSettingsPanel();
	}
	
	/**
	 * Sets the CRN required to edit the importance weights.
	 * @author Piotrek
	 * @param crn The CRN to take weights from or to set the weights to.
	 */
	public void SetCRN(CRN crn)
	{
		if (crn == null)
			throw new NullPointerException("The CRN must not be null.");
		
		this.crn = crn;
				
		// load values
		InitComboBoxes();
	}
	
	/**
	 * Initiates all the combo boxes with the proper values.
	 * @author Piotrek
	 */
	private void InitComboBoxes()
	{
		InitComboBox("UsedMeanOfTransportation", usedMeansOfTransportationComboBox);
		InitComboBox("DepartureHourOfDay", departureHourOfDayComboBox);
		InitComboBox("Origin", originComboBox);
		InitComboBox("Destination", destinationComboBox);
		InitComboBox("FirstStation", firstStationComboBox);
		InitComboBox("LastStation", lastStationComboBox);
		InitComboBox("TotalDuration", totalDurationComboBox);
		InitComboBox("ChangesCount", changesCountComboBox);
		InitComboBox(MeanOfTransportation.GetMeanOfTransportationString(MeanOfTransportation.WALK) + "Rank", walkRankComboBox);
		InitComboBox(MeanOfTransportation.GetMeanOfTransportationString(MeanOfTransportation.TAXI) + "Rank", taxiRankComboBox);
		InitComboBox(MeanOfTransportation.GetMeanOfTransportationString(MeanOfTransportation.BUS) + "Rank", busRankComboBox);
		InitComboBox(MeanOfTransportation.GetMeanOfTransportationString(MeanOfTransportation.TRAIN) + "Rank", trainRankComboBox);
		InitComboBox(MeanOfTransportation.GetMeanOfTransportationString(MeanOfTransportation.AIRPLANE) + "Rank", airplaneRankComboBox);
		InitComboBox(MeanOfTransportation.GetMeanOfTransportationString(MeanOfTransportation.SHIP) + "Rank", shipRankComboBox);
	}
	
	/**
	 * Updates the weights of the IEs in the CRN.
	 * @author Piotrek
	 */
	private void UpdateCRN()
	{		
		UpdateWeight("UsedMeanOfTransportation", usedMeansOfTransportationComboBox);
		UpdateWeight("DepartureHourOfDay", departureHourOfDayComboBox);
		UpdateWeight("Origin", originComboBox);
		UpdateWeight("Destination", destinationComboBox);
		UpdateWeight("FirstStation", firstStationComboBox);
		UpdateWeight("LastStation", lastStationComboBox);
		UpdateWeight("TotalDuration", totalDurationComboBox);
		UpdateWeight("ChangesCount", changesCountComboBox);
		UpdateWeight(MeanOfTransportation.GetMeanOfTransportationString(MeanOfTransportation.WALK) + "Rank", walkRankComboBox);
		UpdateWeight(MeanOfTransportation.GetMeanOfTransportationString(MeanOfTransportation.TAXI) + "Rank", taxiRankComboBox);
		UpdateWeight(MeanOfTransportation.GetMeanOfTransportationString(MeanOfTransportation.BUS) + "Rank", busRankComboBox);
		UpdateWeight(MeanOfTransportation.GetMeanOfTransportationString(MeanOfTransportation.TRAIN) + "Rank", trainRankComboBox);
		UpdateWeight(MeanOfTransportation.GetMeanOfTransportationString(MeanOfTransportation.AIRPLANE) + "Rank", airplaneRankComboBox);
		UpdateWeight(MeanOfTransportation.GetMeanOfTransportationString(MeanOfTransportation.SHIP) + "Rank", shipRankComboBox);
	}
	
	/**
	 * The action listener.
	 * @author Piotrek
	 * @param actionEvent The action event.
	 * @see ActionListener#actionPerformed(ActionEvent)
	 */
	public void actionPerformed(ActionEvent actionEvent)
	{		
		if (actionEvent.getSource() == okButton)
		{
			// validation
			if (!ValidateOtherSettings())
				return;
			
			// updating other settings
			UpdateOtherSettings();
			
			// updating the crn	
			UpdateCRN();
			
			dialogResult = OK;
			
			setVisible(false);
		}
		
		if (actionEvent.getSource() == cancelButton)
		{
			dialogResult = CANCEL;
			
			setVisible(false);
		}
	}
	
	/**
	 * Updates other settings.
	 * @author Piotrek
	 */
	private void UpdateOtherSettings()
	{		
		// max similar changes count difference
		try
		{
			ChangesCountsComparator.MAX_SIMILAR_CHANGES_COUNTS_DIFFERENCE = Integer.parseInt(maxSimilarChangesCountsDifferenceTextField.getText());
		}
		
		catch (NumberFormatException xcp)
		{
			// it is not possible as the data has been validated...
		}
		
		// max similar hours of day difference
		try
		{
			DepartureHoursOfDayComparator.MAX_SIMILAR_HOURS_DIFFERENCE = Integer.parseInt(maxSimilarHoursOfDayDifferenceTextField.getText());
		}
		
		catch (NumberFormatException xcp)
		{
			// it is not possible as the data has been validated...
		}
		
		// max similar locations distance
		try
		{
			LocationsComparator.MAX_SIMILAR_LOCATIONS_DISTANCE = Double.parseDouble(maxSimilarLocationsDistanceTextField.getText());
		}
		
		catch (NumberFormatException xcp)
		{
			// it is not possible as the data has been validated...
		}
		
		// max similar ranks difference
		try
		{
			RanksComparator.MAX_SIMILAR_RANKS_DIFFERENCE = Integer.parseInt(maxSimilarRanksDifferenceTextField.getText());
		}
		
		catch (NumberFormatException xcp)
		{
			// it is not possible as the data has been validated...
		}
		
		// max similar total durations difference
		try
		{
			TotalDurationsComparator.MAX_SIMILAR_DURATIONS_DIFFERENCE = Integer.parseInt(maxSimilarTotalDurationsDifferenceTextField.getText());
		}
		
		catch (NumberFormatException xcp)
		{
			// it is not possible as the data has been validated...
		}
	}
	
	/**
	 * Validates the user input for the other settings.
	 * @author Piotrek
	 * @return A boolean value indicaring whether the user input is OK.
	 */
	private boolean ValidateOtherSettings()
	{
		boolean fOK = true;
		
		String message = "There are some problems:\n";
		
		// max similar changes count difference
		try
		{
			int maxChangesCountDifference = Integer.parseInt(maxSimilarChangesCountsDifferenceTextField.getText());
			
			if (maxChangesCountDifference < 0)
			{
				fOK = false;
				
				message += "- max similar changes counts difference must not be negative\n";
			}
		}
		
		catch (NumberFormatException xcp)
		{
			fOK = false;
			
			message += "- max similar changes counts difference is not a valid integer\n";
		}
		
		// max similar hours of day difference
		try
		{
			int maxHoursOfDayDifference = Integer.parseInt(maxSimilarHoursOfDayDifferenceTextField.getText());
			
			if (maxHoursOfDayDifference < 0)
			{
				fOK = false;
				
				message += "- max similar departure hours of day difference must not be negative\n";
			}
		}
		
		catch (NumberFormatException xcp)
		{
			fOK = false;
			
			message += "- max similar departure hours of day difference is not a valid integer\n";
		}
		
		// max similar locations distance
		try
		{
			double maxLocationsDistance = Double.parseDouble(maxSimilarLocationsDistanceTextField.getText());
			
			if (maxLocationsDistance < 0)
			{
				fOK = false;
				
				message += "- max similar locations distance must not be negative\n";
			}
		}
		
		catch (NumberFormatException xcp)
		{
			fOK = false;
			
			message += "- max similar locations distance is not a valid number\n";
		}
		
		// max similar ranks difference
		try
		{
			int maxRanksDifference = Integer.parseInt(maxSimilarRanksDifferenceTextField.getText());
			
			if (maxRanksDifference < 0)
			{
				fOK = false;
				
				message += "- max similar ranks difference must not be negative\n";
			}
		}
		
		catch (NumberFormatException xcp)
		{
			fOK = false;
			
			message += "- max similar ranks difference is not a valid integer\n";
		}
		
		// max similar total durations difference
		try
		{
			int maxTotalDurationsDifference = Integer.parseInt(maxSimilarTotalDurationsDifferenceTextField.getText());
			
			if (maxTotalDurationsDifference < 0)
			{
				fOK = false;
				
				message += "- max similar total durations difference must not be negative\n";
			}
		}
		
		catch (NumberFormatException xcp)
		{
			fOK = false;
			
			message += "- max similar total duration difference is not a valid integer\n";
		}
		
		if (!fOK)
		{
			JOptionPane.showMessageDialog(null, message, "Error", JOptionPane.ERROR_MESSAGE);
		}
		
		return fOK;
	}
	
	/**
	 * Importance level of the 'UsedMeanOfTransportation' IE.
	 * @author Piotrek
	 */
	private JComboBox usedMeansOfTransportationComboBox = null;
	
	/**
	 * Importance level of the 'DepartureHourOfDay' IE.
	 * @author Piotrek
	 */
	private JComboBox departureHourOfDayComboBox = null;
	
	/**
	 * Importance level of the 'Origin' IE.
	 * @author Piotrek
	 */
	private JComboBox originComboBox = null;
	
	/**
	 * Importance level of the 'Destination' IE.
	 * @author Piotrek
	 */
	private JComboBox destinationComboBox = null;
	
	/**
	 * Importance level of the 'FirstStation' IE.
	 * @author Piotrek
	 */
	private JComboBox firstStationComboBox = null;
	
	/**
	 * Importance level of the 'LastStation' IE.
	 * @author Piotrek
	 */
	private JComboBox lastStationComboBox = null;
	
	/**
	 * Importance level of the 'TotalDuration' IE.
	 * @author Piotrek
	 */
	private JComboBox totalDurationComboBox = null;
	
	/**
	 * Importance level of the 'ChangesCount' IE.
	 * @author Piotrek
	 */
	private JComboBox changesCountComboBox = null;
	
	/**
	 * Importance level of the 'WalkRank' IE.
	 * @author Piotrek
	 */
	private JComboBox walkRankComboBox = null;
	
	/**
	 * Importance level of the 'TaxiRank' IE.
	 * @author Piotrek
	 */
	private JComboBox taxiRankComboBox = null;
	
	/**
	 * Importance level of the 'BusRank' IE.
	 * @author Piotrek
	 */
	private JComboBox busRankComboBox = null;
	
	/**
	 * Importance level of the 'TrainRank' IE.
	 * @author Piotrek
	 */
	private JComboBox trainRankComboBox = null;
	
	/**
	 * Importance level of the 'AirplaneRank' IE.
	 * @author Piotrek
	 */
	private JComboBox airplaneRankComboBox = null;
	
	/**
	 * Importance level of the 'ShipRank' IE.
	 * @author Piotrek
	 */
	private JComboBox shipRankComboBox = null;
	
	/**
	 * The CRN to take weights from or to set the weights to.
	 * @author Piotrek
	 */
	private CRN crn = null;
		
	/**
	 * Inits the given combo box value according to the weight of the IE with the given name.
	 * @author Piotrek
	 * @param ieName The name of the IE.
	 * @param comboBox The combo box to be initiated.
	 */
	protected void InitComboBox(String ieName, JComboBox comboBox)
	{	
		if (crn == null)
			throw new RuntimeException("Set the CRN first.");
		
		float weight = crn.GetIEWeight(ieName);
		
		if (weight == IEImportance.NONE)
			comboBox.setSelectedItem("None");
		else
		if (weight == IEImportance.IRRELEVANT)
			comboBox.setSelectedItem("Irrelevant");
		else
		if (weight == IEImportance.UNIMPORTANT)
			comboBox.setSelectedItem("Unimportant");
		else
		if (weight == IEImportance.NEUTRAL)
			comboBox.setSelectedItem("Neutral");
		else
		if (weight == IEImportance.IMPORTANT)
			comboBox.setSelectedItem("Important");
		else
		if (weight == IEImportance.CRUCIAL)
			comboBox.setSelectedItem("Crucial");
	}
	
	/**
	 * Updates the weight value of the given IE in the CRN according to the value of the given combo box.
	 * @author Piotrek
	 * @param ieName The name of the IE.
	 * @param comboBox The combo box to be initiated.
	 */
	protected void UpdateWeight(String ieName, JComboBox comboBox)
	{	
		if (crn == null)
			throw new RuntimeException("Set the CRN first.");
		
		if (((String)comboBox.getSelectedItem()).compareTo("None") == 0)
			crn.SetIEWeight(ieName, IEImportance.NONE);
		else
		if (((String)comboBox.getSelectedItem()).compareTo("Irrelevant") == 0)
			crn.SetIEWeight(ieName, IEImportance.IRRELEVANT);
		else
		if (((String)comboBox.getSelectedItem()).compareTo("Unimportant") == 0)
			crn.SetIEWeight(ieName, IEImportance.UNIMPORTANT);
		else
		if (((String)comboBox.getSelectedItem()).compareTo("Neutral") == 0)
			crn.SetIEWeight(ieName, IEImportance.NEUTRAL);
		else
		if (((String)comboBox.getSelectedItem()).compareTo("Important") == 0)
			crn.SetIEWeight(ieName, IEImportance.IMPORTANT);
		else
		if (((String)comboBox.getSelectedItem()).compareTo("Crucial") == 0)
			crn.SetIEWeight(ieName, IEImportance.CRUCIAL);
	}
	
	/**
	 * Prepares the panel with the IEs' weights.
	 * @author Piotrek
	 */
	private void AddIEsWeightsPanel()
	{
		JPanel iesWeightsPanel = new JPanel();
		
		iesWeightsPanel.setLayout(null);
	
		iesWeightsPanel.setBounds(10, 10, 320, 455);
		iesWeightsPanel.setBackground(Color.YELLOW);
		
		panel.add(iesWeightsPanel);
		
		// labels
		
		JLabel label = new JLabel("IEs Importance Levels", SwingConstants.CENTER);
		
		label.setForeground(Color.BLUE);
		label.setBackground(Color.ORANGE);
		label.setOpaque(true);
		label.setBounds(10, 7, 300, 20);
		
		iesWeightsPanel.add(label);
		
		// origin
		label = new JLabel("Origin:", SwingConstants.RIGHT);
		
		label.setForeground(Color.BLACK);
		label.setBounds(10, 35, 180, 20);
		
		iesWeightsPanel.add(label);
		
		// destination
		label = new JLabel("Destination:", SwingConstants.RIGHT);
		
		label.setForeground(Color.BLACK);
		label.setBounds(10, 65, 180, 20);
		
		iesWeightsPanel.add(label);
		
		// departure hour of day
		label = new JLabel("Departure hour of day:", SwingConstants.RIGHT);
		
		label.setForeground(Color.BLACK);
		label.setBounds(10, 95, 180, 20);
		
		iesWeightsPanel.add(label);
		
		// first station
		label = new JLabel("First station:", SwingConstants.RIGHT);
		
		label.setForeground(Color.BLACK);
		label.setBounds(10, 125, 180, 20);
		
		iesWeightsPanel.add(label);
		
		// last station
		label = new JLabel("Last station:", SwingConstants.RIGHT);
		
		label.setForeground(Color.BLACK);
		label.setBounds(10, 155, 180, 20);
		
		iesWeightsPanel.add(label);
						
		// total duration
		label = new JLabel("Total duration:", SwingConstants.RIGHT);
		
		label.setForeground(Color.BLACK);
		label.setBounds(10, 185, 180, 20);
		
		iesWeightsPanel.add(label);
					
		// used means of transportation
		label = new JLabel("Used means of transportation:", SwingConstants.RIGHT);
		
		label.setForeground(Color.BLACK);
		label.setBounds(10, 215, 180, 20);
		
		iesWeightsPanel.add(label);
		
		// airplane rank
		label = new JLabel("Airplane rank:", SwingConstants.RIGHT);
		
		label.setForeground(Color.BLACK);
		label.setBounds(10, 245, 180, 20);
		
		iesWeightsPanel.add(label);
				
		// bus rank
		label = new JLabel("Bus rank:", SwingConstants.RIGHT);
		
		label.setForeground(Color.BLACK);
		label.setBounds(10, 275, 180, 20);
		
		iesWeightsPanel.add(label);
		
		// ship rank
		label = new JLabel("Ship rank:", SwingConstants.RIGHT);
		
		label.setForeground(Color.BLACK);
		label.setBounds(10, 305, 180, 20);
		
		iesWeightsPanel.add(label);
		
		// taxi rank
		label = new JLabel("Taxi rank:", SwingConstants.RIGHT);
		
		label.setForeground(Color.BLACK);
		label.setBounds(10, 335, 180, 20);
		
		iesWeightsPanel.add(label);
		
		// train rank
		label = new JLabel("Train rank:", SwingConstants.RIGHT);
		
		label.setForeground(Color.BLACK);
		label.setBounds(10, 365, 180, 20);
		
		iesWeightsPanel.add(label);
		
		// walk rank
		label = new JLabel("Walk rank:", SwingConstants.RIGHT);
		
		label.setForeground(Color.BLACK);
		label.setBounds(10, 395, 180, 20);
		
		iesWeightsPanel.add(label);
		
		// changes count
		label = new JLabel("Changes count:", SwingConstants.RIGHT);
		
		label.setForeground(Color.BLACK);
		label.setBounds(10, 425, 180, 20);
		
		iesWeightsPanel.add(label);
		
		
		// comboboxes
		
		
		// origin
		originComboBox = new JComboBox();
		
		originComboBox.setBounds(200, 35, 110, 20);
		
		iesWeightsPanel.add(originComboBox);
		
		IEImportance.LoadImportanceLevels(originComboBox);
		
		// destination
		
		destinationComboBox = new JComboBox();
		
		destinationComboBox.setBounds(200, 65, 110, 20);
		
		iesWeightsPanel.add(destinationComboBox);
		
		IEImportance.LoadImportanceLevels(destinationComboBox);
		
		// departure hour of day
		
		departureHourOfDayComboBox = new JComboBox();
		
		departureHourOfDayComboBox.setBounds(200, 95, 110, 20);
		
		iesWeightsPanel.add(departureHourOfDayComboBox);
		
		IEImportance.LoadImportanceLevels(departureHourOfDayComboBox);
		
		// first station
		
		firstStationComboBox = new JComboBox();
		
		firstStationComboBox.setBounds(200, 125, 110, 20);
		
		iesWeightsPanel.add(firstStationComboBox);
		
		IEImportance.LoadImportanceLevels(firstStationComboBox);
		
		// last station
		
		lastStationComboBox = new JComboBox();
		
		lastStationComboBox.setBounds(200, 155, 110, 20);
		
		iesWeightsPanel.add(lastStationComboBox);
		
		IEImportance.LoadImportanceLevels(lastStationComboBox);
		
		// total duration		
		totalDurationComboBox = new JComboBox();
		
		totalDurationComboBox.setBounds(200, 185, 110, 20);
		
		iesWeightsPanel.add(totalDurationComboBox);
		
		IEImportance.LoadImportanceLevels(totalDurationComboBox);
		
		// used means of transportation				
		usedMeansOfTransportationComboBox = new JComboBox();
		
		usedMeansOfTransportationComboBox.setBounds(200, 215, 110, 20);
		
		iesWeightsPanel.add(usedMeansOfTransportationComboBox);
		
		IEImportance.LoadImportanceLevels(usedMeansOfTransportationComboBox);
		
		// airplane rank
		airplaneRankComboBox = new JComboBox();
		
		airplaneRankComboBox.setBounds(200, 245, 110, 20);
		
		iesWeightsPanel.add(airplaneRankComboBox);
		
		IEImportance.LoadImportanceLevels(airplaneRankComboBox);
		
		// bus rank
		busRankComboBox = new JComboBox();
		
		busRankComboBox.setBounds(200, 275, 110, 20);
		
		iesWeightsPanel.add(busRankComboBox);
		
		IEImportance.LoadImportanceLevels(busRankComboBox);
		
		// ship rank
		shipRankComboBox = new JComboBox();
		
		shipRankComboBox.setBounds(200, 305, 110, 20);
		
		iesWeightsPanel.add(shipRankComboBox);
		
		IEImportance.LoadImportanceLevels(shipRankComboBox);
		
		// taxi rank
		taxiRankComboBox = new JComboBox();
		
		taxiRankComboBox.setBounds(200, 335, 110, 20);
		
		iesWeightsPanel.add(taxiRankComboBox);
		
		IEImportance.LoadImportanceLevels(taxiRankComboBox);
		
		// train rank
		trainRankComboBox = new JComboBox();
		
		trainRankComboBox.setBounds(200, 365, 110, 20);
		
		iesWeightsPanel.add(trainRankComboBox);
		
		IEImportance.LoadImportanceLevels(trainRankComboBox);
		
		// walk rank
		walkRankComboBox = new JComboBox();
		
		walkRankComboBox.setBounds(200, 395, 110, 20);
		
		iesWeightsPanel.add(walkRankComboBox);
		
		IEImportance.LoadImportanceLevels(walkRankComboBox);
		
		// changes count
		changesCountComboBox = new JComboBox();
		
		changesCountComboBox.setBounds(200, 425, 110, 20);
		
		iesWeightsPanel.add(changesCountComboBox);
		
		IEImportance.LoadImportanceLevels(changesCountComboBox);
		
		
		panel.add(iesWeightsPanel);
	}
	
	/**
	 * A text field used to edit the max similar changes counts difference.
	 * @author Piotrek
	 */
	private JTextField maxSimilarChangesCountsDifferenceTextField = null;
	
	/**
	 * A text field used to edit the max similar hours of day difference.
	 * @author Piotrek
	 */
	private JTextField maxSimilarHoursOfDayDifferenceTextField = null;
	
	/**
	 * A text field used to edit the max similar locations distance.
	 * @author Piotrek
	 */
	private JTextField maxSimilarLocationsDistanceTextField = null;
	
	/**
	 * A text field used to edit the max similar ranks difference.
	 * @author Piotrek
	 */
	private JTextField maxSimilarRanksDifferenceTextField = null;
	
	/**
	 * A text field used to edit the max similar total durations difference.
	 * @author Piotrek
	 */
	private JTextField maxSimilarTotalDurationsDifferenceTextField = null;
	
	/**
	 * Prepares the panel with the other settings.
	 * @author Piotrek
	 */
	private void AddOtherSettingsPanel()
	{
		JPanel otherSettingsPanel = new JPanel();
		
		otherSettingsPanel.setLayout(null);
	
		otherSettingsPanel.setBounds(340, 10, 370, 455);
		otherSettingsPanel.setBackground(Color.YELLOW);
		
		panel.add(otherSettingsPanel);
		
		// labels
		
		JLabel label = new JLabel("Other Settings", SwingConstants.CENTER);
		
		label.setForeground(Color.BLUE);
		label.setBackground(Color.ORANGE);
		label.setOpaque(true);
		label.setBounds(10, 7, 350, 20);
		
		otherSettingsPanel.add(label);
		
		// max similar changes counts difference
		label = new JLabel("Max similar changes counts difference:", SwingConstants.RIGHT);
		
		label.setForeground(Color.BLACK);
		label.setBounds(10, 35, 230, 20);
		
		otherSettingsPanel.add(label);
		
		// max similar hours of day difference 
		label = new JLabel("Max similar hours of day difference:", SwingConstants.RIGHT);
		
		label.setForeground(Color.BLACK);
		label.setBounds(10, 65, 230, 20);
		
		otherSettingsPanel.add(label);
		
		// max similar locations distance
		label = new JLabel("Max similar locations difference:", SwingConstants.RIGHT);
		
		label.setForeground(Color.BLACK);
		label.setBounds(10, 95, 230, 20);
		
		otherSettingsPanel.add(label);
		
		// max similar ranks difference
		label = new JLabel("Max similar ranks difference:", SwingConstants.RIGHT);
		
		label.setForeground(Color.BLACK);
		label.setBounds(10, 125, 230, 20);
		
		otherSettingsPanel.add(label);
		
		// max similar total durations differences
		label = new JLabel("Max similar total durations differences:", SwingConstants.RIGHT);
		
		label.setForeground(Color.BLACK);
		label.setBounds(10, 155, 230, 20);
		
		otherSettingsPanel.add(label);
			
		
		// textfields
		
		
		// max similar changes counts difference
		maxSimilarChangesCountsDifferenceTextField = new JTextField();
		
		maxSimilarChangesCountsDifferenceTextField.setBounds(250, 35, 50, 20);
		
		maxSimilarChangesCountsDifferenceTextField.setText(Integer.toString(ChangesCountsComparator.MAX_SIMILAR_CHANGES_COUNTS_DIFFERENCE));
		
		otherSettingsPanel.add(maxSimilarChangesCountsDifferenceTextField);
		
		// max similar hours of day difference
		maxSimilarHoursOfDayDifferenceTextField = new JTextField();
		
		maxSimilarHoursOfDayDifferenceTextField.setBounds(250, 65, 50, 20);
		
		maxSimilarHoursOfDayDifferenceTextField.setText(Integer.toString(DepartureHoursOfDayComparator.MAX_SIMILAR_HOURS_DIFFERENCE));
		
		otherSettingsPanel.add(maxSimilarHoursOfDayDifferenceTextField);
		
		// max similar locations distance
		maxSimilarLocationsDistanceTextField = new JTextField();
		
		maxSimilarLocationsDistanceTextField.setBounds(250, 95, 50, 20);
		
		maxSimilarLocationsDistanceTextField.setText(Double.toString(LocationsComparator.MAX_SIMILAR_LOCATIONS_DISTANCE));
		
		otherSettingsPanel.add(maxSimilarLocationsDistanceTextField);
		
		// max similar ranks difference
		maxSimilarRanksDifferenceTextField = new JTextField();
		
		maxSimilarRanksDifferenceTextField.setBounds(250, 125, 50, 20);
		
		maxSimilarRanksDifferenceTextField.setText(Integer.toString(RanksComparator.MAX_SIMILAR_RANKS_DIFFERENCE));
		
		otherSettingsPanel.add(maxSimilarRanksDifferenceTextField);
		
		// max similar total durations difference
		maxSimilarTotalDurationsDifferenceTextField = new JTextField();
		
		maxSimilarTotalDurationsDifferenceTextField.setBounds(250, 155, 50, 20);
		
		maxSimilarTotalDurationsDifferenceTextField.setText(Integer.toString(TotalDurationsComparator.MAX_SIMILAR_DURATIONS_DIFFERENCE));
		
		otherSettingsPanel.add(maxSimilarTotalDurationsDifferenceTextField);
		
		
		// unit labels
		
		// max similar changes counts difference
		label = new JLabel("changes", SwingConstants.LEFT);
		
		label.setForeground(Color.BLACK);
		label.setBounds(310, 35, 50, 20);
		
		otherSettingsPanel.add(label);
		
		// max similar hours of day difference 
		label = new JLabel("hours", SwingConstants.LEFT);
		
		label.setForeground(Color.BLACK);
		label.setBounds(310, 65, 50, 20);
		
		otherSettingsPanel.add(label);
		
		// max similar locations distance
		label = new JLabel("units", SwingConstants.LEFT);
		
		label.setForeground(Color.BLACK);
		label.setBounds(310, 95, 50, 20);
		
		otherSettingsPanel.add(label);
		
		// max similar ranks difference
		label = new JLabel("ranks", SwingConstants.LEFT);
		
		label.setForeground(Color.BLACK);
		label.setBounds(310, 125, 50, 20);
		
		otherSettingsPanel.add(label);
		
		// max similar total durations differences
		label = new JLabel("minutes", SwingConstants.LEFT);
		
		label.setForeground(Color.BLACK);
		label.setBounds(310, 155, 50, 20);
		
		otherSettingsPanel.add(label);
		
		
		// help
		
		// label
		label = new JLabel("Help:", SwingConstants.LEFT);
		
		label.setForeground(Color.BLACK);
		label.setBounds(10, 220, 100, 20);
		
		otherSettingsPanel.add(label);
		
		// help itself
		JTextArea help = new JTextArea();
		
		help.setText("\n  All similarity functions map linearly the differences in values\n  from 0 to the above max values to [0; 1] range. All the greater\n  differnces are considered 0 similarity.\n\n  Example:\n\n  Max = 10\n  Difference = 0  =>  Similarity = 1\n  Difference = 5  =>  Similarity = 0.5\n  Difference = 10  =>  Similarity = 0\n  Difference = 11  =>  Similarity = 0");
		help.setBounds(10, 240, 350, 205);
		help.setEditable(false);
		help.setBackground(Color.GRAY);
		help.setForeground(Color.WHITE);
		
		otherSettingsPanel.add(help);
		
		
		panel.add(otherSettingsPanel);
	}
	
	/**
	 * An OK button.
	 * @author Piotrek
	 */
	private JButton okButton = null;
	
	/**
	 * A cancel button.
	 * @author Piotrek
	 */
	private JButton cancelButton = null;
	
	/**
	 * The OK dialog result.
	 * @author Piotrek
	 */
	public static final int OK = 1;
	
	/**
	 * The cancel dialog result.
	 * @author Piotrek
	 */
	public static final int CANCEL = 2;
	
	/**
	 * The dialog result.
	 * @author Piotrek
	 */
	private int dialogResult = CANCEL;
	
	/**
	 * Gets the dialog result.
	 * @author Piotrek
	 * @return The dialog result.
	 */
	public int GetDialogResult()
	{
		return dialogResult;
	}
	
	/**
	 * A main panel of this dialog.
	 * @author Piotrek
	 */
	private Container panel = null;
	
	/**
	 * Adds the OK and Cancel buttons.
	 * @author Piotrek
	 */
	private void AddOKCancel()
	{
		// ok
		
		okButton = new JButton("OK");
		
		okButton.setBounds(10, 475, 155, 25);
		okButton.setBackground(Color.GREEN);
		
		okButton.addActionListener(this);
		
		add(okButton);
		
		// cancel
		
		cancelButton = new JButton("Cancel");
		
		cancelButton.setBounds(175, 475, 155, 25);
		cancelButton.setBackground(Color.GREEN);
		
		cancelButton.addActionListener(this);
		
		add(cancelButton);
	}
}
