/*
 * Created on 2005-03-29
 */
package gui.dialogs;

import gui.FirstStepSolverThread;
import gui.SecondStepSolverThread;
import gui.WorldEditorFileFilter;
import gui.controls.AirportPropertiesPanel;
import gui.controls.BusStopPropertiesPanel;
import gui.controls.CityPropertiesPanel;
import gui.controls.ConnectionEditChooser;
import gui.controls.HarbourPropertiesPanel;
import gui.controls.LocationsChooser;
import gui.controls.StationsChooser;
import gui.controls.TrainStationPropertiesPanel;
import gui.controls.TravelDivisionsChooser;
import gui.controls.TravelSolutionsChooser;
import gui.controls.WorldView;
import java.awt.Color;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.filechooser.FileFilter;
import org.xml.sax.SAXParseException;
import scheduler.TravelConstraints;
import scheduler.TravelSolutionsSorter;
import scheduler.modules.SchedulerCRNManager;
import world.World;
import world.WorldFileParser;
import crn.CRN;

/**
 * A main dialog window of an application.
 * @author Piotrek
 */
public final class MainDialog extends JDialog implements ActionListener
{	
	/**
	 * Constructor.
	 * @author Piotrek
	 * @param title The title of the window.
	 */
	public MainDialog(String title)
	{
		super();
		
		if (title == null)
			throw new NullPointerException("Title must not be null in MainDialog.MainDialog().");
		
		setModal(true);
		setTitle(title);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		setDefaultLookAndFeelDecorated(true);
		
		setSize(1005, 705);
		setResizable(false);
		
		// panel	
		panel = getContentPane();
		
		panel.setLayout(null);
		
		setContentPane(panel);
		
		AddMenuPanel();
		
		AddToolbarPanel();
		
		AddViewPanel();
		
		AddPropertiesPanel();
	}
	
	/**
	 * A main panel of this dialog.
	 * @author Piotrek
	 */
	private Container panel = null;
	
	/**
	 * Adds a menu panel to this dialog.
	 * @author Piotrek
	 */
	private void AddMenuPanel()
	{
		JPanel menuPanel = new JPanel();
		
		menuPanel.setLayout(null);
	
		menuPanel.setBounds(0, 0, 200, 100);
		menuPanel.setBackground(Color.YELLOW);
		
		panel.add(menuPanel);
		
		// new map button
		
		newMapButton = new JButton("New Map");
		
		newMapButton.setBounds(5, 15, 90, 25);
		newMapButton.setBackground(Color.ORANGE);
		newMapButton.addActionListener(this);
		
		menuPanel.add(newMapButton);
		
		// load map button
		
		loadMapButton = new JButton("Load Map");
		
		loadMapButton.setBounds(5, 55, 90, 25);
		loadMapButton.setBackground(Color.ORANGE);
		loadMapButton.addActionListener(this);
		
		menuPanel.add(loadMapButton);
		
		// save map button
		
		saveMapButton = new JButton("Save Map");
		
		saveMapButton.setBounds(105, 15, 90, 25);
		saveMapButton.setBackground(Color.ORANGE);
		saveMapButton.addActionListener(this);
		
		menuPanel.add(saveMapButton);
		
		// exit button
		
		exitButton = new JButton("Exit");
		
		exitButton.setBounds(105, 55, 90, 25);
		exitButton.setBackground(Color.ORANGE);
		exitButton.addActionListener(this);
		
		menuPanel.add(exitButton);
	}
	
	/**
	 * A `new map' button.
	 * @author Piotrek
	 */
	private JButton newMapButton = null;
	
	/**
	 * A `load map' button.
	 * @author Piotrek
	 */
	private JButton loadMapButton = null;
	
	/**
	 * A `save map' button.
	 * @author Piotrek
	 */
	private JButton saveMapButton = null;
	
	/**
	 * A `exit' button.
	 * @author Piotrek
	 */
	private JButton exitButton = null;
	
	/**
	 * Adds a properties panel to this dialog.
	 * @author Piotrek
	 */
	private void AddPropertiesPanel()
	{
		JPanel propertiesPanel = new JPanel();
		
		propertiesPanel.setLayout(null);
	
		propertiesPanel.setBounds(0, 100, 200, 500);
		propertiesPanel.setBackground(Color.LIGHT_GRAY);
		
		panel.add(propertiesPanel);	
		
		// city properties
		cityPropertiesPanel = new CityPropertiesPanel(this);
		
		cityPropertiesPanel.setBounds(10, 10, 180, 235);
		
		propertiesPanel.add(cityPropertiesPanel);
		
		// airport properties
		airportPropertiesPanel = new AirportPropertiesPanel(this);
		
		airportPropertiesPanel.setBounds(10, 10, 180, 235);
		
		propertiesPanel.add(airportPropertiesPanel);
		
		// train station properties
		trainStationPropertiesPanel = new TrainStationPropertiesPanel(this);
		
		trainStationPropertiesPanel.setBounds(10, 10, 180, 235);
		
		propertiesPanel.add(trainStationPropertiesPanel);
		
		// bus stop properties
		busStopPropertiesPanel = new BusStopPropertiesPanel(this);
		
		busStopPropertiesPanel.setBounds(10, 10, 180, 235);
		
		propertiesPanel.add(busStopPropertiesPanel);
		
		// harbour properties
		harbourPropertiesPanel = new HarbourPropertiesPanel(this);
		
		harbourPropertiesPanel.setBounds(10, 10, 180, 235);
		
		propertiesPanel.add(harbourPropertiesPanel);
		
		// stations chooser
		
		stationsChooser = new StationsChooser(worldView, this);
		
		stationsChooser.setBounds(10, 10, 180, 475);
		
		propertiesPanel.add(stationsChooser);
		
		// connections editor
		
		connectionEditChooser = new ConnectionEditChooser(worldView);
		
		connectionEditChooser.setBounds(10, 10, 180, 475);
		
		propertiesPanel.add(connectionEditChooser);
		
		// locations chooser
		locationsChooser = new LocationsChooser(worldView, this);
		
		locationsChooser.setBounds(10, 10, 180, 350);
		
		propertiesPanel.add(locationsChooser);
		
		// travel divisions chooser
		travelDivisionsChooser = new TravelDivisionsChooser(worldView, this);
		
		travelDivisionsChooser.setBounds(10, 10, 180, 475);
		
		propertiesPanel.add(travelDivisionsChooser);
		
		// travel solutions chooser
		travelSolutionsChooser = new TravelSolutionsChooser(worldView, this);
		
		travelSolutionsChooser.setBounds(10, 10, 180, 475);
		
		propertiesPanel.add(travelSolutionsChooser);
	}
	
	/**
	 * A travel divisions chooser panel.
	 * @author Piotrek
	 */
	private TravelDivisionsChooser travelDivisionsChooser = null;
	
	/**
	 * A travel solutions chooser panel.
	 * @author Piotrek
	 */
	private TravelSolutionsChooser travelSolutionsChooser = null;
	
	/**
	 * A locations chooser panel.
	 * @author Piotrek
	 */
	private LocationsChooser locationsChooser = null;
	
	/**
	 * A panel used to choose a connection to be edited or removed.
	 * @author Piotrek
	 */
	private ConnectionEditChooser connectionEditChooser = null;
	
	/**
	 * A panel containing a set of properties of a city.
	 * @author Piotrek
	 */
	private CityPropertiesPanel cityPropertiesPanel = null;
	
	/**
	 * A panel containing a set of properties of an airport.
	 * @author Piotrek
	 */
	private AirportPropertiesPanel airportPropertiesPanel = null;
	
	/**
	 * A panel containing a set of properties of a train station.
	 * @author Piotrek
	 */
	private TrainStationPropertiesPanel trainStationPropertiesPanel = null;
	
	/**
	 * A panel containing a set of properties of a bus stop.
	 * @author Piotrek
	 */
	private BusStopPropertiesPanel busStopPropertiesPanel = null;
	
	/**
	 * A panel containing a set of properties of a harbour.
	 * @author Piotrek
	 */
	private HarbourPropertiesPanel harbourPropertiesPanel = null;
	
	/**
	 * A panel used to build a list of stations when defining a new connection.
	 * @author Piotrek
	 */
	private StationsChooser stationsChooser = null;
	
	/**
	 * Adds a toolbar panel to this dialog.
	 * @author Piotrek
	 */
	private void AddToolbarPanel()
	{
		editionToolbarPanel = new JPanel();
		
		editionToolbarPanel.setLayout(null);
	
		editionToolbarPanel.setBounds(0, 600, 1000, 100);
		editionToolbarPanel.setBackground(Color.LIGHT_GRAY);
		
		panel.add(editionToolbarPanel);	
		
		// adding `add city' button
		addCityButton = new JButton("Add City");
		
		editionToolbarPanel.add(addCityButton);
		
		addCityButton.setBounds(110, 10, 80, 25);
		addCityButton.setBackground(Color.GREEN);
		
		addCityButton.addActionListener(this);
		
		// adding `edit city' button
		editCityButton = new JButton("Edit City");
		
		editionToolbarPanel.add(editCityButton);
		
		editCityButton.setBounds(110, 45, 80, 25);
		editCityButton.setBackground(Color.GREEN);
		
		editCityButton.addActionListener(this);
		
		// adding `zoom in' button
		zoomInButton = new JButton("Zoom In");
		
		editionToolbarPanel.add(zoomInButton);
		
		zoomInButton.setBounds(10, 10, 90, 25);
		zoomInButton.setBackground(Color.GREEN);
		
		zoomInButton.addActionListener(this);
		
		// adding `zoom out' button
		zoomOutButton = new JButton("Zoom Out");
		
		editionToolbarPanel.add(zoomOutButton);
		
		zoomOutButton.setBounds(10, 45, 90, 25);
		zoomOutButton.setBackground(Color.GREEN);
		
		zoomOutButton.addActionListener(this);
		
		// adding `add airport' button
		addAirportButton = new JButton("Add Airport");
		
		editionToolbarPanel.add(addAirportButton);
		
		addAirportButton.setBounds(200, 10, 100, 25);
		addAirportButton.setBackground(Color.GREEN);
		
		addAirportButton.addActionListener(this);
		
		// adding `edit airport' button
		editAirportButton = new JButton("Edit Airport");
		
		editionToolbarPanel.add(editAirportButton);
		
		editAirportButton.setBounds(200, 45, 100, 25);
		editAirportButton.setBackground(Color.GREEN);
		
		editAirportButton.addActionListener(this);
		
		// adding `add train station' button
		addTrainStationButton = new JButton("Add Train Station");
		
		editionToolbarPanel.add(addTrainStationButton);
		
		addTrainStationButton.setBounds(310, 10, 135, 25);
		addTrainStationButton.setBackground(Color.GREEN);
		
		addTrainStationButton.addActionListener(this);
		
		// adding `edit train station' button
		editTrainStationButton = new JButton("Edit Train Station");
		
		editionToolbarPanel.add(editTrainStationButton);
		
		editTrainStationButton.setBounds(310, 45, 135, 25);
		editTrainStationButton.setBackground(Color.GREEN);
		
		editTrainStationButton.addActionListener(this);
		
		// adding `add bus stop' button
		addBusStopButton = new JButton("Add Bus Stop");
		
		editionToolbarPanel.add(addBusStopButton);
		
		addBusStopButton.setBounds(455, 10, 110, 25);
		addBusStopButton.setBackground(Color.GREEN);
		
		addBusStopButton.addActionListener(this);
		
		// adding `edit bus stop' button
		editBusStopButton = new JButton("Edit Bus Stop");
		
		editionToolbarPanel.add(editBusStopButton);
		
		editBusStopButton.setBounds(455, 45, 110, 25);
		editBusStopButton.setBackground(Color.GREEN);
		
		editBusStopButton.addActionListener(this);
		
		// adding `add harbour' button
		addHarbourButton = new JButton("Add Harbour");
		
		editionToolbarPanel.add(addHarbourButton);
		
		addHarbourButton.setBounds(575, 10, 105, 25);
		addHarbourButton.setBackground(Color.GREEN);
		
		addHarbourButton.addActionListener(this);
		
		// adding `edit harbour' button
		editHarbourButton = new JButton("Edit Harbour");
		
		editionToolbarPanel.add(editHarbourButton);
		
		editHarbourButton.setBounds(575, 45, 105, 25);
		editHarbourButton.setBackground(Color.GREEN);
		
		editHarbourButton.addActionListener(this);
				
		// adding `add connection' button
		
		addConnectionButton = new JButton("Add Connection");
		
		editionToolbarPanel.add(addConnectionButton);
		
		addConnectionButton.setBounds(690, 10, 125, 25);
		addConnectionButton.setBackground(Color.GREEN);
		
		addConnectionButton.addActionListener(this);
		
		// adding `edit connection' button
		
		editConnectionButton = new JButton("Edit Connection");
		
		editionToolbarPanel.add(editConnectionButton);
		
		editConnectionButton.setBounds(690, 45, 125, 25);
		editConnectionButton.setBackground(Color.GREEN);
		
		editConnectionButton.addActionListener(this);
				
		// edition panel switch button
		
		editionPanelSwitchButton = new JButton("Show Visibility Options");
		
		editionToolbarPanel.add(editionPanelSwitchButton);
		
		editionPanelSwitchButton.setBounds(825, 10, 165, 25);
		editionPanelSwitchButton.setBackground(Color.GREEN);
		
		editionPanelSwitchButton.addActionListener(this);
		
		// test button
		testButton = new JButton("Test");
		
		editionToolbarPanel.add(testButton);
		
		testButton.setBounds(825, 45, 80, 25);
		testButton.setBackground(Color.GREEN);
		
		testButton.addActionListener(this);
		
		// algorithm configuration
		configButton = new JButton("Config");
		
		editionToolbarPanel.add(configButton);
		
		configButton.setBounds(910, 45, 80, 25);
		configButton.setBackground(Color.GREEN);
		
		configButton.addActionListener(this);
		
		// visibility options toolbar
		
		visibilityToolbarPanel = new JPanel();
		
		visibilityToolbarPanel.setLayout(null);
	
		visibilityToolbarPanel.setBounds(0, 600, 1000, 100);
		visibilityToolbarPanel.setBackground(Color.LIGHT_GRAY);
		visibilityToolbarPanel.setVisible(false);
		
		panel.add(visibilityToolbarPanel);
		
		// adding `show/hide names' button
		showHideNamesButton = new JButton("Hide Names");
		
		visibilityToolbarPanel.add(showHideNamesButton);
		
		showHideNamesButton.setBounds(10, 10, 125, 25);
		showHideNamesButton.setBackground(Color.GREEN);
		
		showHideNamesButton.addActionListener(this);
		
		// adding `show/hide connections' button
		showHideConnectionsButton = new JButton("Hide Connections");
		
		visibilityToolbarPanel.add(showHideConnectionsButton);
		
		showHideConnectionsButton.setBounds(10, 45, 125, 25);
		showHideConnectionsButton.setBackground(Color.GREEN);
		
		showHideConnectionsButton.addActionListener(this);
		
		// visibility panel switch button
		
		visibilityPanelSwitchButton = new JButton("Show Edition Options");
		
		visibilityToolbarPanel.add(visibilityPanelSwitchButton);
		
		visibilityPanelSwitchButton.setBounds(825, 10, 165, 25);
		visibilityPanelSwitchButton.setBackground(Color.GREEN);
		
		visibilityPanelSwitchButton.addActionListener(this);
		
		// adding `show/hide cities' names' button
		showHideCitiesNamesButton = new JButton("Hide Cities' Names");
		
		visibilityToolbarPanel.add(showHideCitiesNamesButton);
		
		showHideCitiesNamesButton.setBounds(825, 45, 165, 25);
		showHideCitiesNamesButton.setBackground(Color.GREEN);
		
		showHideCitiesNamesButton.addActionListener(this);
		
		// adding `show/hide airports' names' button
		showHideAirportsNamesButton = new JButton("Hide Airports' Names");
		
		visibilityToolbarPanel.add(showHideAirportsNamesButton);
		
		showHideAirportsNamesButton.setBounds(145, 10, 150, 25);
		showHideAirportsNamesButton.setBackground(Color.GREEN);
		
		showHideAirportsNamesButton.addActionListener(this);
		
		// adding `show/hide air connections' button
		showHideAirConnectionsButton = new JButton("Hide Air Connections");
		
		visibilityToolbarPanel.add(showHideAirConnectionsButton);
		
		showHideAirConnectionsButton.setBounds(145, 45, 150, 25);
		showHideAirConnectionsButton.setBackground(Color.GREEN);
		
		showHideAirConnectionsButton.addActionListener(this);
		
		// adding `show/hide train stations' names' button
		showHideTrainStationsNamesButton = new JButton("Hide Train Stations' Names");
		
		visibilityToolbarPanel.add(showHideTrainStationsNamesButton);
		
		showHideTrainStationsNamesButton.setBounds(305, 10, 180, 25);
		showHideTrainStationsNamesButton.setBackground(Color.GREEN);
		
		showHideTrainStationsNamesButton.addActionListener(this);
		
		// adding `show/hide railway connections' button
		showHideRailwayConnectionsButton = new JButton("Hide Railway Connections");
		
		visibilityToolbarPanel.add(showHideRailwayConnectionsButton);
		
		showHideRailwayConnectionsButton.setBounds(305, 45, 180, 25);
		showHideRailwayConnectionsButton.setBackground(Color.GREEN);
		
		showHideRailwayConnectionsButton.addActionListener(this);
		
		// adding `show/hide bus stops' names' button
		showHideBusStopsNamesButton = new JButton("Hide Bus Stops' Names");
		
		visibilityToolbarPanel.add(showHideBusStopsNamesButton);
		
		showHideBusStopsNamesButton.setBounds(495, 10, 160, 25);
		showHideBusStopsNamesButton.setBackground(Color.GREEN);
		
		showHideBusStopsNamesButton.addActionListener(this);
		
		// adding `show/hide bus connections' button
		showHideBusConnectionsButton = new JButton("Hide Bus Connections");
		
		visibilityToolbarPanel.add(showHideBusConnectionsButton);
		
		showHideBusConnectionsButton.setBounds(495, 45, 160, 25);
		showHideBusConnectionsButton.setBackground(Color.GREEN);
		
		showHideBusConnectionsButton.addActionListener(this);
		
		// adding `show/hide harbours' names' button
		showHideHarboursNamesButton = new JButton("Hide Harbours' Names");
		
		visibilityToolbarPanel.add(showHideHarboursNamesButton);
		
		showHideHarboursNamesButton.setBounds(665, 10, 150, 25);
		showHideHarboursNamesButton.setBackground(Color.GREEN);
		
		showHideHarboursNamesButton.addActionListener(this);
		
		// adding `show/hide bus connections' button
		showHideSeaConnectionsButton = new JButton("Hide Sea Connections");
		
		visibilityToolbarPanel.add(showHideSeaConnectionsButton);
		
		showHideSeaConnectionsButton.setBounds(665, 45, 150, 25);
		showHideSeaConnectionsButton.setBackground(Color.GREEN);
		
		showHideSeaConnectionsButton.addActionListener(this);
	}
	
	/**
	 * A button used to show/hide the names of the airports.
	 * @author Piotrek
	 */
	private JButton showHideAirportsNamesButton = null;
	
	/**
	 * A button used to show/hide the air connections.
	 * @author Piotrek
	 */
	private JButton showHideAirConnectionsButton = null;
	
	/**
	 * A button used to show/hide the names of the bus stops.
	 * @author Piotrek
	 */
	private JButton showHideBusStopsNamesButton = null;
	
	/**
	 * A button used to show/hide the bus connections.
	 * @author Piotrek
	 */
	private JButton showHideBusConnectionsButton = null;
	
	/**
	 * A button used to show/hide the names of the train stations.
	 * @author Piotrek
	 */
	private JButton showHideTrainStationsNamesButton = null;
	
	/**
	 * A button used to show/hide the railway connections.
	 * @author Piotrek
	 */
	private JButton showHideRailwayConnectionsButton = null;
	
	/**
	 * A button used to show/hide the names of the harbours.
	 * @author Piotrek
	 */
	private JButton showHideHarboursNamesButton = null;
	
	/**
	 * A button used to show/hide the sea connections.
	 * @author Piotrek
	 */
	private JButton showHideSeaConnectionsButton = null;
	
	/**
	 * A button used to show/hide names of the cities.
	 * @author Piotrek
	 */
	private JButton showHideCitiesNamesButton = null;
	
	/**
	 * A panel containing the edition buttons.
	 * @author Piotrek
	 */
	private JPanel editionToolbarPanel = null;
	
	/**
	 * A panel containing the visibility options.
	 * @author Piotrek
	 */
	private JPanel visibilityToolbarPanel = null;
	
	/**
	 * A button used to switch between the edition and visibility toolbar panels.
	 * @author Piotrek
	 */
	private JButton editionPanelSwitchButton = null;
	
	/**
	 * A button used to switch between the visibility and edition toolbar panels.
	 * @author Piotrek
	 */
	private JButton visibilityPanelSwitchButton = null;
	
	/**
	 * The `add city' button placed on the toolbar.
	 * @author Piotrek
	 */
	private JButton addCityButton = null;
	
	/**
	 * The `edit city' button placed on the toolbar.
	 * @author Piotrek
	 */
	private JButton editCityButton = null;
	
	/**
	 * The `zoom in' button placed on the toolbar.
	 * @author Piotrek
	 */
	private JButton zoomInButton = null;
	
	/**
	 * The `zoom out' button placed on the toolbar.
	 * @author Piotrek
	 */
	private JButton zoomOutButton = null;
	
	/**
	 * The `add airport' button placed on the toolbar.
	 * @author Piotrek
	 */
	private JButton addAirportButton = null;
	
	/**
	 * The `edit airport' button placed on the toolbar.
	 * @author Piotrek
	 */
	private JButton editAirportButton = null;
	
	/**
	 * The `add train station' button placed on the toolbar.
	 * @author Piotrek
	 */
	private JButton addTrainStationButton = null;
	
	/**
	 * The `edit train station' button placed on the toolbar.
	 * @author Piotrek
	 */
	private JButton editTrainStationButton = null;
	
	/**
	 * The `add bus stop' button placed on the toolbar.
	 * @author Piotrek
	 */
	private JButton addBusStopButton = null;
	
	/**
	 * The `edit bus stop' button placed on the toolbar.
	 * @author Piotrek
	 */
	private JButton editBusStopButton = null;
	
	/**
	 * The `add harbour' button placed on the toolbar.
	 * @author Piotrek
	 */
	private JButton addHarbourButton = null;
	
	/**
	 * The `edit harbour' button placed on the toolbar.
	 * @author Piotrek
	 */
	private JButton editHarbourButton = null;
	
	/**
	 * The `show/hide names' button placed on the toolbar.
	 * @author Piotrek
	 */
	private JButton showHideNamesButton = null;
	
	/**
	 * The `test' button placed on the toolbar.
	 * @author Piotrek
	 */
	private JButton testButton = null;
	
	/**
	 * The `config' button placed on the toolbar.
	 * @author Piotrek
	 */
	private JButton configButton = null;
	
	/**
	 * The `show/hide connections' button placed on the toolbar.
	 * @author Piotrek
	 */
	private JButton showHideConnectionsButton = null;
	
	/**
	 * The `add connection' button placed on the toolbar.
	 * @author Piotrek
	 */
	private JButton addConnectionButton = null;
	
	/**
	 * The `edit connection' button placed on the toolbar.
	 * @author Piotrek
	 */
	private JButton editConnectionButton = null;
	
	/**
	 * A view panel.
	 * @author Piotrek
	 */
	private WorldView worldView = null;
	
	/**
	 * Adds a view panel to this dialog.
	 * @author Piotrek
	 */
	private void AddViewPanel()
	{
		worldView = new WorldView(this);
						
		JScrollPane pane = new JScrollPane(worldView);
		
		pane.setBounds(200, 0, 800, 600);
		
		worldView.addMouseListener(worldView);
		
		panel.add(pane);
	}
	
	/**
	 * @author Piotrek
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e)
	{
		// city
		
		if (e.getSource() == addCityButton)
		{
			worldView.SetMode(WorldView.MODE_ADD_CITY);
			
			HideAllPropertiesControls();
			
			cityPropertiesPanel.setBounds(10, 10, 180, 235);
			cityPropertiesPanel.setVisible(true);
			
			worldView.editedCity = null;
			
			worldView.repaint();
		}
		
		if (e.getSource() == editCityButton)
		{
			worldView.SetMode(WorldView.MODE_EDIT_CITY);
			
			HideAllPropertiesControls();
			
			worldView.editedCity = null;
			
			worldView.repaint();
		}
		
		if (e.getSource() == cityPropertiesPanel.applyButton)
		{
			synchronized (World.GetInstance())
			{
				cityPropertiesPanel.StoreData(worldView.editedCity);
			}
			
			cityPropertiesPanel.setBounds(10, 10, 180, 235);
			cityPropertiesPanel.setVisible(false);
			
			worldView.editedCity = null;
			
			worldView.SetMode(WorldView.MODE_NORMAL_VIEW);
			
			worldView.repaint();
		}
			
		if (e.getSource() == cityPropertiesPanel.removeButton)
		{
			cityPropertiesPanel.setBounds(10, 10, 180, 235);
			cityPropertiesPanel.setVisible(false);
			
			synchronized (World.GetInstance())
			{
				World.GetInstance().RemoveCity(worldView.editedCity);
			}
			
			worldView.editedCity = null;
			
			worldView.SetMode(WorldView.MODE_NORMAL_VIEW);
			
			worldView.repaint();
		}
		
		// zooming
		
		if (e.getSource() == zoomInButton)
			worldView.ZoomIn();
		
		if (e.getSource() == zoomOutButton)
			worldView.ZoomOut();
		
		// new/save/load/exit
		
		if (e.getSource() == newMapButton)
		{
			synchronized (World.GetInstance())
			{
				World.Clear();
			}
			
			worldView.SetMode(WorldView.MODE_NORMAL_VIEW);
			worldView.repaint();
			
			HideAllPropertiesControls();
		}
		
		if (e.getSource() == loadMapButton)
		{
			OnLoadWorld();
		}
		
		if (e.getSource() == saveMapButton)
		{
			OnSaveWorld();
		}
		
		if (e.getSource() == exitButton)
			dispose();
		
		// airport
		
		if (e.getSource() == addAirportButton)
		{
			worldView.SetMode(WorldView.MODE_ADD_AIRPORT);
			
			HideAllPropertiesControls();
			
			airportPropertiesPanel.setBounds(10, 10, 180, 235);
			airportPropertiesPanel.setVisible(true);
			
			worldView.editedAirport = null;
			
			worldView.repaint();
		}
		
		if (e.getSource() == editAirportButton)
		{
			worldView.SetMode(WorldView.MODE_EDIT_AIRPORT);
			
			HideAllPropertiesControls();
			
			worldView.editedAirport = null;
			
			worldView.repaint();
		}
		
		if (e.getSource() == airportPropertiesPanel.applyButton)
		{
			synchronized (World.GetInstance())
			{
				airportPropertiesPanel.StoreData(worldView.editedAirport);
			}
			
			airportPropertiesPanel.setBounds(10, 10, 180, 235);
			airportPropertiesPanel.setVisible(false);
			
			worldView.editedAirport = null;
			
			worldView.SetMode(WorldView.MODE_NORMAL_VIEW);
			
			worldView.repaint();
		}
			
		if (e.getSource() == airportPropertiesPanel.removeButton)
		{
			airportPropertiesPanel.setBounds(10, 10, 180, 235);
			airportPropertiesPanel.setVisible(false);
			
			synchronized (World.GetInstance())
			{
				World.GetInstance().RemoveAirport(worldView.editedAirport);
			}
			
			worldView.editedAirport = null;
			
			worldView.SetMode(WorldView.MODE_NORMAL_VIEW);
			
			worldView.repaint();
		}
		
		// train station
		
		if (e.getSource() == addTrainStationButton)
		{
			worldView.SetMode(WorldView.MODE_ADD_TRAIN_STATION);
			
			HideAllPropertiesControls();
			
			trainStationPropertiesPanel.setBounds(10, 10, 180, 235);
			trainStationPropertiesPanel.setVisible(true);
			worldView.editedTrainStation = null;
			
			worldView.repaint();
		}
		
		if (e.getSource() == editTrainStationButton)
		{
			worldView.SetMode(WorldView.MODE_EDIT_TRAIN_STATION);
			
			HideAllPropertiesControls();
			
			worldView.editedTrainStation = null;
			
			worldView.repaint();
		}
		
		if (e.getSource() == trainStationPropertiesPanel.applyButton)
		{
			synchronized (World.GetInstance())
			{
				trainStationPropertiesPanel.StoreData(worldView.editedTrainStation);
			}
			
			trainStationPropertiesPanel.setBounds(10, 10, 180, 235);
			trainStationPropertiesPanel.setVisible(false);
			
			worldView.editedTrainStation = null;
			
			worldView.SetMode(WorldView.MODE_NORMAL_VIEW);
			
			worldView.repaint();
		}
			
		if (e.getSource() == trainStationPropertiesPanel.removeButton)
		{
			trainStationPropertiesPanel.setBounds(10, 10, 180, 235);
			trainStationPropertiesPanel.setVisible(false);
			
			synchronized (World.GetInstance())
			{
				World.GetInstance().RemoveTrainStation(worldView.editedTrainStation);
			}
			
			worldView.editedTrainStation = null;
			
			worldView.SetMode(WorldView.MODE_NORMAL_VIEW);
			
			worldView.repaint();
		}
		
		// bus stop
		
		if (e.getSource() == addBusStopButton)
		{
			worldView.SetMode(WorldView.MODE_ADD_BUS_STOP);
			
			HideAllPropertiesControls();
			
			busStopPropertiesPanel.setBounds(10, 10, 180, 235);
			busStopPropertiesPanel.setVisible(true);
			
			worldView.editedBusStop = null;
			
			worldView.repaint();
		}
		
		if (e.getSource() == editBusStopButton)
		{
			worldView.SetMode(WorldView.MODE_EDIT_BUS_STOP);
			
			HideAllPropertiesControls();
			
			worldView.editedBusStop = null;
			
			worldView.repaint();
		}
		
		if (e.getSource() == busStopPropertiesPanel.applyButton)
		{
			synchronized (World.GetInstance())
			{
				busStopPropertiesPanel.StoreData(worldView.editedBusStop);
			}
			
			busStopPropertiesPanel.setBounds(10, 10, 180, 235);
			busStopPropertiesPanel.setVisible(false);
			
			worldView.editedBusStop = null;
			
			worldView.SetMode(WorldView.MODE_NORMAL_VIEW);
			
			worldView.repaint();
		}
			
		if (e.getSource() == busStopPropertiesPanel.removeButton)
		{
			busStopPropertiesPanel.setBounds(10, 10, 180, 235);
			busStopPropertiesPanel.setVisible(false);
			
			synchronized (World.GetInstance())
			{
				World.GetInstance().RemoveBusStop(worldView.editedBusStop);
			}
			
			worldView.editedBusStop = null;
			
			worldView.SetMode(WorldView.MODE_NORMAL_VIEW);
			
			worldView.repaint();
		}
		
		// harbour
		
		if (e.getSource() == addHarbourButton)
		{
			worldView.SetMode(WorldView.MODE_ADD_HARBOUR);
			
			HideAllPropertiesControls();
			
			harbourPropertiesPanel.setBounds(10, 10, 180, 235);
			harbourPropertiesPanel.setVisible(true);
						
			worldView.editedHarbour = null;
			
			worldView.repaint();
		}
		
		if (e.getSource() == editHarbourButton)
		{
			worldView.SetMode(WorldView.MODE_EDIT_HARBOUR);
			
			HideAllPropertiesControls();
			
			worldView.editedHarbour = null;
			
			worldView.repaint();
		}
		
		if (e.getSource() == harbourPropertiesPanel.applyButton)
		{
			synchronized (World.GetInstance())
			{
				harbourPropertiesPanel.StoreData(worldView.editedHarbour);
			}
			
			harbourPropertiesPanel.setBounds(10, 10, 180, 235);
			harbourPropertiesPanel.setVisible(false);
			
			worldView.editedHarbour = null;
			
			worldView.SetMode(WorldView.MODE_NORMAL_VIEW);
			
			worldView.repaint();
		}
			
		if (e.getSource() == harbourPropertiesPanel.removeButton)
		{
			harbourPropertiesPanel.setBounds(10, 10, 180, 235);
			harbourPropertiesPanel.setVisible(false);
			
			synchronized (World.GetInstance())
			{
				World.GetInstance().RemoveHarbour(worldView.editedHarbour);
			}
			
			worldView.editedHarbour = null;
			
			worldView.SetMode(WorldView.MODE_NORMAL_VIEW);
			
			worldView.repaint();
		}
		
		// show/hide names
		
		if (e.getSource() == showHideNamesButton)
		{
			if (worldView.fShowNames)
			{
				worldView.fShowNames = false;
				
				showHideNamesButton.setText("Show Names");
			}
			else
			{
				worldView.fShowNames = true;
				
				showHideNamesButton.setText("Hide Names");
			}
			
			worldView.repaint();
		}
		
		// show/hide cities' names
		
		if (e.getSource() == showHideCitiesNamesButton)
		{
			if (worldView.fShowCitiesNames)
			{
				worldView.fShowCitiesNames = false;
				
				showHideCitiesNamesButton.setText("Show Cities' Names");
			}
			else
			{
				worldView.fShowCitiesNames = true;
				
				showHideCitiesNamesButton.setText("Hide Cities' Names");
			}
			
			worldView.repaint();
		}
		
		// show/hide airports' names
		
		if (e.getSource() == showHideAirportsNamesButton)
		{
			if (worldView.fShowAirportsNames)
			{
				worldView.fShowAirportsNames = false;
				
				showHideAirportsNamesButton.setText("Show Airports' Names");
			}
			else
			{
				worldView.fShowAirportsNames = true;
				
				showHideAirportsNamesButton.setText("Hide Airports' Names");
			}
			
			worldView.repaint();
		}
		
		// show/hide bus stops' names
		
		if (e.getSource() == showHideBusStopsNamesButton)
		{
			if (worldView.fShowBusStopsNames)
			{
				worldView.fShowBusStopsNames = false;
				
				showHideBusStopsNamesButton.setText("Show Bus Stops' Names");
			}
			else
			{
				worldView.fShowBusStopsNames = true;
				
				showHideBusStopsNamesButton.setText("Hide Bus Stops' Names");
			}
			
			worldView.repaint();
		}
		
		// show/hide train stations' names
		
		if (e.getSource() == showHideTrainStationsNamesButton)
		{
			if (worldView.fShowTrainStationsNames)
			{
				worldView.fShowTrainStationsNames = false;
				
				showHideTrainStationsNamesButton.setText("Show Train Stations' Names");
			}
			else
			{
				worldView.fShowTrainStationsNames = true;
				
				showHideTrainStationsNamesButton.setText("Hide Train Stations' Names");
			}
			
			worldView.repaint();
		}
		
		// show/hide harbours' names
		
		if (e.getSource() == showHideHarboursNamesButton)
		{
			if (worldView.fShowHarboursNames)
			{
				worldView.fShowHarboursNames = false;
				
				showHideHarboursNamesButton.setText("Show Harbours' Names");
			}
			else
			{
				worldView.fShowHarboursNames = true;
				
				showHideHarboursNamesButton.setText("Hide Harbours' Names");
			}
			
			worldView.repaint();
		}
				
		// switch toolbar panel buttons
		if (e.getSource() == editionPanelSwitchButton)
		{
			editionToolbarPanel.setVisible(false);
			visibilityToolbarPanel.setVisible(true);
		}
		
		if (e.getSource() == visibilityPanelSwitchButton)
		{
			editionToolbarPanel.setVisible(true);
			visibilityToolbarPanel.setVisible(false);
		}
		
		// show/hide connections
		
		if (e.getSource() == showHideConnectionsButton)
		{
			if (worldView.fShowConnections)
			{
				worldView.fShowConnections = false;
				
				showHideConnectionsButton.setText("Show Conections");
			}
			else
			{
				worldView.fShowConnections = true;
				
				showHideConnectionsButton.setText("Hide Connections");
			}
			
			worldView.repaint();
		}
		
		// show/hide air connections
		
		if (e.getSource() == showHideAirConnectionsButton)
		{
			if (worldView.fShowAirConnections)
			{
				worldView.fShowAirConnections = false;
				
				showHideAirConnectionsButton.setText("Show Air Conections");
			}
			else
			{
				worldView.fShowAirConnections = true;
				
				showHideAirConnectionsButton.setText("Hide Air Connections");
			}
			
			worldView.repaint();
		}
		
		// show/hide railway connections
		
		if (e.getSource() == showHideRailwayConnectionsButton)
		{
			if (worldView.fShowRailwayConnections)
			{
				worldView.fShowRailwayConnections = false;
				
				showHideRailwayConnectionsButton.setText("Show Railway Conections");
			}
			else
			{
				worldView.fShowRailwayConnections = true;
				
				showHideRailwayConnectionsButton.setText("Hide Railway Connections");
			}
			
			worldView.repaint();
		}
		
		// show/hide bus connections
		
		if (e.getSource() == showHideBusConnectionsButton)
		{
			if (worldView.fShowBusConnections)
			{
				worldView.fShowBusConnections = false;
				
				showHideBusConnectionsButton.setText("Show Bus Conections");
			}
			else
			{
				worldView.fShowBusConnections = true;
				
				showHideBusConnectionsButton.setText("Hide Bus Connections");
			}
			
			worldView.repaint();
		}
		
		// show/hide sea connections
		
		if (e.getSource() == showHideSeaConnectionsButton)
		{
			if (worldView.fShowSeaConnections)
			{
				worldView.fShowSeaConnections = false;
				
				showHideSeaConnectionsButton.setText("Show Sea Conections");
			}
			else
			{
				worldView.fShowSeaConnections = true;
				
				showHideSeaConnectionsButton.setText("Hide Sea Connections");
			}
			
			worldView.repaint();
		}
		
		// add connection
		
		if (e.getSource() == addConnectionButton)
		{	
			worldView.repaint();
			
			MeanOfTransportationChooser chooser = new MeanOfTransportationChooser("What type?", "Choose a mean of transportation for a new connection");
			
			chooser.setLocation(200, 200);
			chooser.setVisible(true);
			
			if (chooser.GetDialogResult() == MeanOfTransportationChooser.OK)
			{	
				HideAllPropertiesControls();
								
				stationsChooser.SetStationType(chooser.GetChosenMeanOfTransportation());
								
				worldView.SetMode(WorldView.MODE_CHOOSE_STATION);
				
				worldView.repaint();
				
				stationsChooser.setVisible(true);
			}
		}
		
		// edit connection
		if (e.getSource() == editConnectionButton)
		{
			HideAllPropertiesControls();
			
			connectionEditChooser.LoadConnections();
			connectionEditChooser.setVisible(true);
			
			worldView.SetMode(WorldView.MODE_CONNECTION_EDITION);
			
			worldView.repaint();
		}
		
		// test the algorithm
		if (e.getSource() == testButton)
		{
			worldView.repaint();
			
			HideAllPropertiesControls();
			
			locationsChooser.StartChoosing();
		}
		
		// launch the algorithm configuration dialog
		if (e.getSource() == configButton)
		{
			worldView.repaint();
			
			HideAllPropertiesControls();
			
			AlgorithmConfigurationDialog configurationDialog = new AlgorithmConfigurationDialog(this);
			
			configurationDialog.setLocation(200, 150);
			
			configurationDialog.setVisible(true);
		}
		
		// launch the second step of the algorithm
		if (e.getSource() == travelDivisionsChooser.GetLaunchSecondStepButton())
		{
			worldView.repaint();
						
			HideAllPropertiesControls();
		
			// it will display the `please wait message'
			locationsChooser.setVisible(true);
			
			LockButtons(true);
			
			SecondStepSolverThread worker = new SecondStepSolverThread(this, firstStepSolutions, travelConstraints);
			
			worker.start();
		}
		
		// stop browsing of the travel solutions
		if (e.getSource() == travelSolutionsChooser.GetCloseButton())
		{						
			HideAllPropertiesControls();
				
			LockButtons(false);
			
			worldView.repaint();
		}
		
		// sort travel solutions
		if (e.getSource() == travelSolutionsChooser.GetSortButton())
		{
			SortPropertyChooser sortPropertyChooser = new SortPropertyChooser();
			
			sortPropertyChooser.setLocation(300, 100);
			
			sortPropertyChooser.setVisible(true);
			
			if (sortPropertyChooser.GetDialogResult() == SortPropertyChooser.OK)
			{
				int sortProperty = sortPropertyChooser.GetChosenSortProperty();
				
				// applying the sorting using a comparator based on the choice in the previous dialog
				ArrayList currentSolutionsList = travelSolutionsChooser.GetSolutionsList();
				
				TravelSolutionsSorter sorter = new TravelSolutionsSorter(crnManager);
							
				travelSolutionsChooser.LoadSolutions(sorter.SortTravelSolutions(currentSolutionsList, sortProperty));
			}
		}
		
		// accept the selected travel solution
		if (e.getSource() == travelSolutionsChooser.GetAcceptButton())
		{
			// adding the accepted solution to the CRN
			crnManager.AddTravelSolution(travelSolutionsChooser.GetSelectedSolution(), true);
			
			HideAllPropertiesControls();
			
			LockButtons(false);
			
			worldView.repaint();
		}
		
		// reject the selected travel solution
		if (e.getSource() == travelSolutionsChooser.GetRejectButton())
		{
			crnManager.AddTravelSolution(travelSolutionsChooser.GetSelectedSolution(), false);
			
			travelSolutionsChooser.RemoveSelectedTravelSolution();
		}
	}
	
	/**
	 * An instance of CRN manager used to take care of history handling.
	 * @author Piotrek
	 */
	private SchedulerCRNManager crnManager = new SchedulerCRNManager("./Data/History.xml");
	
	/**
	 * Gets the CRN used by this CRN manager.
	 * @author Piotrek
	 * @return The requested CRN.
	 */
	public CRN GetCRN()
	{
		return crnManager.GetCRN();
	}
	
	/**
	 * Saves a world to the file given by the user.
	 * @author Piotrek
	 */
	private void OnSaveWorld()
	{
		JFileChooser chooser = new JFileChooser("./Worlds/");
		
		FileFilter filter = new WorldEditorFileFilter();
		
		chooser.setFileFilter(filter);
		chooser.setMultiSelectionEnabled(false);
		chooser.setDialogTitle("Enter a name for a world file");
					
		if (chooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) 
		{
		    String fileName = chooser.getSelectedFile().getPath();
		   
		    if (!fileName.endsWith(WorldEditorFileFilter.FILE_EXTENSION))
		      fileName += WorldEditorFileFilter.FILE_EXTENSION;
		    
		    synchronized (World.GetInstance())
		    {
		 		World.GetInstance().SaveAs(fileName);
		    }
		}
	}
	
	/**
	 * Loads a world from the file given by the user.
	 * @author Piotrek
	 */
	private void OnLoadWorld()
	{
		JFileChooser chooser = new JFileChooser("./Worlds/");
		
		FileFilter filter = new WorldEditorFileFilter();
		
		chooser.setFileFilter(filter);
		chooser.setMultiSelectionEnabled(false);
		chooser.setDialogTitle("Enter a name of a world file");
					
		if (chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) 
		{
			String fileName = chooser.getSelectedFile().getPath();
   
			WorldFileParser parser = new WorldFileParser();
   
			try
			{
				synchronized (World.GetInstance())
				{
					parser.Parse(fileName);
					
					worldView.SetMode(WorldView.MODE_NORMAL_VIEW);
					worldView.repaint();
					
					HideAllPropertiesControls();
				}
			}
			
			catch (SAXParseException xcp)
			{
				JOptionPane.showMessageDialog(null, "There was an error while parsing the world file in the line " + xcp.getLineNumber() + ": " + xcp.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
			}
			   
		    catch (Exception xcp)
		    {
		    	JOptionPane.showMessageDialog(null, "There was an error while parsing the world file: " + xcp.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
		    }
		    
		    worldView.repaint();
		}
	}
	
	/**
	 * Hides all controls that are displayed in the properites panel.
	 * @author Piotrek
	 */
	private void HideAllPropertiesControls()
	{
		busStopPropertiesPanel.setVisible(false);
		trainStationPropertiesPanel.setVisible(false);
		cityPropertiesPanel.setVisible(false);
		airportPropertiesPanel.setVisible(false);
		harbourPropertiesPanel.setVisible(false);
		stationsChooser.setVisible(false);
		connectionEditChooser.setVisible(false);
		locationsChooser.setVisible(false);
		travelDivisionsChooser.setVisible(false);
		travelSolutionsChooser.setVisible(false);
		
		// canceling the marking of the edited objects in the view
		worldView.CancelEditionMarkings();
	}
	
	/**
	 * Hides the stations chooser control and switches the view into the normal view mode. 
	 * It is called by a timetable editor when it has finished edition of a new connection. 
	 * @author Piotrek
	 */
	public void HideStationsChooser()
	{
		stationsChooser.setVisible(false);
		
		worldView.SetMode(WorldView.MODE_NORMAL_VIEW);
	}
	
	/**
	 * Gets the city properties panel.
	 * @author Piotrek
	 * @return The requested panel.
	 */
	public CityPropertiesPanel GetCityPropertiesPanel()
	{
		return cityPropertiesPanel;
	}
	
	/**
	 * Gets the airport properties panel.
	 * @author Piotrek
	 * @return The requested panel.
	 */
	public AirportPropertiesPanel GetAirportPropertiesPanel()
	{
		return airportPropertiesPanel;
	}
	
	/**
	 * Gets the train station properties panel.
	 * @author Piotrek
	 * @return The requested panel.
	 */
	public TrainStationPropertiesPanel GetTrainStationPropertiesPanel()
	{
		return trainStationPropertiesPanel;
	}
	
	/**
	 * Gets the bus stop properties panel.
	 * @author Piotrek
	 * @return The requested panel.
	 */
	public BusStopPropertiesPanel GetBusStopPropertiesPanel()
	{
		return busStopPropertiesPanel;
	}
	
	/**
	 * Gets the harbour properties panel.
	 * @author Piotrek
	 * @return The requested panel.
	 */
	public HarbourPropertiesPanel GetHarbourPropertiesPanel()
	{
		return harbourPropertiesPanel;
	}
	
	/**
	 * Locks all the buttons but the zoom in/out, exit and show/hide buttons.
	 * @author Piotrek
	 * @param fLock Indicates whether to lock the buttons.
	 */
	private void LockButtons(boolean fLock)
	{
		addAirportButton.setEnabled(!fLock);
		editAirportButton.setEnabled(!fLock);
		addBusStopButton.setEnabled(!fLock);
		editBusStopButton.setEnabled(!fLock);
		addCityButton.setEnabled(!fLock);
		editCityButton.setEnabled(!fLock);
		addTrainStationButton.setEnabled(!fLock);
		editTrainStationButton.setEnabled(!fLock);
		addHarbourButton.setEnabled(!fLock);
		editHarbourButton.setEnabled(!fLock);
		addConnectionButton.setEnabled(!fLock);
		editConnectionButton.setEnabled(!fLock);
		testButton.setEnabled(!fLock);
		configButton.setEnabled(!fLock);
		
		newMapButton.setEnabled(!fLock);
		loadMapButton.setEnabled(!fLock);
		saveMapButton.setEnabled(!fLock);
	}
	
	/**
	 * Runs a solver of the first step of the travel scheduling algorithm.
	 * @author Piotrek
	 */
	public void LaunchFirstStepSolver()
	{
		travelConstraints = null;
		
		// showing the options dialog (times and some other extra parameters)
		TravelDetailsQueryDialog detailsDialog = new TravelDetailsQueryDialog();
		
		detailsDialog.setLocation(100, 100);
		
		detailsDialog.setVisible(true);
				
		if (detailsDialog.GetDialogResult() == TravelDetailsQueryDialog.OK)
		{
			travelConstraints = detailsDialog.GetTravelConstraints();
			
			LockButtons(true);
		
			Thread thread = new FirstStepSolverThread(this, travelConstraints);
		
			thread.start();
		}
		else
			HideAllPropertiesControls();
	}
	
	/**
	 * The current travek constraints.
	 * @author Piotrek
	 */
	private TravelConstraints travelConstraints = null;
	
	/**
	 * Called once the first step of the travel scheduling algorithm is solved.
	 * @author Piotrek
	 * @param firstStepSolutions A set of first step solutions.
	 */
	public void OnFirstStepReady(ArrayList firstStepSolutions)
	{
		if (firstStepSolutions == null)
			throw new NullPointerException("First step solutions must not be null in MainDialog.OnFirstStepReady().");
		
		locationsChooser.setVisible(false);
		
		LockButtons(false);
		
		travelDivisionsChooser.setVisible(true);
		
		travelDivisionsChooser.LoadDivisions(firstStepSolutions);
		
		this.firstStepSolutions = firstStepSolutions;
		
		worldView.repaint();
	}
	
	/**
	 * A list of the solutions found by the first step of the travel scheduling algorithm.
	 * @author Piotrek
	 */
	private ArrayList firstStepSolutions = null;
	
	/**
	 * Called once the second step of the travel scheduling algorithm is solved.
	 * @author Piotrek
	 * @param secondStepSolutions A set of second step solutions.
	 */
	public void OnSecondStepReady(ArrayList secondStepSolutions)
	{
		if (secondStepSolutions == null)
			throw new NullPointerException("Second step solutions must not be null in MainDialog.OnSecondStepReady().");
		
		// it was showing the `please wait' message
		locationsChooser.setVisible(false);
				
		TravelSolutionsSorter sorter = new TravelSolutionsSorter(crnManager);
		
		secondStepSolutions = sorter.SortTravelSolutions(secondStepSolutions, TravelSolutionsSorter.SORT_ON_SIMILARITIES_TO_HISTORY);
				
		travelSolutionsChooser.LoadSolutions(secondStepSolutions);
		
		travelSolutionsChooser.setVisible(true);
		
		// unlocking the buttons
		LockButtons(false);
		
		worldView.repaint();
	}
	
	/**
	 * Called if there was an error while solving the second step of the travel scheduling algorithm. It shows a message box to the user.
	 * @author Piotrek
	 * @param errorMessage A message shown to the user.
	 */
	public void OnSchedulingAlgorithmStepError(String errorMessage)
	{
		if (errorMessage == null)
			throw new NullPointerException("Error message must not be null in MainDialog.OnSchedulingAlgorithmStepError().");
		
		// it was showing the `please wait' message
		locationsChooser.setVisible(false);
		
		// unlocking the buttons
		LockButtons(false);
						
		worldView.repaint();
		
		// showing the massage box
		JOptionPane.showMessageDialog(null, "There was an exception while solving the step of the scheduling algorithm:\n" + errorMessage, "Error", JOptionPane.ERROR_MESSAGE);
	}
	
	/**
	 * Gets the locations chooser. It can be used to get the origin and destination for the test.
	 * @author Piotrek
	 * @return A currently used locations chooser.
	 */
	public LocationsChooser GetLocationsChooser()
	{
		return locationsChooser;
	}
		
	/**
	 * @see java.awt.Window#dispose()
	 * @author Piotrek
	 */
	public void dispose()
	{
		crnManager.SaveToFile();
		
		super.dispose();
	}
}
