/*
 * Created on 2005-04-05
 */
package gui.dialogs;

import java.awt.Color;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JRadioButton;
import javax.swing.SwingConstants;
import world.MeanOfTransportation;

/**
 * A dialog used to choose one of the means of transportation.
 * @author Piotrek
 */
public final class MeanOfTransportationChooser extends JDialog implements ActionListener
{
	/**
	 * Constructor.
	 * @author Piotrek
	 * @param title A title of the dialog window.
	 * @param message A message that appears at the top of the dialog.
	 */
	public MeanOfTransportationChooser(String title, String message)
	{
		super();
		
		if (title == null)
			throw new NullPointerException("Title must not be null in MeanOfTransportationChooser.MeanOfTransportationChooser().");
		
		if (message == null)
			throw new NullPointerException("Message must not be null in MeanOfTransportationChooser.MeanOfTransportationChooser().");
		
		setModal(true);
		setTitle(title);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		setSize(405, 300);
		setResizable(false);
		
		// panel	
		panel = getContentPane();
		
		panel.setLayout(null);
		
		setContentPane(panel);
		
		// adding a message
		
		JLabel label = new JLabel(message, SwingConstants.CENTER);
		
		add(label);
		
		label.setBounds(10, 25, 380, 25);
		label.setBackground(Color.YELLOW);
		label.setOpaque(true);
		
		// adding radio buttons
		
		// airplane
		
		airplaneOption = new JRadioButton("Airplane", true);
		
		airplaneOption.setBounds(120, 80, 160, 25);
		airplaneOption.addActionListener(this);
		
		add(airplaneOption);
		
		// train
		
		trainOption = new JRadioButton("Train", false);
		
		trainOption.setBounds(120, 115, 160, 25);
		trainOption.addActionListener(this);
	
		add(trainOption);
		
		// bus
		
		busOption = new JRadioButton("Bus", false);
		
		busOption.setBounds(120, 150, 160, 25);
		busOption.addActionListener(this);
		
		add(busOption);
		
		// ship
		
		shipOption = new JRadioButton("Ship", false);
		
		shipOption.setBounds(120, 185, 160, 25);
		shipOption.addActionListener(this);
		
		add(shipOption);
		
		// grouping radio buttons
		
		ButtonGroup buttonGroup = new ButtonGroup();
		
		buttonGroup.add(airplaneOption);
		buttonGroup.add(trainOption);
		buttonGroup.add(busOption);
		buttonGroup.add(shipOption);
		
		// ok and cancel buttons
		
		// ok
		
		okButton = new JButton("OK");
		
		okButton.setBounds(110, 230, 80, 25);
		okButton.setBackground(Color.ORANGE);
		
		okButton.addActionListener(this);
		
		add(okButton);
		
		// cancel
		
		cancelButton = new JButton("Cancel");
		
		cancelButton.setBounds(210, 230, 80, 25);
		cancelButton.setBackground(Color.ORANGE);
		
		cancelButton.addActionListener(this);
		
		add(cancelButton);
	}
	
	/**
	 * An OK button.
	 * @author Piotrek
	 */
	JButton okButton = null;
	
	/**
	 * A cancel button.
	 * @author Piotrek
	 */
	JButton cancelButton = null;
	
	/**
	 * The OK dialog result.
	 * @author Piotrek
	 */
	public static final int OK = 1;
	
	/**
	 * The cancel dialog result.
	 * @author Piotrek
	 */
	public static final int CANCEL = 2;
	
	/**
	 * The dialog result.
	 * @author Piotrek
	 */
	private int dialogResult = CANCEL;
	
	/**
	 * Gets the dialog result.
	 * @author Piotrek
	 * @return The dialog result.
	 */
	public int GetDialogResult()
	{
		return dialogResult;
	}
	
	/**
	 * A radio button with the airplane.
	 * @author Piotrek
	 */
	JRadioButton airplaneOption = null;
	
	/**
	 * A radio button with the train.
	 * @author Piotrek
	 */
	JRadioButton trainOption = null;
	
	/**
	 * A radio button with the bus.
	 * @author Piotrek
	 */
	JRadioButton busOption = null;
	
	/**
	 * A radio button with the ship.
	 * @author Piotrek
	 */
	JRadioButton shipOption = null;
	
	/**
	 * A main panel of this dialog.
	 * @author Piotrek
	 */
	private Container panel = null;
		
	/**
	 * A mean of transportation chosen by the user.
	 * @author Piotrek
	 */
	private int chosenMeanOfTransportation = MeanOfTransportation.AIRPLANE;
	
	/**
	 * Gets a mean of transportation chosen by the user.
	 * @author Piotrek
	 * @return A chosen mean of transportation.
	 */
	public int GetChosenMeanOfTransportation()
	{
		return chosenMeanOfTransportation;
	}
	
	/**
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 * @author Piotrek
	 */
	public void actionPerformed(ActionEvent e)
	{
		if (e.getSource() == okButton)
		{
			setVisible(false);
			
			dialogResult = OK;
		}
		
		if (e.getSource() == cancelButton)
		{
			setVisible(false);
			
			dialogResult = CANCEL;
		}
		
		if (e.getSource() == airplaneOption)
			chosenMeanOfTransportation = MeanOfTransportation.AIRPLANE;
		
		if (e.getSource() == trainOption)
			chosenMeanOfTransportation = MeanOfTransportation.TRAIN;
		
		if (e.getSource() == busOption)
			chosenMeanOfTransportation = MeanOfTransportation.BUS;
			
		if (e.getSource() == shipOption)
			chosenMeanOfTransportation = MeanOfTransportation.SHIP;
	}
}
