package jess;

import java.util.*;
/**
 * The root of the Rete network.
 * (C) 2003 Sandia National Laboratories
 * $Id: RootNode.java,v 1.2.2.2 2004/05/04 23:58:26 ejfried Exp $
 */

class RootNode extends Node1 {
    private HashMap m_map = new HashMap();
    void callNodeRight(Token t, Context context) throws JessException {
        passAlong(t, context);
    }

    public Enumeration getSuccessors() {
        return Collections.enumeration(m_map.values());
    }

    void passAlong(Token t, Context context) throws JessException {
        if (t.m_tag == RU.CLEAR) {
            for (Iterator it = m_map.values().iterator();it.hasNext();)
                ((Node) it.next()).callNodeRight(t, context);
            return;
        }

        Fact fact = t.fact(0);
        Deftemplate template = fact.getDeftemplate();
        while (template != null) {
            String name = template.getName();
            Node node = (Node) m_map.get(name);
            if (node != null) {
                node.callNodeRight(t, context);
            }
            template = template.getParent();
        }
    }

    Node mergeSuccessor(Node n, NodeSink r)
            throws JessException {
        if (n instanceof Node1TECT) {
            Node1TECT tect = (Node1TECT) n;
            Node old;
            if ((old = (Node) m_map.get(tect.getName())) != null) {
                r.addNode(old);
                return old;
            }
        }

        // No match found
        addSuccessor(n, r);
        return n;
    }


    void addSuccessor(Node n, NodeSink r)
            throws JessException {
        if (n instanceof Node1TECT) {
            Node1TECT tect = (Node1TECT) n;
            String name = tect.getName();
            m_map.put(name, tect);
            r.addNode(n);
        }
    }

    void removeSuccessor(Node n) {
        if (n instanceof Node1TECT) {
            Node1TECT tect = (Node1TECT) n;
            String name = tect.getName();
            m_map.remove(name);
        }
    }


    public String toString() {
        return "The root of the Rete network";
    }
}
