/*
 * Created on 2005-05-23
 */
package scheduler.modules;

import java.util.Stack;
import scheduler.ConnectionsFinderTask;

/**
 * A manager used to store the tasks for the process of finding connections for the single travel part.
 * @author Piotrek
 */
public final class ConnectionsFinderTasksManager implements scheduler.modulesinterfaces.ConnectionsFinderTasksManager
{	
	/**
	 * Constructor.
	 * @author Piotrek
	 */
	public ConnectionsFinderTasksManager()
	{
		// does nothing
	}
	
	/**
	 * @author Piotrek
	 * @see scheduler.modulesinterfaces.ConnectionsFinderTasksManager#AddTask(ConnectionsFinderTask)
	 */
	public void AddTask(ConnectionsFinderTask task)
	{
		if (task == null)
			throw new NullPointerException("Task must not be null in the ConnectionsFinderTasksManager.AddTask().");
		
		stack.push(task);
	}
	
	/**
	 * @author Piotrek
	 * @see scheduler.modulesinterfaces.ConnectionsFinderTasksManager#GetTask()
	 */
	public ConnectionsFinderTask GetTask()
	{
		if (stack.size() > 0)
			return (ConnectionsFinderTask)stack.pop();
		
		return null;
	}

	/**
	 * @author Piotrek
	 * @see scheduler.modulesinterfaces.ConnectionsFinderTasksManager#Clear()
	 */
	public void Clear()
	{
		stack.clear();
	}
	
	/**
	 * A stack used to store the tasks.
	 * @author Piotrek
	 */
	private Stack stack = new Stack();
	
	/**
	 * Gets a number of tasks stored in the manager.
	 * @author Piotrek
	 * @return A number of tasks stored in the manager.
	 */
	public int GetTasksCount()
	{
		return stack.size();
	}
}
