/*
 * Created on 2005-05-01
 */
package scheduler.modulesinterfaces;

import java.util.ArrayList;

/**
 * An interface defining the methods to access a source of the travel information on the world.
 * @author Piotrek
 */
public interface LocationsTravelInfoProvider
{
	/**
	 * Gets all the airports located in the specified rectangle.
	 * @author Piotrek
	 * @param minX The minimal X coordinate of the rectangle.
	 * @param minY The minimal Y coordinate of the rectangle.
	 * @param maxX The maximal X coordinate of the rectangle.
	 * @param maxY The maximal Y coordinate of the rectangle.
	 * @return A list of airports. If there are none, the empty list should be returned.
	 */
	public ArrayList GetAirportsInRange(int minX, int minY, int maxX, int maxY);
	
	/**
	 * Gets all the train stations located in the specified rectangle.
	 * @author Piotrek
	 * @param minX The minimal X coordinate of the rectangle.
	 * @param minY The minimal Y coordinate of the rectangle.
	 * @param maxX The maximal X coordinate of the rectangle.
	 * @param maxY The maximal Y coordinate of the rectangle.
	 * @return A list of train stations. If there are none, the empty list should be returned.
	 */
	public ArrayList GetTrainStationsInRange(int minX, int minY, int maxX, int maxY);
	
	/**
	 * Gets all the bus stops located in the specified rectangle.
	 * @author Piotrek
	 * @param minX The minimal X coordinate of the rectangle.
	 * @param minY The minimal Y coordinate of the rectangle.
	 * @param maxX The maximal X coordinate of the rectangle.
	 * @param maxY The maximal Y coordinate of the rectangle.
	 * @return A list of bus stops. If there are none, the empty list should be returned.
	 */
	public ArrayList GetBusStopsInRange(int minX, int minY, int maxX, int maxY);
	
	/**
	 * Gets all the harbours located in the specified rectangle.
	 * @author Piotrek
	 * @param minX The minimal X coordinate of the rectangle.
	 * @param minY The minimal Y coordinate of the rectangle.
	 * @param maxX The maximal X coordinate of the rectangle.
	 * @param maxY The maximal Y coordinate of the rectangle.
	 * @return A list of harbours. If there are none, the empty list should be returned.
	 */
	public ArrayList GetHarboursInRange(int minX, int minY, int maxX, int maxY);
}
