/*
 * Created on 2005-08-29
 */
package scheduler.modulesinterfaces;

import java.util.ArrayList;
import world.Location;

/**
 * An interface that must be implemented by a class that will provide a list of means of transportation appropriate for the main part of travek between the two given locations.
 * @author Piotrek
 */
public interface MeansOfTransportationProvider
{
	/**
	 * Gets a list of means of transportation appropriate for the main part of travek between the two given locations.
	 * @author Piotrek
	 * @param origin An origin of the travel.
	 * @param destination A destination of the travel.
	 * @return A list of means of tranportation. A list must contain instances of Integers set to the one of the values assigned to the means of tranportation.
	 * @see world.MeanOfTransportation#AIRPLANE
	 * @see world.MeanOfTransportation#BUS
	 * @see world.MeanOfTransportation#SHIP
	 * @see world.MeanOfTransportation#TAXI
	 * @see world.MeanOfTransportation#TRAIN
	 * @see world.MeanOfTransportation#WALK
	 */
	public ArrayList GetMeansOfTransportation(Location origin, Location destination);
		
	/**
	 * The minimal distance to be taken by the airplane.
	 * @author Piotrek
	 */
	public static final double MIN_FLIGHT_DISTANCE = 50.0;
	
	/**
	 * The minimal distance to be taken by the ship.
	 * @author Piotrek
	 */
	public static final double MIN_CRUISE_DISTANCE = 5.0;
	
	/**
	 * The minimal distance to be taken by the bus.
	 * @author Piotrek
	 */
	public static final double MIN_BUS_DISTANCE = 1.0;
	
	/**
	 * The minimal distance to be taken by the train.
	 * @author Piotrek
	 */
	public static final double MIN_TRAIN_DISTANCE = 2.0;
	
	/**
	 * The maximal distance to be taken by walk.
	 * @author Piotrek
	 */
	public static final double MAX_WALK_DISTANCE = 1.0;
	
	/**
	 * The minimal distance to be taken by the taxi.
	 * @author Piotrek
	 */
	public static final double MIN_TAXI_DISTANCE = 1.0;
	
	/**
	 * The maximal distance to be taken by the taxi.
	 * @author Piotrek
	 */
	public static final double MAX_TAXI_DISTANCE = 10.0;
}
