/*
 * Created on 2005-05-31
 */
package tests;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import junit.framework.TestCase;
import scheduler.ConnectionsFinderTask;
import scheduler.TravelPartConnectionChunk;
import world.Airport;
import world.Connection;
import world.Location;
import world.MeanOfTransportation;
import world.Station;
import world.TravelTime;

/**
 * Tests the ConnectionsFinderTask class.
 * @author Piotrek
 */
public class ConnectionsFinderTaskTest extends TestCase
{
	/**
	 * Tests the constructor.
	 * @author Piotrek
	 */
	public void testConnectionsFinderTask()
	{
		try
		{
			new ConnectionsFinderTask();
		}
		
		catch (Throwable t)
		{
			TestCase.fail("ConnectionsFinderTask.ConnectionsFinderTask() has thrown an exception.");
		}
	}

	/**
	 * Tests adding of visited stations.
	 * @author Piotrek
	 */
	public void testAddVisitedStation()
	{
		ConnectionsFinderTask task = new ConnectionsFinderTask();
		
		boolean fThrown = false;
		
		try
		{
			task.AddVisitedStation(null);
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("ConnectionsFinderTask.AddVisitedStation() is not checking its parameter properly.");
		
		Station station1 = new Station(new Location(1, 1), "1", MeanOfTransportation.AIRPLANE);
		Station station2 = new Station(new Location(13, 12), "2", MeanOfTransportation.AIRPLANE);
		
		try
		{
			task.AddVisitedStation(station1);
			task.AddVisitedStation(station2);
		}
		
		catch (Throwable t)
		{
			TestCase.fail("ConnectionsFinderTask.AddVisitedStation() has thrown an exception while adding valid stations.");
		}
	}

	/**
	 * Tests of checking if the given station has been visited.
	 * @author Piotrek
	 */
	public void testHasVisitedStation()
	{
		ConnectionsFinderTask task = new ConnectionsFinderTask();
		
		boolean fThrown = false;
		
		try
		{
			task.HasVisitedStation(null);
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("ConnectionsFinderTask.HasVisitedStation() is not checking its parameter properly.");
		
		Station station1 = new Station(new Location(1, 1), "1", MeanOfTransportation.AIRPLANE);
		Station station2 = new Station(new Location(13, 12), "2", MeanOfTransportation.AIRPLANE);
		Station station3 = new Station(new Location(13, 122), "3", MeanOfTransportation.AIRPLANE);
		
		task.AddVisitedStation(station1);
		task.AddVisitedStation(station2);
		
		if (!task.HasVisitedStation(station1))
			TestCase.fail("ConnectionsFinderTask.HasVisitedStation() is not working properly.");
		
		if (!task.HasVisitedStation(station2))
			TestCase.fail("ConnectionsFinderTask.HasVisitedStation() is not working properly.");
		
		if (task.HasVisitedStation(station3))
			TestCase.fail("ConnectionsFinderTask.HasVisitedStation() is not working properly.");
	}

	/**
	 * Tests getting of the indecies of the connection chunks with at least one unset date.
	 * @author Piotrek
	 */
	public void testGetUnsetDatesChunksIndecies()
	{
		ConnectionsFinderTask task = new ConnectionsFinderTask();
		
		TravelPartConnectionChunk chunk1 = new TravelPartConnectionChunk(MeanOfTransportation.WALK, new Location(1, 1), new Location (2, 2));
		
		chunk1.SetStartDate(null);
		chunk1.SetEndDate(null);
		
		TravelPartConnectionChunk chunk2 = new TravelPartConnectionChunk(MeanOfTransportation.BUS, new Location(2, 2), new Location (3, 3));
		
		chunk2.SetStartDate(new Date(0));
		chunk2.SetEndDate(new Date(100000));
		
		TravelPartConnectionChunk chunk3 = new TravelPartConnectionChunk(MeanOfTransportation.TRAIN, new Location(3, 3), new Location (4, 4));
		
		chunk3.SetStartDate(new Date(100000));
		chunk3.SetEndDate(null);
		
		task.AddConnectionChunk(chunk1);
		task.AddConnectionChunk(chunk2);
		task.AddConnectionChunk(chunk3);
		
		Iterator iterator = task.GetUnsetDatesChunksIndecies();
		
		int i = 0;
		
		while (iterator.hasNext())
		{
			Integer index = (Integer)iterator.next();
			
			if (i == 0 && index.intValue() != 2)
				TestCase.fail("ConnectionsFinderTask.GetUnsetDatesChunksIndecies() is returning indecies in the wrong order.");
				
			if (i == 1 && index.intValue() != 0)
				TestCase.fail("ConnectionsFinderTask.GetUnsetDatesChunksIndecies() is returning indecies in the wrong order.");
				
			++i;
		}
		
		if (i != 2)
			TestCase.fail("ConnectionsFinderTask.GetUnsetDatesChunksIndecies() is not working properly.");
	}

	/**
	 * Tests checking if the task has chunks with at least one unset date.
	 * @author Piotrek
	 */
	public void testHasUnsetDatesChunks()
	{
		ConnectionsFinderTask task = new ConnectionsFinderTask();
		
		TravelPartConnectionChunk chunk = new TravelPartConnectionChunk(MeanOfTransportation.WALK, new Location(1, 1), new Location (2, 2));
		
		chunk.SetStartDate(null);
		chunk.SetEndDate(null);
		
		if (task.HasUnsetDatesChunks())
			TestCase.fail("ConnectionsFinderTask.HasUnsetDatesChunks() is not working properly.");
		
		task.AddConnectionChunk(chunk);
		
		if (!task.HasUnsetDatesChunks())
			TestCase.fail("ConnectionsFinderTask.HasUnsetDatesChunks() is not working properly.");
	}

	/**
	 * Tests getting of the chunks by their indecies.
	 * @author Piotrek
	 */
	public void testGetChunk()
	{		
		ConnectionsFinderTask task = new ConnectionsFinderTask();
		
		boolean fThrown = false;
		
		try
		{
			task.GetChunk(-1);
		}
		
		catch (IndexOutOfBoundsException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("ConnectionsFinderTask.GetChunk() is not checking its parameter properly.");
		
		TravelPartConnectionChunk chunk1 = new TravelPartConnectionChunk(MeanOfTransportation.WALK, new Location(1, 1), new Location (2, 2));
		
		chunk1.SetStartDate(null);
		chunk1.SetEndDate(null);
		
		TravelPartConnectionChunk chunk2 = new TravelPartConnectionChunk(MeanOfTransportation.BUS, new Location(2, 2), new Location (3, 3));
		
		chunk2.SetStartDate(new Date(0));
		chunk2.SetEndDate(new Date(100000));
		
		TravelPartConnectionChunk chunk3 = new TravelPartConnectionChunk(MeanOfTransportation.TRAIN, new Location(3, 3), new Location (4, 4));
		
		chunk3.SetStartDate(new Date(100000));
		chunk3.SetEndDate(null);
		
		task.AddConnectionChunk(chunk1);
		task.AddConnectionChunk(chunk2);
		task.AddConnectionChunk(chunk3);
		
		fThrown = false;
		
		try
		{
			task.GetChunk(3);
		}
		
		catch (IndexOutOfBoundsException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("ConnectionsFinderTask.GetChunk() is not checking its parameter properly.");
		
		if (task.GetChunk(0) != chunk1)
			TestCase.fail("ConnectionsFinderTask.GetChunk() is not working properly.");
		
		if (task.GetChunk(1) != chunk2)
			TestCase.fail("ConnectionsFinderTask.GetChunk() is not working properly.");
		
		if (task.GetChunk(2) != chunk3)
			TestCase.fail("ConnectionsFinderTask.GetChunk() is not working properly.");
	}

	/**
	 * Tests adding of the single connection chunk.
	 * @author Piotrek
	 */
	public void testAddConnectionChunk()
	{
		ConnectionsFinderTask task = new ConnectionsFinderTask();
		
		boolean fThrown = false;
		
		try
		{
			task.AddConnectionChunk(null);
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("ConnectionsFinderTask.AddConnectionChunk() is not checking its parameter properly.");
		
		TravelPartConnectionChunk chunk = new TravelPartConnectionChunk(MeanOfTransportation.WALK, new Location(1, 1), new Location (2, 2));
		
		chunk.SetStartDate(null);
		chunk.SetEndDate(null);
		
		try
		{
			task.AddConnectionChunk(chunk);
		}
		
		catch (Throwable t)
		{
			TestCase.fail("ConnectionsFinderTask.AddConnectionChunk() has thrown an exception while adding a valid connection chunk.");
		}
		
		if (task.GetChunk(task.GetConnectionChunksCount() - 1) != chunk)
			TestCase.fail("ConnectionsFinderTask.AddConnectionChunk() is not working properly.");
		
		chunk = new TravelPartConnectionChunk(MeanOfTransportation.BUS, new Location(2, 2), new Location (52, 52));
		
		task.AddConnectionChunk(chunk);
		
		if (task.GetChunk(task.GetConnectionChunksCount() - 1) != chunk)
			TestCase.fail("ConnectionsFinderTask.AddConnectionChunk() is not working properly.");
	}

	/**
	 * Tests getting of the connection chunks iterator.
	 * @author Piotrek
	 */
	public void testGetConnectionChunks()
	{
		ConnectionsFinderTask task = new ConnectionsFinderTask();
		
		TravelPartConnectionChunk chunk1 = new TravelPartConnectionChunk(MeanOfTransportation.WALK, new Location(1, 1), new Location (2, 2));
		
		chunk1.SetStartDate(null);
		chunk1.SetEndDate(null);
		
		TravelPartConnectionChunk chunk2 = new TravelPartConnectionChunk(MeanOfTransportation.BUS, new Location(2, 2), new Location (3, 3));
		
		chunk2.SetStartDate(new Date(0));
		chunk2.SetEndDate(new Date(100000));
		
		TravelPartConnectionChunk chunk3 = new TravelPartConnectionChunk(MeanOfTransportation.TRAIN, new Location(3, 3), new Location (4, 4));
		
		chunk3.SetStartDate(new Date(100000));
		chunk3.SetEndDate(null);
		
		task.AddConnectionChunk(chunk1);
		task.AddConnectionChunk(chunk2);
		task.AddConnectionChunk(chunk3);
		
		Iterator iterator = task.GetConnectionChunks();
		
		if (iterator == null)
			TestCase.fail("ConnectionsFinderTask.GetConnectionChunks() is not working properly.");
		
		int i = 0;
		
		while (iterator.hasNext())
		{
			TravelPartConnectionChunk chunk = (TravelPartConnectionChunk)iterator.next();
			
			if (i == 0 && chunk != chunk1)
				TestCase.fail("ConnectionsFinderTask.GetConnectionChunks() is not working properly.");
			
			if (i == 1 && chunk != chunk2)
				TestCase.fail("ConnectionsFinderTask.GetConnectionChunks() is not working properly.");
			
			if (i == 2 && chunk != chunk3)
				TestCase.fail("ConnectionsFinderTask.GetConnectionChunks() is not working properly.");
			
			++i;
		}
		
		if (i != 3)
			TestCase.fail("ConnectionsFinderTask.GetConnectionChunks() is not working properly.");
	}
	
	/**
	 * Tests getting of the first connection chunk.
	 * @author Piotrek
	 */
	public void testGetFirstConnectionChunk()
	{
		ConnectionsFinderTask task = new ConnectionsFinderTask();
		
		TravelPartConnectionChunk chunk1 = new TravelPartConnectionChunk(MeanOfTransportation.WALK, new Location(1, 1), new Location (2, 2));
		
		chunk1.SetStartDate(null);
		chunk1.SetEndDate(null);
		
		TravelPartConnectionChunk chunk2 = new TravelPartConnectionChunk(MeanOfTransportation.BUS, new Location(2, 2), new Location (3, 3));
		
		chunk2.SetStartDate(new Date(0));
		chunk2.SetEndDate(new Date(100000));
		
		TravelPartConnectionChunk chunk3 = new TravelPartConnectionChunk(MeanOfTransportation.TRAIN, new Location(3, 3), new Location (4, 4));
		
		chunk3.SetStartDate(new Date(100000));
		chunk3.SetEndDate(null);
		
		task.AddConnectionChunk(chunk1);
		task.AddConnectionChunk(chunk2);
		task.AddConnectionChunk(chunk3);
		
		if (task.GetFirstConnectionChunk() != chunk1)
			TestCase.fail("ConnectionsFinderTask.GetFirstConnectionChunk() is not working properly.");
	}

	/**
	 * Tests getting of the last connection chunk.
	 * @author Piotrek
	 */
	public void testGetLastConnectionChunk()
	{
		ConnectionsFinderTask task = new ConnectionsFinderTask();
		
		TravelPartConnectionChunk chunk1 = new TravelPartConnectionChunk(MeanOfTransportation.WALK, new Location(1, 1), new Location (2, 2));
		
		chunk1.SetStartDate(null);
		chunk1.SetEndDate(null);
		
		TravelPartConnectionChunk chunk2 = new TravelPartConnectionChunk(MeanOfTransportation.BUS, new Location(2, 2), new Location (3, 3));
		
		chunk2.SetStartDate(new Date(0));
		chunk2.SetEndDate(new Date(100000));
		
		TravelPartConnectionChunk chunk3 = new TravelPartConnectionChunk(MeanOfTransportation.TRAIN, new Location(3, 3), new Location (4, 4));
		
		chunk3.SetStartDate(new Date(100000));
		chunk3.SetEndDate(null);
		
		task.AddConnectionChunk(chunk1);
		task.AddConnectionChunk(chunk2);
		task.AddConnectionChunk(chunk3);
		
		if (task.GetLastConnectionChunk() != chunk3)
			TestCase.fail("ConnectionsFinderTask.GetLastConnectionChunk() is not working properly.");
	}

	/**
	 * Tests getting of the last connection chunkwith the end date set.
	 * @author Piotrek
	 */
	public void testGetLastConnectionChunkWithEndDate()
	{
		ConnectionsFinderTask task = new ConnectionsFinderTask();
		
		TravelPartConnectionChunk chunk1 = new TravelPartConnectionChunk(MeanOfTransportation.WALK, new Location(1, 1), new Location (2, 2));
		
		chunk1.SetStartDate(null);
		chunk1.SetEndDate(null);
		
		TravelPartConnectionChunk chunk2 = new TravelPartConnectionChunk(MeanOfTransportation.BUS, new Location(2, 2), new Location (3, 3));
		
		chunk2.SetStartDate(new Date(0));
		chunk2.SetEndDate(new Date(100000));
		
		TravelPartConnectionChunk chunk3 = new TravelPartConnectionChunk(MeanOfTransportation.TRAIN, new Location(3, 3), new Location (4, 4));
		
		chunk3.SetStartDate(new Date(100000));
		chunk3.SetEndDate(null);
		
		task.AddConnectionChunk(chunk1);
		task.AddConnectionChunk(chunk2);
		task.AddConnectionChunk(chunk3);
		
		if (task.GetLastConnectionChunkWithEndDate() != chunk2)
			TestCase.fail("ConnectionsFinderTask.GetLastConnectionChunkWithEndDate() is not working properly.");
	}

	/**
	 * Tests cloning.
	 * @author
	 */
	public void testClone()
	{
		ConnectionsFinderTask task = new ConnectionsFinderTask();
		
		TravelPartConnectionChunk chunk1 = new TravelPartConnectionChunk(MeanOfTransportation.WALK, new Location(1, 1), new Location (2, 2));
		
		chunk1.SetStartDate(null);
		chunk1.SetEndDate(null);
		
		TravelPartConnectionChunk chunk2 = new TravelPartConnectionChunk(MeanOfTransportation.BUS, new Location(2, 2), new Location (3, 3));
		
		chunk2.SetStartDate(new Date(0));
		chunk2.SetEndDate(new Date(100000));
		
		TravelPartConnectionChunk chunk3 = new TravelPartConnectionChunk(MeanOfTransportation.TRAIN, new Location(3, 3), new Location (4, 4));
		
		chunk3.SetStartDate(new Date(100000));
		chunk3.SetEndDate(null);
		
		task.AddConnectionChunk(chunk1);
		task.AddConnectionChunk(chunk2);
		task.AddConnectionChunk(chunk3);
		
		task.currentStation = new Station(new Location(1, 2), "1", MeanOfTransportation.AIRPLANE);
		
		ArrayList stations = new ArrayList(2);
		
		Station airport1 = new Airport(new Location(1, 2), "One");
		Station airport2 = new Airport(new Location(11, 12), "Two");
		
		stations.add(airport1);
		stations.add(airport2);
		
		ArrayList arrivals = new ArrayList(2);
		
		arrivals.add(new Integer(60));
		arrivals.add(new Integer(120));
		
		ArrayList departures = new ArrayList(2);
		
		departures.add(new Integer(70));
		departures.add(new Integer(130));
		
		char mask = Connection.MONDAY | Connection.FRIDAY;
		
		task.AddVisitedStation(airport1);
		task.AddVisitedStation(airport2);
		
		task.connection = new Connection("Flight #1", MeanOfTransportation.AIRPLANE, stations, arrivals, departures, mask);
				
		task.dayOfWeek = Connection.FRIDAY;
		
		task.startStation = new Station(new Location(1, 2), "2", MeanOfTransportation.AIRPLANE);
		
		task.startStationDepartureDate = new Date(1001);
		
		task.fDirect = true;
		
		ConnectionsFinderTask clone = (ConnectionsFinderTask)task.clone();
		
		if (clone.connection != task.connection)
			TestCase.fail("ConnectionsFinderTask.clone() is not working properly.");
		
		if (clone.currentStation != task.currentStation)
			TestCase.fail("ConnectionsFinderTask.clone() is not working properly.");
		
		if (clone.dayOfWeek != task.dayOfWeek)
			TestCase.fail("ConnectionsFinderTask.clone() is not working properly.");
		
		if (clone.startStation != task.startStation)
			TestCase.fail("ConnectionsFinderTask.clone() is not working properly.");
		
		if (clone.startStationDepartureDate != task.startStationDepartureDate)
			TestCase.fail("ConnectionsFinderTask.clone() is not working properly.");
		
		if (clone.fDirect != task.fDirect)
			TestCase.fail("ConnectionsFinderTask.clone() is not working properly.");
		
		if (clone.GetChangesCount() != task.GetChangesCount())
			TestCase.fail("ConnectionsFinderTask.clone() is not working properly.");
		
		if (clone.GetConnectionChunksCount() != task.GetConnectionChunksCount())
			TestCase.fail("ConnectionsFinderTask.clone() is not working properly.");
		
		Iterator originalIterator = task.GetConnectionChunks();
		Iterator cloneIterator = clone.GetConnectionChunks();
		
		while (originalIterator.hasNext() && cloneIterator.hasNext())
		{
			TravelPartConnectionChunk originalChunk = (TravelPartConnectionChunk)originalIterator.next();
			TravelPartConnectionChunk cloneChunk = (TravelPartConnectionChunk)cloneIterator.next();
			
			if (originalChunk.GetStartDate() != null && originalChunk.GetEndDate() != null && originalChunk != cloneChunk)
				TestCase.fail("ConnectionsFinderTask.clone() is not working properly.");
		}
		
		if (originalIterator.hasNext() || cloneIterator.hasNext())
			TestCase.fail("ConnectionsFinderTask.clone() is not working properly.");
		
		if (!clone.HasVisitedStation(airport1))
			TestCase.fail("ConnectionsFinderTask.clone() is not working properly.");
		
		if (!clone.HasVisitedStation(airport2))
			TestCase.fail("ConnectionsFinderTask.clone() is not working properly.");
	}

	/**
	 * Tests getting a number of changes.
	 * @author Piotrek
	 */
	public void testGetChangesCount()
	{
		ConnectionsFinderTask task = new ConnectionsFinderTask();
		
		TravelPartConnectionChunk chunk1 = new TravelPartConnectionChunk(MeanOfTransportation.WALK, new Location(1, 1), new Location (2, 2));
		
		chunk1.SetStartDate(null);
		chunk1.SetEndDate(null);
		
		TravelPartConnectionChunk chunk2 = new TravelPartConnectionChunk(MeanOfTransportation.BUS, new Location(2, 2), new Location (3, 3));
		
		chunk2.SetStartDate(new Date(0));
		chunk2.SetEndDate(new Date(100000));
		
		TravelPartConnectionChunk chunk3 = new TravelPartConnectionChunk(MeanOfTransportation.TRAIN, new Location(3, 3), new Location (4, 4));
		
		chunk3.SetStartDate(new Date(100000));
		chunk3.SetEndDate(null);
		
		if (task.GetChangesCount() != 0)
			TestCase.fail("ConnectionsFinderTask.GetChangesCount() is not working properly.");
		
		task.AddConnectionChunk(chunk1);
		
		if (task.GetChangesCount() != 0)
			TestCase.fail("ConnectionsFinderTask.GetChangesCount() is not working properly.");
		
		task.AddConnectionChunk(chunk2);
		
		if (task.GetChangesCount() != 0)
			TestCase.fail("ConnectionsFinderTask.GetChangesCount() is not working properly.");
		
		task.AddConnectionChunk(chunk3);
		
		if (task.GetChangesCount() != 1)
			TestCase.fail("ConnectionsFinderTask.GetChangesCount() is not working properly.");
	}

	/** 
	 * Tests clearing a list of visited stations.
	 * @author Piotrek
	 */
	public void testClearVisitedStations()
	{
		ConnectionsFinderTask task = new ConnectionsFinderTask();
		
		Station airport1 = new Airport(new Location(1, 2), "One");
		Station airport2 = new Airport(new Location(11, 12), "Two");
		
		task.AddVisitedStation(airport1);
		task.AddVisitedStation(airport2);
		
		task.ClearVisitedStations();
		
		if (task.HasVisitedStation(airport1))
			TestCase.fail("ConnectionsFinderTask.ClearVisitedStations() is not working properly.");
		
		if (task.HasVisitedStation(airport2))
			TestCase.fail("ConnectionsFinderTask.ClearVisitedStations() is not working properly.");
	}

	/**
	 * Tests setting of the current working data to the same values as of the other task's.
	 * @author Piotrek
	 */
	public void testSetCurrentWorkingData()
	{
		ConnectionsFinderTask task = new ConnectionsFinderTask();
				
		task.currentStation = new Station(new Location(1, 2), "1", MeanOfTransportation.AIRPLANE);
		
		ArrayList stations = new ArrayList(2);
		
		Station airport1 = new Airport(new Location(1, 2), "One");
		Station airport2 = new Airport(new Location(11, 12), "Two");
		
		stations.add(airport1);
		stations.add(airport2);
		
		ArrayList arrivals = new ArrayList(2);
		
		arrivals.add(new Integer(60));
		arrivals.add(new Integer(120));
		
		ArrayList departures = new ArrayList(2);
		
		departures.add(new Integer(70));
		departures.add(new Integer(130));
		
		char mask = Connection.MONDAY | Connection.FRIDAY;
		
		task.AddVisitedStation(airport1);
		task.AddVisitedStation(airport2);
		
		task.connection = new Connection("Flight #1", MeanOfTransportation.AIRPLANE, stations, arrivals, departures, mask);
		
		task.dayOfWeek = Connection.FRIDAY;
		
		task.startStation = new Station(new Location(1, 2), "2", MeanOfTransportation.AIRPLANE);
		
		task.startStationDepartureDate = new Date(1001);
		
		task.fDirect = true;
		
		ConnectionsFinderTask task2 = new ConnectionsFinderTask();
		
		task2.SetCurrentWorkingData(task);
		
		if (task2.currentStation != task.currentStation)
			TestCase.fail("ConnectionsFinderTask.SetCurrentWorkingData() is not working properly.");
		
		if (task2.startStation != task.startStation)
			TestCase.fail("ConnectionsFinderTask.SetCurrentWorkingData() is not working properly.");
		
		if (task2.startStationDepartureDate != task.startStationDepartureDate)
			TestCase.fail("ConnectionsFinderTask.SetCurrentWorkingData() is not working properly.");
		
		if (task2.connection != task.connection)
			TestCase.fail("ConnectionsFinderTask.SetCurrentWorkingData() is not working properly.");
		
		if (task2.dayOfWeek != task.dayOfWeek)
			TestCase.fail("ConnectionsFinderTask.SetCurrentWorkingData() is not working properly.");
		
		if (task2.fDirect != task.fDirect)
			TestCase.fail("ConnectionsFinderTask.SetCurrentWorkingData() is not working properly.");
	}

	/**
	 * Tests getting of a number of connection chunks.
	 * @author Piotrek
	 */
	public void testGetConnectionChunksCount()
	{
		ConnectionsFinderTask task = new ConnectionsFinderTask();
		
		TravelPartConnectionChunk chunk1 = new TravelPartConnectionChunk(MeanOfTransportation.WALK, new Location(1, 1), new Location (2, 2));
		
		chunk1.SetStartDate(null);
		chunk1.SetEndDate(null);
		
		TravelPartConnectionChunk chunk2 = new TravelPartConnectionChunk(MeanOfTransportation.BUS, new Location(2, 2), new Location (3, 3));
		
		chunk2.SetStartDate(new Date(0));
		chunk2.SetEndDate(new Date(100000));
		
		TravelPartConnectionChunk chunk3 = new TravelPartConnectionChunk(MeanOfTransportation.TRAIN, new Location(3, 3), new Location (4, 4));
		
		chunk3.SetStartDate(new Date(100000));
		chunk3.SetEndDate(null);
		
		if (task.GetConnectionChunksCount() != 0)
			TestCase.fail("ConnectionsFinderTask.GetConnectionChunksCount() is not working properly.");
		
		task.AddConnectionChunk(chunk1);
		
		if (task.GetConnectionChunksCount() != 1)
			TestCase.fail("ConnectionsFinderTask.GetConnectionChunksCount() is not working properly.");
		
		task.AddConnectionChunk(chunk2);
		
		if (task.GetConnectionChunksCount() != 2)
			TestCase.fail("ConnectionsFinderTask.GetConnectionChunksCount() is not working properly.");
		
		task.AddConnectionChunk(chunk3);
		
		if (task.GetConnectionChunksCount() != 3)
			TestCase.fail("ConnectionsFinderTask.GetConnectionChunksCount() is not working properly.");
		
		if (task.GetConnectionChunksCount() != 3)
			TestCase.fail("ConnectionsFinderTask.GetConnectionChunksCount() is not working properly.");
	}
	
	/**
	 * Tests getting of the total duration of travel.
	 * @author Piotrek
	 */
	public void testGetTotalDuration()
	{
		ConnectionsFinderTask task = new ConnectionsFinderTask();
		
		TravelPartConnectionChunk chunk1 = new TravelPartConnectionChunk(MeanOfTransportation.WALK, new Location(1, 1), new Location (2, 2));
		
		chunk1.SetStartDate(null);
		chunk1.SetEndDate(null);
		
		TravelPartConnectionChunk chunk2 = new TravelPartConnectionChunk(MeanOfTransportation.BUS, new Location(2, 2), new Location (3, 3));
		
		chunk2.SetStartDate(new Date(0));
		chunk2.SetEndDate(new Date(60000));
		
		TravelPartConnectionChunk chunk3 = new TravelPartConnectionChunk(MeanOfTransportation.TRAIN, new Location(3, 3), new Location (4, 4));
		
		chunk3.SetStartDate(new Date(60000));
		chunk3.SetEndDate(new Date(600000));
		
		if (task.GetTotalDuration() != 0)
			TestCase.fail("ConnectionsFinderTask.GetTotalDuration() is not working properly.");
		
		task.AddConnectionChunk(chunk1);
		
		if (task.GetTotalDuration() != -1)
			TestCase.fail("ConnectionsFinderTask.GetTotalDuration() is not working properly.");
		
		task = new ConnectionsFinderTask();
		
		task.AddConnectionChunk(chunk2);
		
		if (task.GetTotalDuration() != 1)
			TestCase.fail("ConnectionsFinderTask.GetTotalDuration() is not working properly.");
		
		task.AddConnectionChunk(chunk3);
		
		if (task.GetTotalDuration() != 10)
			TestCase.fail("ConnectionsFinderTask.GetTotalDuration() is not working properly." + task.GetTotalDuration());
	}
	
	/**
	 * Test the similarity test.
	 * @author Piotrek
	 */
	public void testIsSimilar()
	{
		ConnectionsFinderTask task1 = new ConnectionsFinderTask();
		ConnectionsFinderTask task2 = new ConnectionsFinderTask();
		ConnectionsFinderTask task3 = new ConnectionsFinderTask();
		ConnectionsFinderTask task4 = new ConnectionsFinderTask();
		ConnectionsFinderTask task5 = new ConnectionsFinderTask();
		ConnectionsFinderTask task6 = new ConnectionsFinderTask();
		
		// start chunks
		TravelPartConnectionChunk chunk1 = new TravelPartConnectionChunk(MeanOfTransportation.BUS, new Location(3, 3), new Location(5, 5));
		TravelPartConnectionChunk chunk2 = new TravelPartConnectionChunk(MeanOfTransportation.BUS, new Location(3, 3), new Location(5, 5));
		// middle chunks
		TravelPartConnectionChunk chunk3 = new TravelPartConnectionChunk(MeanOfTransportation.BUS, new Location(5, 5), new Location(45, 45));
		TravelPartConnectionChunk chunk4 = new TravelPartConnectionChunk(MeanOfTransportation.BUS, new Location(5, 5), new Location(45, 45));
		TravelPartConnectionChunk chunk5 = new TravelPartConnectionChunk(MeanOfTransportation.BUS, new Location(5, 5), new Location(44, 45));
		// endchunks
		TravelPartConnectionChunk chunk6 = new TravelPartConnectionChunk(MeanOfTransportation.TRAIN, new Location(45, 45), new Location(46, 46));
		TravelPartConnectionChunk chunk7 = new TravelPartConnectionChunk(MeanOfTransportation.TRAIN, new Location(45, 45), new Location(46, 46));
		TravelPartConnectionChunk chunk8 = new TravelPartConnectionChunk(MeanOfTransportation.TRAIN, new Location(44, 45), new Location(46, 46));
		
		chunk1.SetStartDate(new Date(1000));
		chunk1.SetEndDate(new Date(1006));
		
		chunk2.SetStartDate(new Date(1005));
		chunk2.SetEndDate(new Date(1006));
		
		chunk3.SetStartDate(new Date(1006));
		chunk3.SetEndDate(new Date(1007));
		
		chunk4.SetStartDate(new Date(1008));
		chunk4.SetEndDate(new Date(10010));
		
		chunk5.SetStartDate(new Date(1006));
		chunk5.SetEndDate(new Date(1007));
		
		chunk5.SetStartDate(new Date(10016));
		chunk5.SetEndDate(new Date(10017));
		
		chunk6.SetStartDate(new Date(10016));
		chunk6.SetEndDate(new Date(10018));
		
		task1.AddConnectionChunk(chunk1);
		task2.AddConnectionChunk(chunk2);
		task3.AddConnectionChunk(chunk2);
		task4.AddConnectionChunk(chunk2);
		task5.AddConnectionChunk(chunk2);
		task6.AddConnectionChunk(chunk2);

		task1.AddConnectionChunk(chunk3);
		task2.AddConnectionChunk(chunk3);
		task3.AddConnectionChunk(chunk3);
		task4.AddConnectionChunk(chunk4);
		task5.AddConnectionChunk(chunk5);
		task6.AddConnectionChunk(chunk3);
		
		task1.AddConnectionChunk(chunk6);
		task2.AddConnectionChunk(chunk6);
		task3.AddConnectionChunk(chunk6);
		task4.AddConnectionChunk(chunk6);
		task5.AddConnectionChunk(chunk8);
		task6.AddConnectionChunk(chunk7);
		
		if (task1.IsSimilar(task2))
			TestCase.fail("ConnectionsFinderTask.IsSimilar() is not working properly.");
		
		if (!task3.IsSimilar(task2))
			TestCase.fail("ConnectionsFinderTask.IsSimilar() is not working properly.");
		
		if (!task2.IsSimilar(task3))
			TestCase.fail("ConnectionsFinderTask.IsSimilar() is not working properly.");
		
		if (task2.IsSimilar(task5))
			TestCase.fail("ConnectionsFinderTask.IsSimilar() is not working properly.");
		
		if (!task3.IsSimilar(task4))
			TestCase.fail("ConnectionsFinderTask.IsSimilar() is not working properly.");
		
		if (task2.IsSimilar(task6))
			TestCase.fail("ConnectionsFinderTask.IsSimilar() is not working properly.");
	}
	
	/**
	 * Tests getting of the max change duration.
	 * @author Piotrek
	 */
	public void testGetMaxChangeDuration()
	{
		ConnectionsFinderTask task1 = new ConnectionsFinderTask();
		ConnectionsFinderTask task2 = new ConnectionsFinderTask();
		
		if (task1.GetMaxChangeDuration() != -1)
			TestCase.fail("ConnectionsFinderTask.GetMaxChangeDuration() is not working properly.");
		
		// start chunks
		TravelPartConnectionChunk chunk1 = new TravelPartConnectionChunk(MeanOfTransportation.BUS, new Location(3, 3), new Location(5, 5));
		TravelPartConnectionChunk chunk2 = new TravelPartConnectionChunk(MeanOfTransportation.BUS, new Location(3, 3), new Location(5, 5));
		// middle chunks 1
		TravelPartConnectionChunk chunk3 = new TravelPartConnectionChunk(MeanOfTransportation.WALK, new Location(5, 5), new Location(45, 45));
		TravelPartConnectionChunk chunk4 = new TravelPartConnectionChunk(MeanOfTransportation.WALK, new Location(5, 5), new Location(45, 45));
		// middle chunks 2
		TravelPartConnectionChunk chunk5 = new TravelPartConnectionChunk(MeanOfTransportation.TRAIN, new Location(45, 45), new Location(46, 46));
		TravelPartConnectionChunk chunk6 = new TravelPartConnectionChunk(MeanOfTransportation.TRAIN, new Location(45, 45), new Location(46, 46));
		// endchunks
		TravelPartConnectionChunk chunk7 = new TravelPartConnectionChunk(MeanOfTransportation.TRAIN, new Location(46, 46), new Location(46, 46));
		TravelPartConnectionChunk chunk8 = new TravelPartConnectionChunk(MeanOfTransportation.TRAIN, new Location(46, 46), new Location(46, 46));
				
		chunk1.SetStartDate(new Date());
		chunk1.SetEndDate(new Date());
		TravelTime.ShiftDate(chunk1.GetEndDate(), 1);
		
		chunk2.SetStartDate(new Date());
		chunk2.SetEndDate(new Date());
		TravelTime.ShiftDate(chunk2.GetEndDate(), 1);
		
		chunk3.SetStartDate(new Date());
		TravelTime.ShiftDate(chunk3.GetStartDate(), 1);
		chunk3.SetEndDate(new Date());
		TravelTime.ShiftDate(chunk3.GetEndDate(), 2);
		
		chunk4.SetStartDate(new Date());
		TravelTime.ShiftDate(chunk4.GetStartDate(), 1);
		chunk4.SetEndDate(new Date());
		TravelTime.ShiftDate(chunk4.GetEndDate(), 3);
		
		chunk5.SetStartDate(new Date());
		TravelTime.ShiftDate(chunk5.GetStartDate(), 2);
		chunk5.SetEndDate(new Date());
		TravelTime.ShiftDate(chunk5.GetEndDate(), 4);
		
		chunk6.SetStartDate(new Date());
		TravelTime.ShiftDate(chunk6.GetStartDate(), 3);
		chunk6.SetEndDate(new Date());
		TravelTime.ShiftDate(chunk6.GetEndDate(), 6);
		
		chunk7.SetStartDate(new Date());
		TravelTime.ShiftDate(chunk7.GetStartDate(), 6);
		chunk7.SetEndDate(new Date());
		TravelTime.ShiftDate(chunk7.GetEndDate(), 7);
		
		chunk8.SetStartDate(new Date());
		TravelTime.ShiftDate(chunk8.GetStartDate(), 7);
		chunk8.SetEndDate(new Date());
		TravelTime.ShiftDate(chunk8.GetEndDate(), 8);
		
		task1.AddConnectionChunk(chunk1);
		task2.AddConnectionChunk(chunk2);
		
		task1.AddConnectionChunk(chunk3);
		task2.AddConnectionChunk(chunk4);
		
		task1.AddConnectionChunk(chunk5);
		task2.AddConnectionChunk(chunk6);
		
		if (task1.GetMaxChangeDuration() != 1)
			TestCase.fail("ConnectionsFinderTask.GetMaxChangeDuration() is not working properly.");
		
		if (task2.GetMaxChangeDuration() != 2)
			TestCase.fail("ConnectionsFinderTask.GetMaxChangeDuration() is not working properly.");
		
		task1.AddConnectionChunk(chunk7);
		task2.AddConnectionChunk(chunk8);
		
		if (task1.GetMaxChangeDuration() != 2)
			TestCase.fail("ConnectionsFinderTask.GetMaxChangeDuration() is not working properly.");
		
		if (task2.GetMaxChangeDuration() != 2)
			TestCase.fail("ConnectionsFinderTask.GetMaxChangeDuration() is not working properly.");
	}
	
	/**
	 * Tests getting of the average change duration.
	 * @author Piotrek
	 */
	public void testGetAverageChangeDuration()
	{
		ConnectionsFinderTask task1 = new ConnectionsFinderTask();
		ConnectionsFinderTask task2 = new ConnectionsFinderTask();
		
		if (task1.GetAverageChangeDuration() != -1)
			TestCase.fail("ConnectionsFinderTask.GetAverageChangeDuration() is not working properly.");
		
		// start chunks
		TravelPartConnectionChunk chunk1 = new TravelPartConnectionChunk(MeanOfTransportation.BUS, new Location(3, 3), new Location(5, 5));
		TravelPartConnectionChunk chunk2 = new TravelPartConnectionChunk(MeanOfTransportation.BUS, new Location(3, 3), new Location(5, 5));
		// middle chunks 1
		TravelPartConnectionChunk chunk3 = new TravelPartConnectionChunk(MeanOfTransportation.WALK, new Location(5, 5), new Location(45, 45));
		TravelPartConnectionChunk chunk4 = new TravelPartConnectionChunk(MeanOfTransportation.WALK, new Location(5, 5), new Location(45, 45));
		// middle chunks 2
		TravelPartConnectionChunk chunk5 = new TravelPartConnectionChunk(MeanOfTransportation.TRAIN, new Location(45, 45), new Location(46, 46));
		TravelPartConnectionChunk chunk6 = new TravelPartConnectionChunk(MeanOfTransportation.TRAIN, new Location(45, 45), new Location(46, 46));
		// endchunks
		TravelPartConnectionChunk chunk7 = new TravelPartConnectionChunk(MeanOfTransportation.TRAIN, new Location(46, 46), new Location(46, 46));
		TravelPartConnectionChunk chunk8 = new TravelPartConnectionChunk(MeanOfTransportation.TRAIN, new Location(46, 46), new Location(46, 46));
				
		chunk1.SetStartDate(new Date());
		chunk1.SetEndDate(new Date());
		TravelTime.ShiftDate(chunk1.GetEndDate(), 1);
		
		chunk2.SetStartDate(new Date());
		chunk2.SetEndDate(new Date());
		TravelTime.ShiftDate(chunk2.GetEndDate(), 1);
		
		chunk3.SetStartDate(new Date());
		TravelTime.ShiftDate(chunk3.GetStartDate(), 1);
		chunk3.SetEndDate(new Date());
		TravelTime.ShiftDate(chunk3.GetEndDate(), 2);
		
		chunk4.SetStartDate(new Date());
		TravelTime.ShiftDate(chunk4.GetStartDate(), 1);
		chunk4.SetEndDate(new Date());
		TravelTime.ShiftDate(chunk4.GetEndDate(), 3);
		
		chunk5.SetStartDate(new Date());
		TravelTime.ShiftDate(chunk5.GetStartDate(), 2);
		chunk5.SetEndDate(new Date());
		TravelTime.ShiftDate(chunk5.GetEndDate(), 4);
		
		chunk6.SetStartDate(new Date());
		TravelTime.ShiftDate(chunk6.GetStartDate(), 3);
		chunk6.SetEndDate(new Date());
		TravelTime.ShiftDate(chunk6.GetEndDate(), 6);
		
		chunk7.SetStartDate(new Date());
		TravelTime.ShiftDate(chunk7.GetStartDate(), 7);
		chunk7.SetEndDate(new Date());
		TravelTime.ShiftDate(chunk7.GetEndDate(), 8);
		
		chunk8.SetStartDate(new Date());
		TravelTime.ShiftDate(chunk8.GetStartDate(), 10);
		chunk8.SetEndDate(new Date());
		TravelTime.ShiftDate(chunk8.GetEndDate(), 11);
		
		task1.AddConnectionChunk(chunk1);
		task2.AddConnectionChunk(chunk2);
		
		task1.AddConnectionChunk(chunk3);
		task2.AddConnectionChunk(chunk4);
		
		task1.AddConnectionChunk(chunk5);
		task2.AddConnectionChunk(chunk6);
		
		if (task1.GetAverageChangeDuration() != 1)
			TestCase.fail("ConnectionsFinderTask.GetAverageChangeDuration() is not working properly.");
		
		if (task2.GetAverageChangeDuration() != 2)
			TestCase.fail("ConnectionsFinderTask.GetAverageChangeDuration() is not working properly.");
		
		task1.AddConnectionChunk(chunk7);
		task2.AddConnectionChunk(chunk8);
		
		if (task1.GetAverageChangeDuration() != 2)
			TestCase.fail("ConnectionsFinderTask.GetAverageChangeDuration() is not working properly.");
		
		if (task2.GetAverageChangeDuration() != 3)
			TestCase.fail("ConnectionsFinderTask.GetAverageChangeDuration() is not working properly.");
	}
	
	/**
	 * Tests getting of the total distance.
	 * @author Piotrek
	 */
	public void testGetTotalDistance()
	{
		ConnectionsFinderTask task = new ConnectionsFinderTask();
		
		if (task.GetTotalDistance() != 0)
			TestCase.fail("ConnectionsFinderTask.GetTotalDistance() is not working properly.");
		
		TravelPartConnectionChunk chunk1 = new TravelPartConnectionChunk(MeanOfTransportation.BUS, new Location(3, 3), new Location(5, 3));
		TravelPartConnectionChunk chunk2 = new TravelPartConnectionChunk(MeanOfTransportation.BUS, new Location(5, 3), new Location(9, 3));
		TravelPartConnectionChunk chunk3 = new TravelPartConnectionChunk(MeanOfTransportation.WALK, new Location(9, 3), new Location(15, 3));
		TravelPartConnectionChunk chunk4 = new TravelPartConnectionChunk(MeanOfTransportation.WALK, new Location(15, 3), new Location(15, -5));
				
		task.AddConnectionChunk(chunk1);
		
		if (task.GetTotalDistance() != 2)
			TestCase.fail("ConnectionsFinderTask.GetTotalDistance() is not working properly.");
		
		task.AddConnectionChunk(chunk2);
		
		if (task.GetTotalDistance() != 6)
			TestCase.fail("ConnectionsFinderTask.GetTotalDistance() is not working properly.");
		
		task.AddConnectionChunk(chunk3);
		
		if (task.GetTotalDistance() != 12)
			TestCase.fail("ConnectionsFinderTask.GetTotalDistance() is not working properly.");
		
		task.AddConnectionChunk(chunk4);
		
		if (task.GetTotalDistance() != 20)
			TestCase.fail("ConnectionsFinderTask.GetTotalDistance() is not working properly.");
	}
	
	/**
	 * Tests getting of the used means of transportation.
	 * @author Piotrek
	 */
	public void testGetUsedMeansOfTransportation()
	{
		ConnectionsFinderTask task = new ConnectionsFinderTask();
		
		if (task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.AIRPLANE)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.TAXI)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.BUS)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.SHIP)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.TRAIN)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.WALK)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		TravelPartConnectionChunk chunk1 = new TravelPartConnectionChunk(MeanOfTransportation.BUS, new Location(3, 3), new Location(5, 3));
		TravelPartConnectionChunk chunk2 = new TravelPartConnectionChunk(MeanOfTransportation.WALK, new Location(5, 3), new Location(9, 3));
		TravelPartConnectionChunk chunk3 = new TravelPartConnectionChunk(MeanOfTransportation.TRAIN, new Location(9, 3), new Location(1, 3));
		TravelPartConnectionChunk chunk4 = new TravelPartConnectionChunk(MeanOfTransportation.TRAIN, new Location(1, 3), new Location(15, 3));
		TravelPartConnectionChunk chunk5 = new TravelPartConnectionChunk(MeanOfTransportation.TAXI, new Location(15, 3), new Location(15, -5));
		TravelPartConnectionChunk chunk6 = new TravelPartConnectionChunk(MeanOfTransportation.SHIP, new Location(15, -5), new Location(15, 0));
		TravelPartConnectionChunk chunk7 = new TravelPartConnectionChunk(MeanOfTransportation.AIRPLANE, new Location(15, 0), new Location(30, 0));
				
		task.AddConnectionChunk(chunk1);
		
		if (task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.AIRPLANE)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.TAXI)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (!task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.BUS)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.SHIP)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.TRAIN)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.WALK)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		task.AddConnectionChunk(chunk2);
		
		if (task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.AIRPLANE)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.TAXI)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (!task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.BUS)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.SHIP)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.TRAIN)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (!task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.WALK)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		task.AddConnectionChunk(chunk3);
		
		if (task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.AIRPLANE)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.TAXI)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (!task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.BUS)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.SHIP)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (!task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.TRAIN)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (!task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.WALK)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		task.AddConnectionChunk(chunk4);
		
		if (task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.AIRPLANE)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.TAXI)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (!task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.BUS)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.SHIP)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (!task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.TRAIN)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (!task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.WALK)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		task.AddConnectionChunk(chunk5);
		
		if (task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.AIRPLANE)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (!task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.TAXI)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (!task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.BUS)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.SHIP)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (!task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.TRAIN)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (!task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.WALK)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		task.AddConnectionChunk(chunk6);
		
		if (task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.AIRPLANE)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (!task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.TAXI)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (!task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.BUS)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (!task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.SHIP)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (!task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.TRAIN)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (!task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.WALK)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		task.AddConnectionChunk(chunk7);
		
		if (!task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.AIRPLANE)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (!task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.TAXI)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (!task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.BUS)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (!task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.SHIP)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (!task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.TRAIN)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
		
		if (!task.GetUsedMeansOfTransportation().contains(new Integer(MeanOfTransportation.WALK)))
			TestCase.fail("ConnectionsFinderTask.GetUsedMeansOfTransportation() is not working properly.");
	}
}