/*
 * Created on 2005-06-12
 */
package tests;

import junit.framework.TestCase;
import crn.Case;
import crn.IE;

/**
 * Tests the Information Entity class.
 * @author Piotrek
 */
public class IETest extends TestCase
{
	/**
	 * Tests the constructor.
	 * @author Piotrek
	 */
	public void testIEStringString()
	{
		boolean fThrown = false;
		
		try
		{
			new IE(null, "value");
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("IE.IE() is not checking its parameters properly.");
		
		fThrown = false;
		
		try
		{
			new IE("name", null);
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("IE.IE() is not checking its parameters properly.");
		
		IE ie = null;
		
		try
		{
			ie = new IE("name", "value");
		}
		
		catch (Throwable t)
		{
			TestCase.fail("IE.IE() has thrown an excpetion when given valid parameters.");
		}
		
		if (ie.GetCaseReferencesCount() > 0)
			TestCase.fail("IE.IE() is not working properly.");
	}

	/**
	 * Tests the copy constructor.
	 * @author Piotrek
	 */
	public void testIEIE()
	{
		IE original = new IE("n", "v");
		
		IE copy = new IE(original);
		
		if (copy.name.compareTo("n") != 0 || copy.value.compareTo("v") != 0)
			TestCase.fail("IE.IE() is not working properly.");
		
		if (copy.GetCaseReferencesCount() > 0)
			TestCase.fail("IE.IE() is not working properly.");
	}

	/**
	 * Tests the equality test.
	 * @author Piotrek
	 */
	public void testEqualsObject()
	{		
		IE ie1 = new IE("1", "v1");
		IE ie2 = new IE("1", "v1");
		IE ie3 = new IE("2", "v1");
		IE ie4 = new IE("1", "v2");
		IE ie5 = new IE("3", "v4");
		
		boolean fThrown = false;
		
		try
		{
			ie1.equals(null);
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("IE.equals() is not checking its parameter properly.");
		
		if (!ie1.equals(ie2))
			TestCase.fail("IE.equals() is not working properly.");
		
		if (ie1.equals(ie3))
			TestCase.fail("IE.equals() is not working properly.");
		
		if (ie1.equals(ie4))
			TestCase.fail("IE.equals() is not working properly.");
		
		if (ie1.equals(ie5))
			TestCase.fail("IE.equals() is not working properly.");
		
		if (ie1.equals(new String("1")))
			TestCase.fail("IE.equals() is not working properly.");
		
		if (ie1.equals(new Integer(1)))
			TestCase.fail("IE.equals() is not working properly.");
		
	}
	
	/**
	 * Tests the equality test.
	 * @author Piotrek
	 */
	public void testEqualsIE()
	{		
		IE ie1 = new IE("1", "v1");
		IE ie2 = new IE("1", "v1");
		IE ie3 = new IE("2", "v1");
		IE ie4 = new IE("1", "v2");
		IE ie5 = new IE("3", "v4");
		
		boolean fThrown = false;
		
		try
		{
			ie1.IsEqual(null);
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("IE.IsEqual() is not checking its parameter properly.");
		
		if (!ie1.IsEqual(ie2))
			TestCase.fail("IE.IsEqual() is not working properly.");
		
		if (ie1.IsEqual(ie3))
			TestCase.fail("IE.IsEqual() is not working properly.");
		
		if (ie1.IsEqual(ie4))
			TestCase.fail("IE.IsEqual() is not working properly.");
		
		if (ie1.IsEqual(ie5))
			TestCase.fail("IE.IsEqual() is not working properly.");
	}

	/**
	 * Tests adding and removing of the references to the cases.
	 * @author Piotrek
	 */
	public void testAddCaseReference()
	{
		Case testCase1 = new Case("c1", true, null);
		Case testCase2 = new Case("c2", true, null);
		
		IE ie = new IE("n", "v");
		
		ie.AddCaseReference(testCase1);
		ie.AddCaseReference(testCase2);
		
		if (ie.GetCaseReferencesCount() != 2)
			TestCase.fail("IE.AddCaseReference() is not working properly.");
		
		ie.RemoveCaseReference(testCase1);
		
		if (ie.GetCaseReferencesCount() != 1)
			TestCase.fail("IE.RemoveCaseReference() is not working properly.");
		
		ie.RemoveCaseReference(testCase2);
		
		if (ie.GetCaseReferencesCount() != 0)
			TestCase.fail("IE.RemoveCaseReference() is not working properly.");
	}

	/**
	 * Tests getting of the cases references count.
	 * @author Piotrek
	 */
	public void testGetCaseReferencesCount()
	{
		Case testCase1 = new Case("c1", true, null);
		Case testCase2 = new Case("c2", true, null);
		
		IE ie = new IE("n", "v");
		
		if (ie.GetCaseReferencesCount() != 0)
			TestCase.fail("IE.GetCaseReferencesCount() is not working properly.");
		
		ie.AddCaseReference(testCase1);
		
		if (ie.GetCaseReferencesCount() != 1)
			TestCase.fail("IE.GetCaseReferencesCount() is not working properly.");
		
		ie.AddCaseReference(testCase2);
		
		if (ie.GetCaseReferencesCount() != 2)
			TestCase.fail("IE.GetCaseReferencesCount() is not working properly.");
	}

	/**
	 * Tests checking if the IE has a reference to the given case.
	 * @author Piotrek
	 */
	public void testContainsCase()
	{
		Case testCase1 = new Case("c1", true, null);
		Case testCase2 = new Case("c2", true, null);
		
		IE ie = new IE("n", "v");
		
		ie.AddCaseReference(testCase1);
		
		if (ie.ContainsCase(testCase2))
			TestCase.fail("IE.ContainsCase() is not working properly.");
	}
}
