/*
 * Created on 2005-05-31
 */
package tests;

import java.util.ArrayList;
import junit.framework.TestCase;
import scheduler.TravelConstraints;
import scheduler.TravelDivisionsConnectionsFinder;
import scheduler.modulesinterfaces.ConnectionsTravelInfoProvider;
import world.Station;

/**
 * Tests the TravelDivisionsConnectionsFinder class.
 * @author Piotrek
 */
public class TravelDivisionsConnectionsFinderTest extends TestCase
{
	/**
	 * Tests the constructor.
	 * @author Piotrek
	 */
	public void testTravelDivisionsConnectionsFinder()
	{
		try
		{
			new TravelDivisionsConnectionsFinder();
		}
		
		catch (Throwable t)
		{
			TestCase.fail("testTravelDivisionsConnectionsFinder constructor has thrown an exception.");
		}
	}

	/**
	 * Tests the setting of the info provider.
	 * @author Piotrek
	 */
	public void testSetTravelInfoProvider()
	{
		TravelDivisionsConnectionsFinder finder = new TravelDivisionsConnectionsFinder();
		
		boolean fThrown = false;
		
		try
		{
			finder.SetTravelInfoProvider(null);
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
		{
			TestCase.fail("TravelDivisionsConnectionsFinder.SetTravelInfoProvider() is not checking its parameters properly.");
		}
		
		try
		{
			finder.SetTravelInfoProvider(new ConnectionsTravelInfoProvider()
								{
									/**
									 * @see scheduler.modulesinterfaces.ConnectionsTravelInfoProvider#GetConnectionsForStation(world.Station, int)
									 */
									public ArrayList GetConnectionsForStation(Station station, int meanOfTransportation)
									{
										return null;
									}
								}
								);
		}
		
		catch (Throwable t)
		{
			TestCase.fail("TravelDivisionsConnectionsFinder.SetTravelInfoProvider() has thrown an exception while adding a valid information provider.");
		}
	}

	/**
	 * Tests parameters checking only of the FindConnections().
	 * @author Piotrek
	 */
	public void testFindConnections()
	{
		TravelDivisionsConnectionsFinder finder = new TravelDivisionsConnectionsFinder();
		
		boolean fThrown = false;
		
		try
		{
			finder.FindConnections(null, new TravelConstraints());
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelDivisionsConnectionsFinder.FindConnections() is not checking its parameters properly.");
		
		fThrown = false;
		
		try
		{
			finder.FindConnections(new ArrayList(), null);
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelDivisionsConnectionsFinder.FindConnections() is not checking its parameters properly.");
		
		fThrown = false;
		
		try
		{
			finder.FindConnections(new ArrayList(), new TravelConstraints());
		}
		
		catch (RuntimeException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelDivisionsConnectionsFinder.FindConnections() has not detected that the info provider is not set.");
	}
	
	/**
	 * Tests the time being reserved for the walk travel part.
	 * @author Piotrek
	 */
	public void testReservedWalkTime()
	{
		if (TravelDivisionsConnectionsFinder.MAX_LENGTH_WALK_RESERVED_TIME < 0)
			TestCase.fail("TravelDivisionsConnectionsFinder.MAX_LENGTH_WALK_RESERVED_TIME must not be negative.");
	}
	
	/**
	 * Tests the time being reserved for the taxi travel part.
	 * @author Piotrek
	 */
	public void testReservedTaxiTime()
	{
		if (TravelDivisionsConnectionsFinder.MAX_LENGTH_TAXI_RESERVED_TIME < 0)
			TestCase.fail("TravelDivisionsConnectionsFinder.MAX_LENGTH_TAXI_RESERVED_TIME must not be negative.");
	}
	
	/**
	 * Tests the max number of starting connections.
	 * @author Piotrek
	 */
	public void testMaxStartingConnectionsCount()
	{
		if (TravelDivisionsConnectionsFinder.MAX_STARTING_CONNECTIONS_COUNT <= 0)
			TestCase.fail("TravelDivisionsConnectionsFinder.MAX_STARTING_CONNECTIONS_COUNT must be positive.");
	}
}
