/*
 * Created on 2005-05-12
 */
package tests;

import junit.framework.TestCase;
import scheduler.TravelPart;
import world.Location;
import world.MeanOfTransportation;
import world.Station;
import world.TrainStation;

/**
 * Tests the TravelPart class.
 * @author Piotrek
 */
public class TravelPartTest extends TestCase
{
	/**
	 * Tests the constructor.
	 * @author Piotrek
	 */
	public void testConstructor()
	{
		try
		{
			for (int meanOfTransportation = MeanOfTransportation.AIRPLANE; meanOfTransportation <= MeanOfTransportation.WALK; ++meanOfTransportation)
			{
				TravelPart travelPart = new TravelPart(meanOfTransportation, new Location(0, 0), new Location(1, 1));
				
				if (travelPart.GetMeanOfTransportation() != meanOfTransportation)
					TestCase.fail("TravelPart constructor has created an instance with a different mean of transportation then specified by the parameter.");
				
				if (!travelPart.GetOrigin().IsEqual(new Location(0, 0)))
					TestCase.fail("TravelPart constructor has created an instance with a different origin location then specified by the parameter.");
				
				if (!travelPart.GetDestination().IsEqual(new Location(1, 1)))
					TestCase.fail("TravelPart constructor has created an instance with a different destination location then specified by the parameter.");
			}
		}
		
		catch (Throwable t)
		{
			TestCase.fail("TravelPart constructor has thrown an exception given a valid data: " + t.getMessage());
		}
		
		boolean fHasThrown = false;
		
		try
		{
			new TravelPart(9999, new Location(0, 0), new Location(1, 1));
		}
		
		catch (IllegalArgumentException xcp)
		{
			fHasThrown = true;
		}
		
		if (!fHasThrown)
			TestCase.fail("TravelPart constructor should have thrown an exception given an invalid mean of transportation.");
		
		fHasThrown = false;
		
		try
		{
			new TravelPart(MeanOfTransportation.AIRPLANE, null, new Location(1, 1));
		}
		
		catch (NullPointerException xcp)
		{
			fHasThrown = true;
		}
		
		if (!fHasThrown)
			TestCase.fail("TravelPart constructor should have thrown an exception given a null origin location.");
		
		fHasThrown = false;
		
		try
		{
			new TravelPart(MeanOfTransportation.AIRPLANE, new Location(1, 1), null);
		}
		
		catch (NullPointerException xcp)
		{
			fHasThrown = true;
		}
		
		if (!fHasThrown)
			TestCase.fail("TravelPart constructor should have thrown an exception given a null destination location.");
	}
	
	/**
	 * Tests the GetMeanOfTransportation() method.
	 * @author Piotrek
	 */
	public void testGetMeanOfTransportation()
	{
		TravelPart travelPart = new TravelPart(MeanOfTransportation.AIRPLANE, new Location(0, 0), new Location(1, 1));
		
		if (travelPart.GetMeanOfTransportation() != MeanOfTransportation.AIRPLANE)
			TestCase.fail("TravelPart.GetMeanOfTransportation() has returned an unexpected value.");
	}

	/**
	 * Tests the GetOrigin method.
	 * @author Piotrek
	 */
	public void testGetOrigin()
	{
		TravelPart travelPart = new TravelPart(MeanOfTransportation.AIRPLANE, new Location(0, 0), new Location(1, 1));
		
		if (!travelPart.GetOrigin().IsEqual(new Location(0, 0)))
			TestCase.fail("TravelPart.GetOrigin() has returned an unexpected value.");
	}

	/**
	 * Tests the GetDestination method.
	 * @author Piotrek
	 */
	public void testGetDestination()
	{
		TravelPart travelPart = new TravelPart(MeanOfTransportation.AIRPLANE, new Location(0, 0), new Location(1, 1));
		
		if (!travelPart.GetDestination().IsEqual(new Location(1, 1)))
			TestCase.fail("TravelPart.GetDestination() has returned an unexpected value.");
	}
	
	/**
	 * Tests the getting of the reverted travel part.
	 * @author Piotrek
	 */
	public void testGetRevertedPart()
	{
		Location locationA = new Location(1, 1);
		Location locationB = new Location(2, 2);
		
		Station stationA = new TrainStation(locationA, "A");
		Station stationB = new TrainStation(locationB, "B");
		
		TravelPart travelPart = new TravelPart(MeanOfTransportation.TRAIN, locationA, locationB);
		
		travelPart.SetOriginStation(stationA);
		travelPart.SetDestinationStation(stationB);
		
		travelPart = travelPart.GetRevertedPart();
		
		if (travelPart.GetMeanOfTransportation() != MeanOfTransportation.TRAIN)
			TestCase.fail("The travel parts reverting does not work properly.");
		
		if (!travelPart.GetOrigin().IsEqual(locationB))
			TestCase.fail("The travel parts reverting does not work properly.");
		
		if (!travelPart.GetDestination().IsEqual(locationA))
			TestCase.fail("The travel parts reverting does not work properly.");
		
		if (!travelPart.GetOriginStation().IsEqual(stationB))
			TestCase.fail("The travel parts reverting does not work properly.");
		
		if (!travelPart.GetDestinationStation().IsEqual(stationA))
			TestCase.fail("The travel parts reverting does not work properly.");
	}
}
