/*
 * Created on 2005-05-12
 */
package tests;

import java.util.ArrayList;
import junit.framework.TestCase;
import scheduler.TravelConstraints;
import scheduler.TravelPartsGenerator;
import scheduler.modules.FeedbackTravelPartsProviderStub;
import scheduler.modules.MeansOfTransportationChooserStub;
import world.Location;
import world.WorldTravelInfoProvider;

/**
 * Tests the TravelPartsGenerator class.
 * @author Piotrek
 */
public class TravelPartsGeneratorTest extends TestCase
{
	/**
	 * Tests the AreCloseEnough() method.
	 * @author Piotrek
	 */
	public void testAreCloseEnough()
	{
		Location location1 = new Location(0, 0);
		Location location2 = new Location((int)TravelPartsGenerator.MAX_TRAVEL_PARTS_SPACE, 0);
		Location location3 = new Location(0, (int)TravelPartsGenerator.MAX_TRAVEL_PARTS_SPACE);
		Location location4 = new Location((int)(0.5 * TravelPartsGenerator.MAX_TRAVEL_PARTS_SPACE), 0);
		Location location5 = new Location((int)TravelPartsGenerator.MAX_TRAVEL_PARTS_SPACE + 1, 0);
		
		if (!TravelPartsGenerator.AreCloseEnough(location1, location2))
			TestCase.fail("TravelPartsGenerator.AreCloseEnough() does not work properly.");
		
		if (!TravelPartsGenerator.AreCloseEnough(location1, location3))
			TestCase.fail("TravelPartsGenerator.AreCloseEnough() does not work properly.");
		
		if (!TravelPartsGenerator.AreCloseEnough(location1, location4))
			TestCase.fail("TravelPartsGenerator.AreCloseEnough() does not work properly.");
		
		if (TravelPartsGenerator.AreCloseEnough(location1, location5))
			TestCase.fail("TravelPartsGenerator.AreCloseEnough() does not work properly.");
	}
	
	/**
	 * Tests the MAX_DEPTH value.
	 * @author Piotrek
	 */
	public void testMaxDepthValue()
	{
		if (TravelPartsGenerator.MAX_DEPTH < 2)
			TestCase.fail("The MAX_DEPTH must be at least 2.");
	}
	
	/**
	 * Tests the getting of the max parts count.
	 * @author Piotrek
	 */
	public void testGetMaxPartsCount()
	{
		if (TravelPartsGenerator.GetMaxPartsCount() < 1)
			TestCase.fail("The returned max parts count is definitely wrong.");
	}
		
	/**
	 * Tests the values of search ranges for the all types of stations.
	 * @author Piotrek
	 */
	public void testSearchStartValues()
	{
		if (TravelPartsGenerator.HARBOURS_START_SEARCH_RANGE <= 0)
			TestCase.fail("TravelPartsGenerator.HARBOURS_START_SEARCH_RANGE must be positive.");
		
		if (TravelPartsGenerator.AIRPORTS_START_SEARCH_RANGE <= 0)
			TestCase.fail("TravelPartsGenerator.AIRPORTS_START_SEARCH_RANGE must be positive.");
		
		if (TravelPartsGenerator.BUS_STOPS_START_SEARCH_RANGE <= 0)
			TestCase.fail("TravelPartsGenerator.BUS_STOPS_START_SEARCH_RANGE must be positive.");
		
		if (TravelPartsGenerator.HARBOURS_START_SEARCH_RANGE <= 0)
			TestCase.fail("TravelPartsGenerator.HARBOURS_START_SEARCH_RANGE must be positive.");
	}
		
	/**
	 * Tests the value of the max space between the travel parts.
	 * @author Piotrek
	 */
	public void testMaxTravelPartsSpace()
	{
		if (TravelPartsGenerator.MAX_TRAVEL_PARTS_SPACE < 0)
			TestCase.fail("TravelPartsGenerator.MAX_TRAVEL_PARTS_SPACE must not be negative.");
	}
	
	/**
	 * Tests the maximal numbers of stations in range of each type.
	 * @author Piotrek
	 */
	public void testMaxNumbersOfStationsInRange()
	{
		if (TravelPartsGenerator.MAX_AIRPORTS_IN_RANGE_NUMBER < 1)
			TestCase.fail("TravelPartsGenerator.MAX_AIRPORTS_IN_RANGE_NUMBER must be at least 1.");
		
		if (TravelPartsGenerator.MAX_BUS_STOPS_IN_RANGE_NUMBER < 1)
			TestCase.fail("TravelPartsGenerator.MAX_BUS_STOPS_IN_RANGE_NUMBER must be at least 1.");
		
		if (TravelPartsGenerator.MAX_TRAIN_STATIONS_IN_RANGE_NUMBER < 1)
			TestCase.fail("TravelPartsGenerator.MAX_TRAIN_STATIONS_IN_RANGE_NUMBER must be at least 1.");
		
		if (TravelPartsGenerator.MAX_HARBOURS_IN_RANGE_NUMBER < 1)
			TestCase.fail("TravelPartsGenerator.MAX_HARBOURS_IN_RANGE_NUMBER must be at least 1.");
	}
	
	/**
	 * Tests if the GenerateTravelParts() method is checking for null parameters and does return an empty list if there is no division.
	 * @author Piotrek
	 */
	public void testGenerateTravelParts()
	{
		boolean fHasThrown = false;
		
		FeedbackTravelPartsProviderStub feedbackProvider = new FeedbackTravelPartsProviderStub();
		
		feedbackProvider.SetTravelConstraints(new TravelConstraints());
		
		try
		{
			TravelPartsGenerator generator = new TravelPartsGenerator();
			
			generator.SetMeansOfTransportationProvider(new MeansOfTransportationChooserStub());
			generator.SetFeedbackTravelPartsProvider(feedbackProvider);
			
			generator.GenerateTravelParts(null, new Location(0, 0), new TravelConstraints());
		}
		
		catch (NullPointerException xcp)
		{
			fHasThrown = true;
		}
		
		if (!fHasThrown)
			TestCase.fail("TravelPartsGenerator.GenerateTravelParts() should have thrown an exception.");
		
		fHasThrown = false;
		
		try
		{
			TravelPartsGenerator generator = new TravelPartsGenerator();
			
			generator.SetMeansOfTransportationProvider(new MeansOfTransportationChooserStub());
			generator.SetFeedbackTravelPartsProvider(feedbackProvider);
			
			generator.GenerateTravelParts(new Location(0, 0), null, new TravelConstraints());
		}
		
		catch (NullPointerException xcp)
		{
			fHasThrown = true;
		}
		
		if (!fHasThrown)
			TestCase.fail("TravelPartsGenerator.GenerateTravelParts() should have thrown an exception.");
		
		fHasThrown = false;
		
		try
		{
			TravelPartsGenerator generator = new TravelPartsGenerator();
			
			generator.SetMeansOfTransportationProvider(new MeansOfTransportationChooserStub());
			generator.SetFeedbackTravelPartsProvider(feedbackProvider);
			
			generator.GenerateTravelParts(new Location(0, 0), new Location(30, 20), null);
		}
		
		catch (NullPointerException xcp)
		{
			fHasThrown = true;
		}
		
		if (!fHasThrown)
			TestCase.fail("TravelPartsGenerator.GenerateTravelParts() should have thrown an exception.");
		
		fHasThrown = false;
		
		try
		{
			TravelPartsGenerator generator = new TravelPartsGenerator();
			
			generator.SetMeansOfTransportationProvider(new MeansOfTransportationChooserStub());
			generator.SetFeedbackTravelPartsProvider(feedbackProvider);
			
			generator.GenerateTravelParts(null, null, null);
		}
		
		catch (NullPointerException xcp)
		{
			fHasThrown = true;
		}
		
		if (!fHasThrown)
			TestCase.fail("TravelPartsGenerator.GenerateTravelParts() should have thrown an exception.");
		
		// checking the return value
		TravelPartsGenerator generator = new TravelPartsGenerator();
		
		generator.SetTravelInfoProvider(new WorldTravelInfoProvider());
		
		generator.SetMeansOfTransportationProvider(new MeansOfTransportationChooserStub());
		generator.SetFeedbackTravelPartsProvider(feedbackProvider);
		
		ArrayList resultList = generator.GenerateTravelParts(new Location(0, 0), new Location(10, 10), new TravelConstraints());
		
		if (resultList == null)
			TestCase.fail("TravelPartsGenerator.GenerateTravelParts() must not return a null value.");
		
		if (resultList.size() != 0)
			TestCase.fail("TravelPartsGenerator.GenerateTravelParts() should have returned an empty list.");
	}
}
