/*
 * Created on 2005-05-14
 */
package tests;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import junit.framework.TestCase;
import world.Connection;
import world.Location;
import world.MeanOfTransportation;
import world.TravelTime;

/**
 * Tests the TravelTime class.
 * @author Piotrek
 */
public class TravelTimeTest extends TestCase
{
	/**
	 * Tests the counting of the travel time between locations. 
	 * @author Piotrek
	 */
	public void testGetTravelTime()
	{
		boolean fThrown = false;
		
		try
		{
			TravelTime.GetTravelTime(null, null, -1);
		}
		
		catch (Throwable t)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("The TravelTime.GetTravelTime() method is not validating parameters thoroughfully.");
		
		fThrown = false;
		
		try
		{
			TravelTime.GetTravelTime(null, new Location(2, 4), MeanOfTransportation.AIRPLANE);
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("The TravelTime.GetTravelTime() method is not validating parameters thoroughfully.");
		
		fThrown = false;
		
		try
		{
			TravelTime.GetTravelTime(new Location(2, 4), null, MeanOfTransportation.AIRPLANE);
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("The TravelTime.GetTravelTime() method is not validating parameters thoroughfully.");
		
		fThrown = false;
		
		try
		{
			TravelTime.GetTravelTime(new Location(3, 5), new Location(2, 4), -1);
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("The TravelTime.GetTravelTime() method is not validating parameters thoroughfully.");
		
		for (int meanOftransportation = MeanOfTransportation.AIRPLANE; meanOftransportation <= MeanOfTransportation.WALK; ++meanOftransportation)
		{
			if (TravelTime.GetTravelTime(new Location(0, 0), new Location(10, 10), meanOftransportation) <= 0)
				TestCase.fail("TravelTime.GetTravelTime() returns invalid travel times for the mean of transportation code " + meanOftransportation + ".");
			
			if (TravelTime.GetTravelTime(new Location(0, 0), new Location(0, 0), meanOftransportation) != 0)
				TestCase.fail("TravelTime.GetTravelTime() returns invalid travel times for the mean of transportation code " + meanOftransportation + ".");
		}
	}

	/**
	 * Tests the getting of the change times.
	 * @author Piotrek
	 */
	public void testGetChangeTime()
	{
		for (int meanOftransportation = MeanOfTransportation.AIRPLANE; meanOftransportation <= MeanOfTransportation.WALK; ++meanOftransportation)
		{
			if (TravelTime.GetChangeTime(meanOftransportation) < 0)
				TestCase.fail("Change time is negative for the " + MeanOfTransportation.GetMeanOfTransportationString(meanOftransportation));
		}
	}

	/**
	 * Tests the getting of the shift time in a string format.
	 * @author Piotrek
	 */
	public void testGetShiftTime()
	{
		boolean fThrown = false;
		
		try
		{
			TravelTime.GetShiftTime(-1);
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelTime.GetShiftTime() does not check its parameters properly.");
		
		String shiftTime = TravelTime.GetShiftTime(10);
		
		if (shiftTime.length() != 6 || !shiftTime.startsWith("+") || shiftTime.substring(3, 4).compareTo(":") != 0)
		{
			TestCase.fail("TravelTime.GetShiftTime() has returned an ill-formated time.");
		}
	}

	/**
	 * Tests the getting of the time in a string format.
	 * @author Piotrek
	 *
	 */
	public void testGetTime()
	{
		boolean fThrown = false;
		
		try
		{
			TravelTime.GetTime(-1);
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelTime.GetTime() does not check its parameters properly.");
		
		String shiftTime = TravelTime.GetTime(10);
		
		if (shiftTime.length() != 5 || shiftTime.substring(2, 3).compareTo(":") != 0)
			TestCase.fail("TravelTime.GetTime() has returned an ill-formated time.");
	}

	/**
	 * Test the parsing of the duration time.
	 * @author Piotrek
	 */
	public void testParseDuration()
	{
		boolean fThrown = false;
		
		try
		{
			TravelTime.ParseDuration(null);
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		catch (ParseException xcp)
		{
			// added only to satisfy the Java exceptions handling mechanism
		}
		
		if (!fThrown)
			TestCase.fail("TravelTime.ParseDuration() does not check its parameters properly.");
		
		fThrown = false;
		
		try
		{
			TravelTime.ParseDuration("1:2:2");
		}
		
		catch (ParseException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelTime.ParseDuration() does not check its parameters properly.");
		
		fThrown = false;
		
		try
		{
			TravelTime.ParseDuration("-1:22");
		}
		
		catch (ParseException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelTime.ParseDuration() does not check its parameters properly.");
		
		fThrown = false;
		
		try
		{
			TravelTime.ParseDuration("+2:-2");
		}
		
		catch (ParseException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelTime.ParseDuration() does not check its parameters properly.");
		
		fThrown = false;
		
		try
		{
			TravelTime.ParseDuration("-22:-22");
		}
		
		catch (ParseException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelTime.ParseDuration() does not check its parameters properly.");
		
		fThrown = false;
		
		try
		{
			TravelTime.ParseDuration("+x:y");
		}
		
		catch (ParseException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelTime.ParseDuration() does not check its parameters properly.");
		
		try
		{
			if (TravelTime.ParseDuration("+1:20") != 80)
				TestCase.fail("TravelTime.ParseDuration() has returned an incorrect value.");
		}
		
		catch (Throwable t)
		{
			TestCase.fail("TravelTime.ParseDuration() has thrown an exception while parsing the well-formatted string.");
		}
	}

	/**
	 * Tests the parsing of time from a string.
	 * @author Piotrek
	 */
	public void testParseTime()
	{
		boolean fThrown = false;
		
		try
		{
			TravelTime.ParseTime(null);
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		catch (ParseException xcp)
		{
			// added only to satisfy the Java exceptions handling mechanism
		}
		
		if (!fThrown)
			TestCase.fail("TravelTime.ParseTime() does not check its parameters properly.");
		
		fThrown = false;
		
		try
		{
			TravelTime.ParseTime("x:y");
		}
		
		catch (ParseException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelTime.ParseTime() does not check its parameters properly.");
		
		fThrown = false;
		
		try
		{
			TravelTime.ParseTime("1:2:2");
		}
		
		catch (ParseException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelTime.ParseTime() does not check its parameters properly.");
		
		fThrown = false;
		
		try
		{
			TravelTime.ParseTime("-1:2");
		}
		
		catch (ParseException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelTime.ParseTime() does not check its parameters properly.");
		
		fThrown = false;
		
		try
		{
			TravelTime.ParseTime("1:-2");
		}
		
		catch (ParseException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelTime.ParseTime() does not check its parameters properly.");
		
		fThrown = false;
		
		try
		{
			TravelTime.ParseTime("-1:-22");
		}
		
		catch (ParseException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelTime.ParseTime() does not check its parameters properly.");
		
		fThrown = false;
		
		try
		{
			TravelTime.ParseTime("+1:22");
		}
		
		catch (ParseException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelTime.ParseTime() does not check its parameters properly.");
		
		try
		{
			if (TravelTime.ParseTime("1:20") != 80)
				TestCase.fail("TravelTime.ParseTime() has returned an incorrect value.");
		}
		
		catch (Throwable t)
		{
			TestCase.fail("TravelTime.ParseTime() has thrown an exception while parsing the well-formatted string.");
		}
	}

	/**
	 * Tests the getting the times per day generation.
	 * @author Piotrek
	 */
	public void testGetTimesPerDay()
	{
		boolean fThrown = false;
		
		try
		{
			TravelTime.GetTimesPerDay(-1, 1, 1, 2);
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelTime.GetTimesPerDay() does not check its parameters properly.");
		
		fThrown = false;
		
		try
		{
			TravelTime.GetTimesPerDay(1, 0, 1, 2);
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelTime.GetTimesPerDay() does not check its parameters properly.");
		
		fThrown = false;
		
		try
		{
			TravelTime.GetTimesPerDay(1, 1, -1, 2);
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelTime.GetTimesPerDay() does not check its parameters properly.");
		
		fThrown = false;
		
		try
		{
			TravelTime.GetTimesPerDay(1, 1, 1, -2);
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelTime.GetTimesPerDay() does not check its parameters properly.");
		
		fThrown = false;
		
		try
		{
			TravelTime.GetTimesPerDay(14444, 1, 1, 2);
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelTime.GetTimesPerDay() does not check its parameters properly.");
		
		fThrown = false;
		
		try
		{
			TravelTime.GetTimesPerDay(1, 16666, 1, 2);
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelTime.GetTimesPerDay() does not check its parameters properly.");
		
		fThrown = false;
		
		try
		{
			TravelTime.GetTimesPerDay(1, 1, 18888, 2);
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelTime.GetTimesPerDay() does not check its parameters properly.");
		
		fThrown = false;
		
		try
		{
			TravelTime.GetTimesPerDay(1, 1, 1, 27777);
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelTime.GetTimesPerDay() does not check its parameters properly.");
		
		fThrown = false;
		
		try
		{
			TravelTime.GetTimesPerDay(1, 1, 7, 2);
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelTime.GetTimesPerDay() does not check its parameters properly.");
		
		ArrayList result = TravelTime.GetTimesPerDay(360, 6, 350, 370);
		
		if (result.size() != 3)
			TestCase.fail("TravelTime.GetTimesPerDay() does not work well.");
		
		if (((Integer)result.get(0)).intValue() != 354)
			TestCase.fail("TravelTime.GetTimesPerDay() does not work well.");
		
		if (((Integer)result.get(1)).intValue() != 360)
			TestCase.fail("TravelTime.GetTimesPerDay() does not work well.");
		
		if (((Integer)result.get(2)).intValue() != 366)
			TestCase.fail("TravelTime.GetTimesPerDay() does not work well.");
	}
	
	/**
	 * Test getting the time since Sunday in minutes for the given day of week and time on that day (also in minutes).
	 * @author Piotrek
	 */
	public void testGetTimeSinceSunday()
	{
		boolean fThrown = false;
		
		try
		{
			TravelTime.GetTimeSinceSunday(Connection.MONDAY, -1);
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("The TravelTime.GetTimeSinceSunday() has not spotted the negative day time.");
		
		fThrown = false;
		
		try
		{
			TravelTime.GetTimeSinceSunday(Connection.MONDAY, 24 * 60);
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("The TravelTime.GetTimeSinceSunday() has not spotted the too big day time.");
		
		fThrown = false;
		
		try
		{
			TravelTime.GetTimeSinceSunday((char)123, 1);
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("The TravelTime.GetTimeSinceSunday() has not spotted the invalid day of week.");
		
		fThrown = false;
		
		try
		{
			if (TravelTime.GetTimeSinceSunday(Connection.SUNDAY, 12) != 12)
				TestCase.fail("The TravelTime.GetTimeSinceSunday() is not working properly.");
		}
		
		catch (Throwable t)
		{
			TestCase.fail("The TravelTime.GetTimeSinceSunday() has thrown an exception for valid parameters.");
		}
		
		try
		{
			if (TravelTime.GetTimeSinceSunday(Connection.MONDAY, 12) != 12 + 24 * 60)
				TestCase.fail("The TravelTime.GetTimeSinceSunday() is not working properly.");
		}
		
		catch (Throwable t)
		{
			TestCase.fail("The TravelTime.GetTimeSinceSunday() has thrown an exception for valid parameters.");
		}
		
		try
		{
			if (TravelTime.GetTimeSinceSunday(Connection.TUESDAY, 12) != 12 + 2 * 24 * 60)
				TestCase.fail("The TravelTime.GetTimeSinceSunday() is not working properly.");
		}
		
		catch (Throwable t)
		{
			TestCase.fail("The TravelTime.GetTimeSinceSunday() has thrown an exception for valid parameters.");
		}
		
		try
		{
			if (TravelTime.GetTimeSinceSunday(Connection.WEDNESDAY, 12) != 12 + 3 * 24 * 60)
				TestCase.fail("The TravelTime.GetTimeSinceSunday() is not working properly.");
		}
		
		catch (Throwable t)
		{
			TestCase.fail("The TravelTime.GetTimeSinceSunday() has thrown an exception for valid parameters.");
		}
		
		try
		{
			if (TravelTime.GetTimeSinceSunday(Connection.THURSDAY, 12) != 12 + 4 * 24 * 60)
				TestCase.fail("The TravelTime.GetTimeSinceSunday() is not working properly.");
		}
		
		catch (Throwable t)
		{
			TestCase.fail("The TravelTime.GetTimeSinceSunday() has thrown an exception for valid parameters.");
		}
		
		try
		{
			if (TravelTime.GetTimeSinceSunday(Connection.FRIDAY, 12) != 12 + 5 * 24 * 60)
				TestCase.fail("The TravelTime.GetTimeSinceSunday() is not working properly.");
		}
		
		catch (Throwable t)
		{
			TestCase.fail("The TravelTime.GetTimeSinceSunday() has thrown an exception for valid parameters.");
		}
				
		try
		{
			if (TravelTime.GetTimeSinceSunday(Connection.SATURDAY, 12) != 12 + 6 * 24 * 60)
				TestCase.fail("The TravelTime.GetTimeSinceSunday() is not working properly.");
		}
		
		catch (Throwable t)
		{
			TestCase.fail("The TravelTime.GetTimeSinceSunday() has thrown an exception for valid parameters.");
		}
	}
	
	/**
	 * Tests shifting of dates by the given number of minutes.
	 * @author Piotrek
	 */
	public void testShiftDate()
	{
		boolean fThrown = false;
		
		try
		{
			TravelTime.ShiftDate(null, 10000);
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelTime.ShiftDate() has not spotted the null date.");
				
		Date date = new Date(0); // January 1, 1970, 00:00:00 GMT
		
		TravelTime.ShiftDate(date, 1);
		
		if (!date.equals(new Date(60000)))
			TestCase.fail("TravelTime.ShiftDate() is not working properly.");
		
		date = new Date(0); // January 1, 1970, 00:00:00 GMT
		
		TravelTime.ShiftDate(date, 100);
		
		if (!date.equals(new Date(6000000)))
			TestCase.fail("TravelTime.ShiftDate() is not working properly.");
		
		date = new Date(0); // January 1, 1970, 00:00:00 GMT
		
		TravelTime.ShiftDate(date, 10000);
		
		if (!date.equals(new Date(600000000)))
			TestCase.fail("TravelTime.ShiftDate() is not working properly.");
		
		date = new Date(0); // January 1, 1970, 00:00:00 GMT
		
		TravelTime.ShiftDate(date, 0);
		
		if (!date.equals(new Date(0)))
			TestCase.fail("TravelTime.ShiftDate() is not working properly.");
		
		date = new Date(120000); // January 1, 1970, 00:02:00 GMT
		
		TravelTime.ShiftDate(date, -2);
		
		if (!date.equals(new Date(0)))
			TestCase.fail("TravelTime.ShiftDate() is not working properly.");
	}
	
	/**
	 * Tests counting of a time difference of the two dates in minutes.
	 * @author Piotrek
	 */
	public void testGetDatesDifference()
	{
		Date date1 = new Date(6000000); // 100 min.
		Date date2 = new Date(12000000); // 200 min.
		
		boolean fThrown = false;
		
		try
		{
			TravelTime.GetDatesDifference(null, date2);
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelTime.GetDatesDifference() is not checking its parameters properly.");
		
		fThrown = false;
		
		try
		{
			TravelTime.GetDatesDifference(date1, null);
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelTime.GetDatesDifference() is not checking its parameters properly.");
		
		fThrown = false;
		
		try
		{
			TravelTime.GetDatesDifference(date2, date1);
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelTime.GetDatesDifference() is not checking its parameters properly.");
		
		if (TravelTime.GetDatesDifference(date1, date2) != 100)
			TestCase.fail("TravelTime.GetDatesDifference() is not working properly.");
		
		Date date3 = new Date(12030000); // 200,5 min.
		
		if (TravelTime.GetDatesDifference(date1, date3) != 101)
			TestCase.fail("TravelTime.GetDatesDifference() is not working properly.");
	}
}
