/*
 * Created on 2005-05-02
 */
package world;

import java.util.ArrayList;
import java.util.Iterator;
import scheduler.TestOptions;
import scheduler.modulesinterfaces.ConnectionsTravelInfoProvider;
import scheduler.modulesinterfaces.LocationsTravelInfoProvider;

/**
 * A provider of travel information on world.
 * @author Piotrek
 */
public class WorldTravelInfoProvider implements LocationsTravelInfoProvider, ConnectionsTravelInfoProvider
{
	/**
	 * Constructor.
	 * @author Piotrek
	 */
	public WorldTravelInfoProvider()
	{
		// does nothing
	}
	
	/**
	 * @author Piotrek
	 * @see scheduler.modulesinterfaces.LocationsTravelInfoProvider#GetAirportsInRange(int, int, int, int)
	 */
	public ArrayList GetAirportsInRange(int minX, int minY, int maxX, int maxY)
	{
		ArrayList results = new ArrayList();
		
		World world = World.GetInstance();
	
		synchronized (world)
		{
			Iterator airports = world.GetAirports();
			
			while (airports.hasNext())
			{
				Airport airport = (Airport)airports.next();
				
				if (airport.GetLocation().IsInRange(minX, minY, maxX, maxY))
					results.add(airport);
			}
		}
		
		TestOptions.IncreaseStationsInRangeQueriesCount();
		
		return results;
	}
	
	/**
	 * @author Piotrek
	 * @see scheduler.modulesinterfaces.LocationsTravelInfoProvider#GetBusStopsInRange(int, int, int, int)
	 */
	public ArrayList GetBusStopsInRange(int minX, int minY, int maxX, int maxY)
	{
		ArrayList results = new ArrayList();
		
		World world = World.GetInstance();
	
		synchronized (world)
		{
			Iterator busStops = world.GetBusStops();
			
			while (busStops.hasNext())
			{
				BusStop busStop = (BusStop)busStops.next();
				
				if (busStop.GetLocation().IsInRange(minX, minY, maxX, maxY))
					results.add(busStop);
			}
		}
		
		TestOptions.IncreaseStationsInRangeQueriesCount();
		
		return results;
	}
	
	/**
	 * @author Piotrek
	 * @see scheduler.modulesinterfaces.LocationsTravelInfoProvider#GetHarboursInRange(int, int, int, int)
	 */
	public ArrayList GetHarboursInRange(int minX, int minY, int maxX, int maxY)
	{
		ArrayList results = new ArrayList();
		
		World world = World.GetInstance();
	
		synchronized (world)
		{
			Iterator harbours = world.GetHarbours();
			
			while (harbours.hasNext())
			{
				Harbour harbour = (Harbour)harbours.next();
				
				if (harbour.GetLocation().IsInRange(minX, minY, maxX, maxY))
					results.add(harbour);
			}
		}
		
		TestOptions.IncreaseStationsInRangeQueriesCount();
		
		return results;
	}
	
	/**
	 * @author Piotrek
	 * @see scheduler.modulesinterfaces.LocationsTravelInfoProvider#GetTrainStationsInRange(int, int, int, int)
	 */
	public ArrayList GetTrainStationsInRange(int minX, int minY, int maxX, int maxY)
	{
		ArrayList results = new ArrayList();
		
		World world = World.GetInstance();
	
		synchronized (world)
		{
			Iterator trainStations = world.GetTrainStations();
			
			while (trainStations.hasNext())
			{
				TrainStation trainStation = (TrainStation)trainStations.next();
				
				if (trainStation.GetLocation().IsInRange(minX, minY, maxX, maxY))
					results.add(trainStation);
			}
		}
		
		TestOptions.IncreaseStationsInRangeQueriesCount();
		
		return results;
	}
			
	/**
	 * @see ConnectionsTravelInfoProvider#GetConnectionsForStation(Station, int)
	 * @author Piotrek
	 */
	public ArrayList GetConnectionsForStation(Station station, int meanOfTransportation)
	{
		if (station == null)
			throw new NullPointerException("Station must not be null in the WorldTravelInfoProvider.GetConnectionsForStation().");
		
		if (!MeanOfTransportation.IsValidMeanOfTransportation(meanOfTransportation))
			throw new IllegalArgumentException("Unexpected mean of transportation in the WorldTravelInfoProvider.GetConnectionsForStation().");
		
		if (station.GetMeanOfTransportation() != meanOfTransportation)
			throw new IllegalArgumentException("Station supports some other mean of transportation then specified in the WorldTravelInfoProvider.GetConnectionsForStation().");
		
		ArrayList connections = new ArrayList();
				
		World world = World.GetInstance();
		
		synchronized (world)
		{		
			Iterator iterator = world.GetConnections(); 
			
			while (iterator.hasNext())
			{
				Connection connection = (Connection)iterator.next();
				
				if (connection.GetMeanOfTransportation() != station.GetMeanOfTransportation())
					continue;
				
				int index = connection.GetStationIndex(station);
				
				if (index >= 0)
					connections.add(connection);
			}
		}
		
		TestOptions.IncreaseConnectionsForStationsQueriesCount();
		
		return connections;
	}
}